#include <arch/pxa255.h>
#include <migos_stdio.h>
#include <dev/lcd/lcd.h>
#include <string.h>
#include <gui/font.h>
//#include <irq.h>


int lcd_init( void );
int init_lcd( void )
{
	// 640x480x16bit colors 

	// GPIO Alternavice Function Register 
	// for LCD
	// 16bit active 640x480 TFT
	// GPIO 58 - 77 

	// GPDR ( GPIO Pin Directtion Register ) = Out = bit 1 
	// GPDR1 GPIO<63:32> OUT=1 IN=0
	// GPDR2 GPIO<80:64>

	// GAFR ( GPIO Alternative Function Register ) = 2bits 10
	// GAFR1_U GPIO<63:48>
	// GAFR2_L GPIO<79:64>

	// PinName = GPIO #Number = Value:     GAFR's 2bit / GPDR
	// ------------------------------------------------------
	// L_DD[0..15] = GPIO [58..73] = Pixel Data pin 10 / OUT
	// LCD_FCLK = GPIO 74 = LCD Frame Clock         10 / OUT 
	// LCD_LCLK = GPIO 75 = LCD Line Clock          10 / OUT
	// LCD_PCLK = GPIO 76 = LCD Pixel Clock         10 / OUT
	// LCD_ACBI = GPIO 77 = LCD AC Bias             10 / OUT
/*	
	GPDR1 |= 0xFD000000;
	GPDR2 |= 0x00003FFF;

	GAFR1_U &= 0xAAAFFFFF;		 	
	GAFR2_L &= 0xFAAAAAAA;
	*/
	
	
	//printf("GPDR1 = %08X GAFR1_U = %08X GAFR2_L = %08X \r\n", GPDR1, GAFR1_U, GAFR2_L  );

	
	// bits 58-77
        GPDR1 |= (0x3f << 26);
        GPDR2 |= 0x00003fff;

        GAFR1_U = (GAFR1_U & ~(0xfff << 20)) | (0xaaa << 20);
        GAFR2_L = (GAFR2_L & 0xf0000000) | 0x0aaaaaaa;

	//printf("GPDR1 = %08X GAFR1_U = %08X GAFR2_L = %08X \r\n", GPDR1, GAFR1_U, GAFR2_L  );
	
	// Program LCD Control Register
	// Frame Descriptor , DMA Controller  
	// LCCR1, LCCR2, LCCR3,  
	LCCR3 = LCD_LCCR3;

	
	LCCR2 = LCD_YRES-1;
	LCCR2 |= LCD_VERTICAL_SYNC_PULSE_WIDTH << 10;
	LCCR2 |= LCD_END_OF_FRAME_WAIT_COUNT << 16;
	LCCR2 |= LCD_BEGIN_FRAME_WAIT_COUNT << 24;
	
	
	LCCR1 = LCD_XRES-1;
	LCCR1 |= LCD_HORIZONTAL_SYNC_PULSE_WIDTH << 10;
	LCCR1 |= LCD_END_OF_LINE_WAIT_COUNT << 16;
	LCCR1 |= LCD_BEGIN_OF_LINE_WAIT_COUNT << 24;
	
	//LCCR1 = 0x0909067f;
	//LCCR1 = 0x3030A7F;
	
	LCCR0 = LCD_LCCR0  & ~LCCR0_ENB;

	// DMA memory set and FDADRx 	
	//FDADR0 = fbi->fdadr0;	
	//FDADR1 = fbi->fdadr1;	
	{
	struct pxafb_dma_desc* pfd;

	pfd = (struct pxafb_dma_desc*)(LCD_BASE_ADDR-4*16);

#define BYTES_PER_PANEL ((LCD_LCCR0 & LCCR0_SDS) ? (LCD_XRES * LCD_YRES * LCD_BPP / 8 / 2) : \
		                                                            (LCD_XRES * LCD_YRES * LCD_BPP / 8))

	pfd->fdadr = (int32u)pfd;		// next descriptor addr ( itself )
	pfd->fsadr = (int32u)LCD_BASE_ADDR;		// DMA source address
	pfd->fidr = 0;			// lcd interrupt id
	/*
	pfd->ldcmd = 0;
	pfd->ldcmd |= 0 << 26;	// PAL (0:no palette, 16col)
	pfd->ldcmd |= 0 << 22;	// SOFINT ( no interrupt on new frame )
	pfd->ldcmd |= 0 << 21;	// EOFINT ( no interrupt on end frame )
	pfd->ldcmd |= LCD_WIDTH*LCD_HEIGHT*LCD_BPP/8; // LEN (transfer len)
	*/
	pfd->ldcmd = BYTES_PER_PANEL;

	FDADR0 = (int32u)pfd;
	

	// FDADR1 is used for dual lcd panel
	FDADR1 = 0;//0xa1e00fb0;

	//printf("FDADR0 = 0x%08X FDADR1 = 0x%08X\r\n", FDADR0, FDADR1 );

	//printf("pfd->fdadr = 0x%08X fsadr = 0x%08X ldcmd = 0x%08X\r\n", 
	//			pfd->fdadr, pfd->fsadr, pfd->ldcmd );
	// last, enable lcd!
	LCCR0 |= LCCR0_ENB;

	CKEN |= CKEN16_LCD;

	//printf("LCCR0 = 0x%08X LCCR1 = 0x%08X LCCR2 = 0x%08X LCCR3 = 0x%08X\r\n", LCCR0, LCCR1, LCCR2, LCCR3 );


	kernel_printf("lcd device init... [ok] \n");

	}

	lcd_clear( 0 );

	// test
	{


	int y;
	int x;
	for( y = 0 ; y < 100; y ++ )
	{
		for( x = 0 ; x < LCD_XRES; x++ )
		{
			put_pixel_16( x,y, RGB(255,0,0) );
		}
	}

	for( y = 100 ; y < 200; y ++ )
	{
		for( x = 0 ; x < LCD_XRES; x++ )
		{
			put_pixel_16( x,y, RGB(0,255,0) );
		}
	}
	}

//	lcd_init();

	return 1;
	
}

void lcd_clear( int16u col )
{
	unsigned long* lcd = (int32u*)LCD_BASE_ADDR;
	unsigned long lcol = ((int32u)col)<<16|col;
	int count = LCD_XRES*LCD_YRES*sizeof(int16u)/4;

	while( count--)
		*lcd++=lcol; 
}


void put_image( int16u* image )
{
	//memcpy( (int16u*)LCD_BASE_ADDR, image, LCD_XRES*LCD_YRES*sizeof(int16u) );
	/*
	unsigned long* lcd = (int32u*)LCD_BASE_ADDR;
	unsigned long* img = (int32u*)image;
	int count = LCD_XRES*LCD_YRES*sizeof(int16u)/4;

	while( count--)
		*lcd++=*img++; 
	*/


	int16u* base = (int16u*)LCD_BASE_ADDR;
	unsigned long* data = (unsigned long*)image;
	int count;
	count = (LCD_YRES*LCD_XRES) / 16;



#if 1
	__asm 
	{
//		ldr r2, [base]
//		ldr r3, [data]
		mov r2, base
		mov r3, data
	}


	while( count-- )
	{
			__asm
			{
					ldmia r3!, {r5-r12}
					stmia r2!, {r5-r12}
			}
	}
#endif
#if 0
	__asm __volatile__(
		"ldr r2, %0\n\t"
		"ldr r3, %1\n\t"
		:
		: "m" (base), "m" (data)
		: "r2", "r3"
	);


	while( count-- )
	{
	__asm__ __volatile__(
		"ldmia r3!, {r5-r12}\n\t"
		"stmia r2!, {r5-r12}\n\t"
	);
	}
#endif

}

void put_rect_image( int x, int y, int x2, int y2, int16u* img )
{
	/*
	int offset = y*LCD_XRES+x;
	int16u* base = (int16u*)LCD_BASE_ADDR + offset;
	int w = x2-x+1;
	int count;
	img = (int16u*)img + offset;
	int32u* base2;
	int32u* img2;

	for( ; y <= y2; y++ )
	{
		base2 = (int32u*)base;
		img2 = (int32u*)img;
		count = w/2;
		while( count-- )
			*base2++ = *img2++;
		base = (int16u*)base2;
		img = (int16u*)img2;
		if( w%2 )
			*base++ = *img++;
		base += (LCD_XRES-w);
		img += (LCD_XRES-w);
	}
	*/
	int offset = y*LCD_XRES+x;
	int16u* base = (int16u*)LCD_BASE_ADDR + offset;
	int w = x2-x+1;
	int count;
	img = (int16u*)img + offset;

	for( ; y <= y2; y++ )
	{
		count = w;
		while( count-- )
			*base++ = *img++;
		base += (LCD_XRES-w);
		img += (LCD_XRES-w);
	}
}

typedef struct _font_lcd
{
        int16u* pcrText;
        int16u* pcrBack;
}font_lcd;

void lcd_font_setpixel( int x, int y, int put, void* data )
{
        font_lcd* fd = (font_lcd*)data;

        if( put )
        {
                if( fd->pcrText )
                        put_pixel_16( x, y, *fd->pcrText );
        }
        else
        {
                if( fd->pcrBack ) 
                        put_pixel_16( x, y, *fd->pcrBack );
        }
}
#if 1
extern gputs_setpixel font_setpixel;
inline void put_text_16( int x, int y, char* s, int16u* col_text, int16u* col_back )
{
	font_lcd fl;
	gputs_setpixel old = font_setpixel;

	init_font( lcd_font_setpixel );

	fl.pcrText = col_text; fl.pcrBack = col_back;

	gputs( s, x, y, &fl );
	
	init_font( old );
}
#endif

inline void put_pixel_16( int x, int y, int16u col )
{
	*( ((int16u*)LCD_BASE_ADDR) + (y*LCD_XRES+x) ) = col;
}

inline void put_line_16( int x, int y, int x2, int y2, int16u col )
{
	if( y == y2 )
	{
	//put_fast_hline_16( x, y, x2, col );
	int32u* base = (int32u*) ( (int16u*)LCD_BASE_ADDR + (y*LCD_XRES+x) );
	int w = x2-x+1;
	int count;
	int col32 = ((int32u)col)<<16|col;

	if( w < 16 )
	{
		count = w / 2;
		while( count-- )
			*base++ = col32;
		if( w % 2 )
			*(int16u*)base = col;
		return;
	}

	count = w / 16;

#if 1
	__asm
	{

		mov r5, col32
		mov r6, r5
		mov r7, r5
		mov r8, r5
		mov r9, r5
		mov r10, r5
		mov r11, r5
		mov r12, r5
		mov r2, base
	}

	while( count-- )
	{
			__asm
			{
					stmia r2!, {r5-r12}
			}
	}

	__asm
	{
		str r2, [base]
	}
#endif
#if 0
	__asm__ __volatile__(

		"ldr r5, %0\n\t"
		"mov r6, r5\n\t" 
		"mov r7, r5\n\t" 
		"mov r8, r5\n\t" 
		"mov r9, r5\n\t" 
		"mov r10, r5\n\t" 
		"mov r11, r5\n\t" 
		"mov r12, r5\n\t" 
		"ldr r2, %1\n\t"
		:
		: "m" (col32), "m" (base)
		: "r2", "r5", "r6", "r7", "r8", "r9", "r10", "r11", "r12"
	);

	while( count-- )
	{
	__asm__ __volatile__(
		"stmia r2!, {r5-r12}\n\t"
	);
	}

	__asm__ __volatile__(
		"str r2, %0\n\t"
		:
		: "m" (base)
	);
#endif



	w = w % 16;
	count = w / 2;
	while( count-- )
		*base++ = col32;
	if( w % 2 )
		*(int16u*)base = col;
	/*
	count = w % 16;

	while( count-- )
		*base++ = col;
		*/
	/*
		// horizen
		int16u* px;
		int count = x2-x;
		px = (int16u*)LCD_BASE_ADDR + (y*LCD_XRES+x);
		while( count-- )
			*px++ = col;
	*/		
			
	}
	else
	{
		// vertical
		int16u* py;
		int count = y2-y;
		py = (int16u*)LCD_BASE_ADDR + (y*LCD_XRES+x);
		while( count-- )
		{
			*py = col;
			py += LCD_XRES;
		}
	}
}


int lcd_open( void )
{
	return 1;
}

int lcd_read( void* buf, int len )
{
	return 1;
}

int lcd_write( void* buf, int len )
{
	return 1;
}

int lcd_ioctl( void* wbuf, int wlen, void* rbuf, int rlen )
{
	return 1;
}

int lcd_close( void )
{
	return 1;
}
int lcd_term( void )
{
	return 1;
}
int lcd_init( void )
{
#if 0
	device* dev = dev_alloc();
	if( dev == NULL ) 
	{
		printf(" lcd device alloc fail.. \n");
		return 0;
	}
	
	strcpy( dev->name, "lcd" );

	dev->init = init_lcd;
        dev->open = lcd_open;                             
        dev->read = lcd_read;                             
        dev->write = lcd_write;                           
        dev->ioctl = lcd_ioctl;                           
        dev->close = lcd_close;                           
	dev->term = lcd_term;

#endif
	return 1;
}
