/*
   mk712 touch device driver
   by bro(bro@shinbiro.com)
*/
#include <arch/pxa255.h>
#include<kernel/timer.h>
//#include <stdio.h>
//#include <mem.h>
//#include <time.h>
//#include <string.h>
//#include <irq.h>
#include <cqa.h>
#include <dev/touch/touch.h>
#include <dev/lcd/lcd.h>


void pxa_set_irq( int gpio, int rise_edge );
void pxa_ack_irq( int gpio );


// irq.h Ǿ.
#define PXA_RISE_EDGE		1



#define MK712_SCREEN_X			665 // 640 is correct, but 665 is ok;
#define MK712_SCREEN_Y			480
#define MK712_MAX_X			4000
#define MK712_MAX_Y			4000

#define MK712_GPIO			22
//#define	MK712_IO			0xf1300000
#define MK712_IO	(PXA_CS1_PHYS+0x400000) // 0x04400000 nCS1+A20 by ez-x5
#define MK712_IRQ			RQ_GPIO(22) // gpio = 22, irq = 45

/* eight 8-bit registers */
#define MK712_STATUS_LOW        (0<<1)          /* READ */
#define MK712_STATUS_HIGH       (1<<1)          /* READ */
#define MK712_X_LOW             (2<<1)          /* READ */
#define MK712_X_HIGH            (3<<1)          /* READ */
#define MK712_Y_LOW             (4<<1)          /* READ */
#define MK712_Y_HIGH            (5<<1)          /* READ */
#define MK712_CONTROL           (6<<1)          /* R/W */
#define MK712_RATE              (7<<1)          /* R/W */

	/* status */
#define MK712_STATUS_TOUCH              0x10
#define MK712_CONVERSION_COMPLETE       0x80

#define MK712_ENABLE_INT                        0x01 /* enable interrupts */
#define MK712_INT_ON_CONVERSION_COMPLETE        0x02 /* if bit 0 = 1 */
#define MK712_INT_ON_CHANGE_IN_TOUCH_STATUS_A   0x04 /* if bit 0 = 1 */
#define MK712_INT_ON_CHANGE_IN_TOUCH_STATUS_B   0x08 /* if bit 0 = 1 */
#define MK712_ENABLE_PERIODIC_CONVERSIONS       0x10
#define MK712_READ_ONE_POINT                    0x20
#define MK712_POWERDOWN_A                       0x40
#define MK712_POWERDOWN_B                       0x80

#define _outb(val,addr)		(*(volatile unsigned char*)(addr)=(val))
#define _inb(addr)		(*(volatile unsigned char*)(addr))
#define _byte_order_16(x)	(x)
#define _inw(addr)		(_byte_order_16(*(volatile unsigned short*)(addr)) )

#define MK712_STATUS_DOWN			0
#define MK712_STATUS_MOVE			1
#define MK712_STATUS_UP				2

typedef struct _mk712_data
{
	int status; // 0= down 1=move 2=up
	int x, y;
}mk712_data;

mk712_data	g_mk712_data[MAX_MK712_DATA];
int		g_mk712_data_index;
CQA_DATA	g_mk712_cqa_data[MAX_MK712_DATA];
cqa		g_mk712_cqa;	

mk712_data g_mk712_prev;
int		g_mk712_debug;

int mk712_new_data(void);
void mk712_do_irq(void);

typedef struct _mk712_fix
{
	int filter;
	int x_y_flip;
	int x_flip, y_flip;
	int txs, txw; // tx start, tx width
	int tys, tyh; // ty start, ty height

	int sx, sy; // screen
}mk712_fix;

mk712_fix	g_mk712_fix;

void _swap(int *a,int *b)
{
	int t=*a;
	*a=*b;*b=t;
}
#if 0
int abs( int a )
{
	return (a<0)?-a:a;	
}
#endif
int min( int a, int b )
{
	return (a>b)?b:a;
}
int max( int a, int b )
{
	return (a>b)?a:b;
}
void mk712_setup( int screen_width, int screen_height, int x1, int y1, int x2, int y2, int x3, int y3, mk712_fix* f )
{

	f->x_flip = x1>x2;
	f->y_flip = y1>y3;

	f->txs = min(min(x1,x2),x3);
	f->txw = max(max(x1,x2),x3) - f->txs;

	//printf("y1=%d y2=%d y3=%d\r\n max(y1,y2)=%d max(max(y1,y2),y3)=%d f->tys=%d\r\n", y1,y2,y3, max(y1,y2), max(max(y1,y2),y3), f->tys );
	f->tys = min(min(y1,y2),y3);
	f->tyh = max(max(y1,y2),y3) - f->tys;
	//printf("mk712: x_y_flip=%d x_flip=%d y_flip=%d \r\n\t txs=%d txw=%d tys=%d tyh=%d\r\n", f->x_y_flip, f->x_flip, f->y_flip, f->txs, f->txw, f->tys, f->tyh );

	f->x_y_flip = abs(x3-x2)>abs(y3-y2);
	if( f->x_y_flip )
	{
		_swap(&f->txs,&f->tys);		
		_swap(&f->txw,&f->tyh);
		_swap(&f->x_flip,&f->y_flip); 
	}
	f->sx = screen_width;
	f->sy = screen_height;
	f->filter = 1;

	migos_printf("mk712: setup_touch:\r\n\tx_y_flip=%d x_flip=%d y_flip=%d \r\n\t txs=%d txw=%d tys=%d tyh=%d\r\n sx=%d sy=%d\r\n", f->x_y_flip, f->x_flip, f->y_flip, f->txs, f->txw, f->tys, f->tyh, f->sx, f->sy );

}

void mk712_fixup( unsigned short* sx, unsigned short* sy, mk712_fix* f )
{
	int x = *sx;
	int y = *sy;

	if( !f->filter )
		return;
	//printf("x=%d y=%d\r\n",x,y);
	// x-y flip
	if( f->x_y_flip )
		_swap(&x,&y);
	//printf("swap x=%d y=%d\r\n",x,y);

	// x delta
	x -= f->txs;
	if( x<0 ) x =0;
	// x flip
	if( f->x_flip )
		//x = f->txw+f->txs-x;
		x = f->txw-x; 
	//printf("x flip x=%d y=%d\r\n",x,y);
	// y delta
	y -= f->tys;
	if( y < 0 ) y = 0;
	// y flip
	if( f->y_flip )
		//y = f->tyh+f->tys-y;
		y = f->tyh-y;
	//printf("y flip x=%d y=%d\r\n",x,y);
	
	// logical coordinate
	x = ( x * f->sx ) / f->txw;
	y = ( y * f->sy ) / f->tyh;

	//x = ( ((LCD_WIDTH-20)*(x-f->txs))+(10*f->txw) ) / f->txw; //+ 30;
	//x =  (LCD_WIDTH*x)/ (f->txw+f->txs);
	//y =  (LCD_HEIGHT*y)/ (f->tyh+f->tys*2);
	//y = ( ((LCD_HEIGHT-20)*(y-f->tys))+(10*f->tyh) ) / f->tyh;// - 10; 
	if( x < 0 ) x = 0;
	if( y < 0 ) y = 0;

	//printf("x=%d y=%d\r\n",x,y);

	*sx = (unsigned short)x;
	*sy = (unsigned short)y;
}


// where = 0 topleft  =1 topright =2 rightbottom
void get_touch_xy( int* x, int* y, int where  )
{
	int status;
	int rx=-1, ry=-1;
	int cx, cy; 
	while(1)
	{
		if( touch_get_event( &cx, &cy, &status ) == 0 )
			continue;
		if( rx == -1 )
		{
			rx =cx; ry =cy;
		}

		if( status == MK712_STATUS_UP )
		{
			break;
		}

		if( where == 0 )
		{
			if( cx < rx ) rx = cx;
			if( cy < ry ) ry = cy;
		}
		else
		if( where == 1 )
		{
			if( cx > rx ) rx = cx;
			if( cy < ry ) ry = cy;
		}
		else
		if( where == 2 )
		{
			if( cx > rx ) rx = cx;
			if( cy > ry ) ry = cy;
		}
	}

	*x =rx; *y = ry;

	migos_printf("mk712: get_touch_xy step=[%d] %d %d\r\n", where, rx, ry );
}


void setup_touch()
{
	int16u col,col2;
	int x1,y1,x2,y2,x3,y3;

	memset( &g_mk712_fix, 0, sizeof(mk712_fix) );
	// clear lcd
	lcd_clear(RGB(255,255,255));
	col = RGB(0,0,0);
	col2 = RGB(255,255,255);

	// flush touch event
	while( touch_get_event( &x1, &y2, &x2 ) );

	put_text_16( 100, 100, " ʽÿ..ũ  ȭԴϴ!!", &col, &col2  );
	put_text_16( 100, 120, "ġũ     ʽÿ!     ",&col, &col2 );
	put_line_16( 0, 10, 20,10, RGB(0,0,0) );
	put_line_16( 10, 0, 10,20, RGB(0,0,0) );
	// get point screen (x1,y1)
	get_touch_xy( &x1, &y1, 0 );

	usleep(500);

	// flush touch event
	while( touch_get_event( &x1, &y2, &x2 ) );

	put_text_16( 100, 140, "ġũ     ʽÿ!     ",&col, &col2 );
	put_line_16( LCD_WIDTH-20, 10, LCD_WIDTH, 10, RGB(0,0,0) );
	put_line_16( LCD_WIDTH-10, 0, LCD_WIDTH-10, 20, RGB(0,0,0) );
	get_touch_xy( &x2, &y2, 1 );

	put_line_16( LCD_WIDTH-20, LCD_HEIGHT-10, LCD_WIDTH, LCD_HEIGHT-10, RGB(0,0,0) );
	put_line_16( LCD_WIDTH-10, LCD_HEIGHT-20, LCD_WIDTH-10, LCD_HEIGHT, RGB(0,0,0) );

	usleep(500);

	// flush touch event
	while( touch_get_event( &x1, &y2, &x2 ) );
	put_text_16( 100, 160, "ġũ   Ʒ  ʽÿ!     ",&col, &col2 );

	get_touch_xy( &x3, &y3, 2 );
	
	mk712_setup( LCD_WIDTH, LCD_HEIGHT, x1, y1, x2, y2, x3, y3, &g_mk712_fix );

	// flush touch event
	while( touch_get_event( &x1, &y2, &x2 ) );
}

void init_touch()
{
	migos_printf("init_touch() ...\r\n");
/*	
	// GPDR = input, GAFR = general
	// GPIO input configuration
	// GPDR[22] = 0 (input)
	// GPDR0[31:0]
	GPDR0 = GPDR0 & ~(1<<22); 
	printf("GPDR0 = %08X   ~(1<<22) = %08X \r\n", &GPDR0, ~(1<<22) );

	// GAFR addtional function configuration
	// GAFR[22] = 0 (general)
	// GAFR1_L[47:32]
	GAFR0_U = GAFR0_U & ~(3<<12);

	// set rising edge so GRER = 1, GFER = 0
	// GRER[22] = 1 (rising edge)
	// GFER[22] = 0
	GRER0 = GRER0 | (1<<22);
	GFER0 = GFER0 & ~(1<<22);

	ICLR = ICLR & ~(1<<22);

	// ICMR[22] = 1 ( no mask, occur event )
	ICMR |= (1<<22);

*/	
	// mk712 packet data init
	memset(  &g_mk712_fix, 0 ,sizeof( mk712_fix ) );
	g_mk712_debug = 0;
	g_mk712_data_index = 0;
	g_mk712_prev.status = MK712_STATUS_UP;
	cqa_init( &g_mk712_cqa, g_mk712_cqa_data, MAX_MK712_DATA );


#if IRQ_FUNCTION
	pxa_set_irq( MK712_GPIO , PXA_RISE_EDGE );
#endif


//	ICCR &= ~(1<<0);
	// MK712 Control Enable
	{
	unsigned char control = 0;
	_outb( 0, MK712_IO + MK712_CONTROL ); 
        control |= (MK712_ENABLE_INT |
	            MK712_INT_ON_CONVERSION_COMPLETE |
		    MK712_INT_ON_CHANGE_IN_TOUCH_STATUS_B |
		    MK712_ENABLE_PERIODIC_CONVERSIONS |
		    MK712_POWERDOWN_A);


	_outb( control, MK712_IO + MK712_CONTROL );
	_outb( 10, MK712_IO + MK712_RATE ); /* default count = 10 */
	}

	ICMR |= (1<<10);
	// set interrupt CPSR
	//enable_irq();

}

void touch_ack()
{
	unsigned char control = 0;
	_outb( 0, MK712_IO + MK712_CONTROL ); 
	 control |= (MK712_ENABLE_INT |
	            MK712_INT_ON_CONVERSION_COMPLETE |
		    MK712_INT_ON_CHANGE_IN_TOUCH_STATUS_B |
		    MK712_ENABLE_PERIODIC_CONVERSIONS |
		    MK712_POWERDOWN_A);

	_outb( control, MK712_IO + MK712_CONTROL );
	_outb( 10, MK712_IO + MK712_RATE ); /* default count = 10 */


#if IRQ_FUNCTION
//	pxa_ack_irq( MK712_GPIO );
#endif
	
	
}

int touch_get( mk712_data* pd )
{
	unsigned short x, y, xl,yl,xh, yh;
	unsigned char status;
	status = _inb( MK712_IO + MK712_STATUS_LOW );	
	
	if( !(status & MK712_CONVERSION_COMPLETE) )
	{
		/*
		pd->x = g_mk712_prev.x;
		pd->y = g_mk712_prev.y;
		pd->status = MK712_STATUS_UP;
		g_mk712_prev.status = MK712_STATUS_UP;
		*/
		migos_printf("mk712: !conversion_complete\r\n");
		return 0;
	}

	if( !(status & MK712_STATUS_TOUCH) )
	{
		//if( g_mk712_prev.status == MK712_STATUS_UP )
		//	return 0;
		g_mk712_prev.status = MK712_STATUS_UP;

		pd->x = g_mk712_prev.x;
		pd->y = g_mk712_prev.y;
		pd->status = MK712_STATUS_UP;
		if( g_mk712_debug )
			migos_printf("mk712: [%04d:%04d] %d\r\n", pd->x, pd->y, pd->status );
		return 1;
	}

        xl = _inw(MK712_IO + MK712_X_LOW) & 0x0ff;
	yl = _inw(MK712_IO + MK712_Y_LOW) & 0x0ff;
	xh = _inw(MK712_IO + MK712_X_HIGH) & 0x0ff;
	yh = _inw(MK712_IO + MK712_Y_HIGH) & 0x0ff;

	x = ( ( xh << 8 ) & 0xf00 ) | ( xl  & 0xff );
	y = ( ( yh << 8 ) & 0xf00 ) | ( yl  & 0xff );

	//printf("mk712: *[%04d:%04d] %d\r\n", x, y, pd->status );
//	mk712_fixup( &x, &y, &g_mk712_fix );
	//x = (x * MK712_SCREEN_Y ) / MK712_MAX_Y;
	//y = (y * MK712_SCREEN_X ) / MK712_MAX_X;

	//if( g_mk712_prev.x == x && g_mk712_prev.y == y )
//		return 0;
	if( x >= g_mk712_prev.x-MK712_SKIP_PIXEL &&
		x <= g_mk712_prev.x+MK712_SKIP_PIXEL &&
		y >= g_mk712_prev.y-MK712_SKIP_PIXEL &&
		y <= g_mk712_prev.y+MK712_SKIP_PIXEL )
		return 0;

	if( g_mk712_prev.status == MK712_STATUS_UP )
	{
		pd->status = MK712_STATUS_DOWN;
		g_mk712_prev.status = MK712_STATUS_DOWN;
	}
	else
	if( g_mk712_prev.status == MK712_STATUS_DOWN )
	{
		pd->status = MK712_STATUS_MOVE;
		g_mk712_prev.status = MK712_STATUS_MOVE;
	}
	else
		pd->status = MK712_STATUS_MOVE;


	pd->x = x; pd->y = y;
	g_mk712_prev.x = x; g_mk712_prev.y = y;

	if( g_mk712_debug )
		migos_printf("mk712: [%04d:%04d] %d\r\n", x, y, pd->status );

	return 1;
}

int mk712_new_data()
{
	int i = g_mk712_data_index;
	g_mk712_data_index=(g_mk712_data_index+1)%MAX_MK712_DATA;
	return i;
}

void mk712_do_irq()
{
	mk712_data d;
	int cpu_sr;
	
	if( touch_get( &d ) )
	{
		CQA_DATA i;
		mk712_data* pd;
		MK712_LOCK;
		i = mk712_new_data();
		cqa_push( &g_mk712_cqa, (CQA_DATA)i );
		pd = &g_mk712_data[i];
		*pd = d;
		MK712_UNLOCK;
	}

	touch_ack();
}

void touch_poll()
{
	while(1)
	{
		if( (ICPR & GPIO_bit(10)) && 
			(GEDR(MK712_GPIO) & GPIO_bit(MK712_GPIO)) )
		{
			mk712_do_irq();
		}	
	}

}

int touch_get_event( int* x, int* y, int* status )
{
	CQA_DATA i;
	mk712_data* pd;
	int cpu_sr;

	MK712_LOCK;
	i = cqa_pop( &g_mk712_cqa );

	if( i != CQA_EMPTY )
	{	
		pd = &g_mk712_data[i];
		*x = pd->x;
		*y = pd->y;
		*status = pd->status;
	}
	MK712_UNLOCK;

	return (i != CQA_EMPTY);
}

// ó  .
void temp_taxoo()
{
	return;
}


void pxa_ack_irq( int gpio )
{
	ICMR &= ~GPIO_bit(gpio);
	GEDR(gpio) |= GPIO_bit(gpio);
}

void pxa_set_irq( int gpio, int rise_edge )
{
	// input gpio
	GPDR(gpio) &= ~GPIO_bit(gpio);

	if( rise_edge )
	{
		GRER(gpio) |= GPIO_bit(gpio);
		GFER(gpio) &= ~GPIO_bit(gpio);
	}
	else
	{
		GRER(gpio) &= ~GPIO_bit(gpio);
		GFER(gpio) |= GPIO_bit(gpio);
	}

	// irq or fiq 
	//if( irq )
		ICLR &= ~(1<<gpio);
	//else
	//	ICLR |= (1<<gpio); 	

}


#if 0
void touch_handler(void)
{
		int cpu_sr;

		for(;;){

				wait_thread(get_current_thread());
				mk712_do_irq();
				ICMR |= (1<<10);
		}

}
#endif
