#include <string.h>
#include <kernel/mm.h>
//#include <malloc.h>
//#include "gui.h"
#include <gui/gui.h>
//#include "font.h"
#include <gui/font.h>
#include <migos_stdio.h>

struct _gui gui;
blist msgs;
blist wndclss;

void _gui_line( int x, int y, int x2, int y2, COLOR cr );

bitmap* create_bitmap( int width, int height, int bpp )
{
	int i;

	// create bitmap memory with extra helper line buffer
	bitmap* bm = (bitmap*)k_malloc( sizeof(bitmap) + sizeof( char* ) * height );
	if( bm == 0 )
		return 0;
	
	bm->width = width;
	bm->height = height;
	bm->bpp = bpp;

	bm->byte_per_pixel = (bpp+7)/8;

	// create bitmap data
	bm->data = k_malloc( width * height * bm->byte_per_pixel );
	if( bm->data == 0 )
	{
		k_free( bm );
		return 0;
	}

	// set helper line array
	bm->line[0] = bm->data;
	
	for( i = 0 ; i < height ; i ++ )
		bm->line[i] = bm->line[i-1] + bm->byte_per_pixel;
	
	return bm;
}

void destory_bitmap( bitmap* bm )
{
	if( bm )
	{
		k_free( bm->data );
		k_free( bm );
	}
}

void putpix( bitmap* bm, int x, int y, COLOR col )
{
	switch( bm->bpp )
	{
	case 1:
		break;
	case 2:
		break;
	case 4:
		break;
	case 8:
		bm->line[y][x] = (char)col;
		break;
	case 16:
		*((short*)&bm->line[y][x]) = (short)col;
		break;
	case 32:
		*((long*)&bm->line[y][x]) = (long)col;
	}
}

COLOR getpix( bitmap* bm, int x, int y )
{
	COLOR col = 0;	
	switch( bm->bpp )
	{
	case 1:
		break;
	case 2:
		break;
	case 4:
		break;
	case 8:
		col = bm->line[y][x];
		break;
	case 16:
		col = *((short*)&bm->line[y][x]);
		break;
	case 32:
		col = *((long*)&bm->line[y][x]);
	}
	return col;
}

void blit( bitmap* des, int desx, int desy, bitmap* src, int x, int y, int w, int h )
{
	
}

wndclass* find_class( char* clsname )
{
	blist_node* cls = wndclss.head;
	while( cls )
	{
		wndclass* c = BNODE_ITEM_ADDR( cls, wndclass, node );
		if( strcmp( c->name, clsname ) == 0 )
			return c;
		cls = cls->next;
	}
	return 0;
}

int register_class( wndclass* wndcls )
{
	wndclass* c;

	if( find_class( wndcls->name ) )
		return 0;

	c = k_malloc( sizeof(wndclass) );
	if( c == 0 )
		return 0;
	memset( c, 0 , sizeof(wndclass) );

	//*c = *wndcls;
	c->winproc = wndcls->winproc;
	c->cb_wnd_extra = wndcls->cb_wnd_extra;
	strcpy( c->name, wndcls->name );
	blist_add_tail( &wndclss, &c->node );	

	return 1;
}

void set_rect( rect* rt, int l, int t, int r, int b )
{
	rt->left = l;
	rt->top = t;
	rt->right = r;
	rt->bottom = b;
}

void rect_smaller( rect* rt, int i )
{
	rt->top += i;
	rt->left += i;
	rt->right -= i;
	rt->bottom -= i;
}

void rect_move( rect* rt, point* org )
{
	rt->left += org->x;
	rt->right += org->x;
	rt->top += org->y;
	rt->bottom += org->y;
}

int rect_in( rect* rt, int x, int y )
{
	if( rt->left <= x && rt->right >= x &&
		rt->top <= y && rt->bottom >= y )
		return 1;
	return 0;
}

int rect_in_rect( rect* rtBig, int x1, int y1, int x2, int y2 )
{
	// in rect
	if( rtBig->left <= x1 && rtBig->right >= x2 &&
		rtBig->top <= y1 && rtBig->bottom >= y2 )
		return 1;
	
	// subset
	//if( rtBig->left <= x1 && rtBig->right <= x2 &&

	return 0;
}

int rect_compare( rect* rt1, rect* rt2 )
{
	// out of rect
	if( rt1->right < rt2->left ||     // rt1   rt2
		rt2->right < rt1->left ||     // rt2   rt1
		rt1->bottom < rt2->top ||     // rt1(up) rt2(down)
		rt2->bottom < rt1->top )      // rt2(up) rt1(down)
		return 0;

	// include big rt1
	if( rt1->left <= rt2->left && rt1->right >= rt2->right &&
		rt1->top <= rt2->top && rt1->bottom >= rt2->bottom )
		return 1;
	
	// include big rt2
	if( rt2->left <= rt1->left && rt2->right >= rt1->right &&
		rt2->top <= rt1->top && rt2->bottom >= rt1->bottom )
		return 2;

	// intersect
	return 3;
}

#define MIN(x,y)	(x>y?y:x)
#define MAX(x,y)	(x>y?x:y)

void rect_intersect( rect* rt1, rect* rt2, rect* rtsub )
{
	rtsub->left = MAX(rt1->left, rt2->left);
	rtsub->top = MAX(rt1->top, rt2->top);
	rtsub->right = MIN(rt1->right, rt2->right);
	rtsub->bottom = MIN(rt1->bottom, rt2->bottom);
}

void client_to_screen( hwnd w, point* pt )
{
	window* wnd = (window*)w;
	int dx = 0 , dy = 0;

	if( (hwnd)wnd == gui.desktop )
		return;

	while( wnd->parent )
	{
		dx += wnd->rt.left;
		dy += wnd->rt.top;
		dx += wnd->rtclient.left;
		dy += wnd->rtclient.top;
		wnd = wnd->parent;
		if( (hwnd)wnd == gui.desktop )
			break;

	}
	pt->x += dx;
	pt->y += dy;
}

void client_to_screen_rt( hwnd w, rect* rt )
{
	window* wnd = (window*)w;
	int dx = 0 , dy = 0;

	if( (hwnd)wnd == gui.desktop )
		return;

	while( wnd->parent )
	{
		wnd = wnd->parent;
		if( (hwnd)wnd == gui.desktop )
			break;
	
		dx += wnd->rt.left;
		dy += wnd->rt.top;
		dx += wnd->rtclient.left;
		dy += wnd->rtclient.top;
	}
	rt->left += dx;
	rt->top += dy;
	rt->right += dx;
	rt->bottom += dy;
}

void screen_to_client( hwnd w, point* pt )
{
	window* wnd = (window*)w;
	int dx = 0 , dy = 0;

	if( (hwnd)wnd == gui.desktop )
		return;

	while( wnd->parent )
	{
		dx += wnd->rt.left;
		dy += wnd->rt.top;
		dx += wnd->rtclient.left;
		dy += wnd->rtclient.top;
		wnd = wnd->parent;
		if( (hwnd)wnd == gui.desktop )
			break;

	}
	pt->x -= dx;
	pt->y -= dy;
}

hdc get_dc( hwnd w )
{
	window* wnd = (window*)w;
	device_context* pdc;
	rect rt;
	pdc = k_malloc( sizeof(device_context) );
	memset( pdc, 0, sizeof(device_context) );
	
	pdc->w = w;

	// set org
	get_window_rect( w, &rt );
	pdc->org.x = 0;
	pdc->org.y = 0;
	client_to_screen( w, &pdc->org );

	// set clip bounds
	pdc->clip_count = 1;
	pdc->clip[0].left = 0;
	pdc->clip[0].top = 0;
	pdc->clip[0].right = wnd->rtclient.right - wnd->rtclient.left;
	pdc->clip[0].bottom = wnd->rtclient.bottom - wnd->rtclient.top;
	
	return (hdc)pdc;
}

hdc get_window_dc( hwnd w )
{
	device_context* pdc;
	rect rt;
	pdc = k_malloc( sizeof(device_context) );
	memset( pdc, 0, sizeof(device_context) );
	
	pdc->w = w;

	// set org
	get_window_rect( w, &rt );
	pdc->org.x = rt.left;
	pdc->org.y = rt.top;
	
	// set clip bounds
	pdc->clip_count = 1;
	pdc->clip[0].left = 0;
	pdc->clip[0].top = 0;
	pdc->clip[0].right = rt.right - rt.left;
	pdc->clip[0].bottom = rt.bottom - rt.top;
	
	return (hdc)pdc;
}

void release_dc( hdc dc )
{
	k_free( (device_context*)dc );
}

int _gdi_is_clip( hdc hdc, int x, int y )
{
	int i;
	device_context* pdc = (device_context*)hdc;
	
	// check clipping
	for( i = 0; i < pdc->clip_count; i ++ )
	{
		if( rect_in( &pdc->clip[i], x, y ) )
			return 1;
	}

	return 0;
}

int _gdi_is_clip_rect( hdc hdc, int x1, int y1, int x2, int y2 )
{
	int i;
	device_context* pdc = (device_context*)hdc;
	
	// check clipping
	for( i = 0; i < pdc->clip_count; i ++ )
	{
		if( rect_in_rect( &pdc->clip[i], x1, y1, x2, y2 ) )
			return 1;
	}

	return 0;
}

void gdi_setpixel( hdc dc, int x, int y, COLOR cr )
{
	device_context* pdc = (device_context*)dc;

	if( !_gdi_is_clip( dc, x, y ) )
		return;

	x += pdc->org.x;
	y += pdc->org.y;

	gui.setpixel( x, y, cr );
}

#define ABS(x) (x>0?x:-x)

void gdi_line( hdc dc, int x1, int y1, int x2, int y2, COLOR cr )
{
	device_context* pdc = (device_context*)dc;
	int i;
	if( y1 == y2 )
	{
		// horizon
		int x_1 = x1, x_2 = x2;
		for( i = 0; i < pdc->clip_count; i ++ )
		{
			if( y1 >= pdc->clip[i].top && y1 <= pdc->clip[i].bottom )
			{
				if( x1 <= pdc->clip[i].right || x2 >= pdc->clip[i].left )
				{
					if( x1 < pdc->clip[i].left )
						x_1 = pdc->clip[i].left;
					if( x2 > pdc->clip[i].right )
						x_2 = pdc->clip[i].right;
					gui.line( x_1 +pdc->org.x, y1 +pdc->org.y, x_2+1+pdc->org.x, y2 +pdc->org.y, cr );
				}
			}
		}
	}
	else
	if( x1 == x2 )
	{
		// vertical
		int y_1 = y1, y_2 = y2;
		for( i = 0; i < pdc->clip_count; i ++ )
		{
			if( x1 >= pdc->clip[i].left && x1 <= pdc->clip[i].right )
			{
				if( y1 <= pdc->clip[i].bottom || y2 >= pdc->clip[i].top )
				{
					if( y1 < pdc->clip[i].top )
						y_1 = pdc->clip[i].top;
					if( y2 > pdc->clip[i].bottom )
						y_2 = pdc->clip[i].bottom;
					gui.line( x1 +pdc->org.x, y_1 +pdc->org.y, x2 +pdc->org.x, y_2 +pdc->org.y, cr );
				}
			}
		}
	}
	else
	{
		/*
		// cross
		int dx, dy, step, k;
		float incx, incy, x, y;

		dx = x2-x1;
		dy = y2-y1;
		if(ABS(dx) > ABS(dy))
			step = ABS(dx);
		else
			step = ABS(dy);

		incx = dx/(float)step;
		incy = dy/(float)step;
		
		x = (float)x1;
		y = (float)y1;

		gdi_setpixel( dc, x1, y1, cr );
		for(k=1; k<step; k++)
			gdi_setpixel( dc, (int)(x+=incx), (int)(y+=incy), cr );
			*/
	}
}

void _gui_line( int x, int y, int x2, int y2, COLOR cr )
{
	int i;
	for( i = x; i <= x2; i++)
		gui.setpixel( i, y, cr );
}

void _gui_rect( rect* rt, COLOR cr, int fill )
{
	if( fill )
	{
		int y;
		for( y = rt->top; y <= rt->bottom; y++ )
			gui.line( rt->left, y, rt->right+1, y, cr );
	}
	else
	{
		gui.line( rt->left, rt->top, rt->right, rt->top, cr );
		gui.line( rt->left, rt->bottom, rt->right+1, rt->bottom, cr );
		gui.line( rt->left, rt->top, rt->left, rt->bottom, cr );
		gui.line( rt->right, rt->top, rt->right, rt->bottom, cr );
	}
}

void gdi_rect( hdc dc, rect* rt, COLOR cr, int fill )
{
	device_context* pdc = (device_context*)dc;
	rect rt2;
	int i, compare;

	// check clipping
	for( i = 0; i < pdc->clip_count; i ++ )
	{
		compare = rect_compare( rt, &pdc->clip[i] );
		if( compare == 0 ) // out of
			continue;
		else
		if( compare == 1 ) // rt > clip
		{
			rt2 = pdc->clip[i];
			rect_move( &rt2, &pdc->org );
			_gui_rect( &rt2, cr, fill );
		}
		else
		if( compare == 2 ) // rt < clip
		{
			rt2 = *rt;
			rect_move( &rt2, &pdc->org );
			_gui_rect( &rt2, cr, fill );
		}
		else
		if( compare == 3 ) // intersect
		{
			rect_intersect( rt, &pdc->clip[i], &rt2 );
			rect_move( &rt2, &pdc->org );
			_gui_rect( &rt2, cr, fill );
		}
	}
}

long send_message( hwnd w, unsigned long message, unsigned long wparam, unsigned long lparam )
{
	window* wnd = (window*)w;
	if( wnd )
		return wnd->wndcls->winproc( w, message, wparam, lparam );
	return 0;
}

long post_message( hwnd w, unsigned long message, unsigned long wparam, unsigned long lparam )
{
	msg* m;

	m = k_malloc( sizeof(msg) );
	if( m == 0 )
	{
		return 0;
	}
	memset( m, 0 , sizeof(msg) );

	m->wnd = w;
	m->message = message;
	m->wparam = wparam;
	m->lparam = lparam;
	blist_add_tail( &msgs, &m->node );
	return 1;
}

hwnd create_window( char* clsname, char* name, unsigned long style, int x, int y, int w, int h, hwnd parent, int wid )
{
	hwnd hw;
	window* wnd;

	wnd = k_malloc( sizeof(window) );
	if( wnd == 0 )
		return 0;
	memset( wnd, 0, sizeof(window) );

	wnd->wndcls = find_class( clsname );
	if( wnd->wndcls == 0 )
	{
		k_free(wnd);
		return 0;
	}

	hw = (hwnd)wnd;	 
	wnd->rt.left = x;
	wnd->rt.top = y;
	wnd->rt.right = x+w;
	wnd->rt.bottom = y+h;

	strcpy( wnd->name, name );
	wnd->wid = wid;

	if( style == 0 )
		style = ws_overlappedwindow;
	wnd->ws_style = style;

	if( wnd->wndcls->cb_wnd_extra )
	{
		wnd->extra = k_malloc( wnd->wndcls->cb_wnd_extra );
		memset( wnd->extra, 0, wnd->wndcls->cb_wnd_extra );
	}

	//default show
	wnd->showhide = 1;

	// calc client rect
	wnd->rtclient.left = 0;
	wnd->rtclient.top = 0;
	wnd->rtclient.right = w;
	wnd->rtclient.bottom = h;

	// window relative setting
	wnd->next = 0;
	wnd->prev = 0;
	wnd->child = 0;

	set_parent( parent, hw );
	if( wnd->parent == (window*)gui.desktop )
	{
		send_message( gui.taskbar, wm_taskcommand, 1, hw );
	}

	// creation flow
	post_message( hw, wm_create, 0, 0 );
	post_message( hw, wm_ncpaint, 0, 0 );
	post_message( hw, wm_erasebkgnd, 0, 0 );
	post_message( hw, wm_paint, 0, 0 );
	return hw;
}

hwnd get_top_window( hwnd w )
{
	window* c = (window*)w;
	while( c->parent && c->parent != (window*)gui.desktop )
		c = c->parent;
	
	if( c->parent == 0 )
		c->parent = (window*)gui.desktop;
	return (hwnd)c;
}

window* _get_head_sibling_window( window* w )
{
	window* c = w;
	while( c && c->prev )
		c = c->prev;
	return c;
}

window* _get_last_sibling_window( window* w )
{
	window* c = w;
	while( c && c->next )
		c = c->next;
	return c;
}

window* _get_prev_sibling_window( window* w )
{
	window* p = w->parent;
	while( p && p->next && p->next != w )
		p = p->next;
	return p;
}

window* _detach_window( window* w )
{
	if( w->prev )
		w->prev->next = w->next;
	if( w->next )
		w->next->prev = w->prev;
	return w;
}

void _set_sibling_window( window* org, window* next )
{
	// detach previos link
	next = _detach_window( next );
	
	// attach to org
	if( org->next )
		org->next->prev = next;
	next->next = org->next;

	org->next = next;
	next->prev = org;
}

void _set_first_sibling_window( window* w )
{
	if( w->parent == 0 )
		return;
	if( w->parent->child == 0 )
		return;
	if( w->parent->child == w )
		return;	

	// detach previos link
	if( w->prev )
		w->prev->next = w->next;

	// attach to first
	if( w->parent->child )
		w->parent->child->prev = w;
	w->next = w->parent->child;
	w->prev = 0;
	w->parent->child = w;	
}

void _set_last_sibling_window( window* w )
{
	window* last_sibling_wnd;

	if( w->parent == 0 )
		return;
	if( w->parent->child == 0 )
		return;

	if( w->parent->child->next == 0 )
		return;

	last_sibling_wnd = _get_last_sibling_window( w );
	if( last_sibling_wnd == w )
		return;
	_set_sibling_window( last_sibling_wnd, w );

	if( w->parent->child == w )
		w->parent->child = _get_head_sibling_window(w);
}

void _add_child_front( window* parent, window* child )
{
	child->parent = parent;
	child->next = parent->child;

	if( parent->child )
	{
		if( parent->child->next ) 
			parent->child->next->prev = child;
	}
	
	parent->child = child;
}

void _add_child_last( window* parent, window* child )
{
	window* last_sibling_wnd;
	
	child->parent = parent;
	
	if( parent->child )
	{
		last_sibling_wnd = _get_last_sibling_window( parent->child );
		_set_sibling_window( last_sibling_wnd, child );
	}
	else
		parent->child = child;
}

void set_parent( hwnd w, hwnd c )
{
	window* parent_wnd = (window*)w;
	window* child_wnd = (window*)c;

	if( w == 0 )
		parent_wnd = (window*)gui.desktop;
	
	if( w == -1 )
	{
		child_wnd->parent = 0;
		return; 
	}

	if( parent_wnd == child_wnd )
		return;

	_add_child_last( parent_wnd, child_wnd );
}


int _window_from_point( hwnd w, point pt, hwnd* found )
{
	window* wnd = (window*)w;
	window* child;
	rect rt;
	
	if( wnd->showhide == 0 ) // hide
		return 0;
	
	child = _get_last_sibling_window( wnd->child );
	
	while( child )
	{
		if( _window_from_point( (hwnd)child, pt, found ) )
			return 1;

		child = child->prev;
	}

	get_window_rect( w, &rt );
	if( rect_in( &rt, pt.x, pt.y ) == 1 )
	{
		*found = w;
		return 1;
	}

	return 0;
}

hwnd window_from_point( point pt )
{	
	hwnd found;
	if( !_window_from_point( gui.desktop, pt, &found ) )
		return gui.desktop;
	return found;
}

void move_window( hwnd hw, int x, int y, int w, int h, int repaint )
{
	window* wnd = (window*)hw;
	set_rect( &wnd->rt, x, y, x+w, y+h );
	if( repaint )
		draw_window( (hwnd)wnd->parent );
}

void set_window_text( hwnd w, char* text )
{
	window* pw = (window*)w;
	strcpy( pw->name, text );
}

int get_window_text( hwnd w, char* text, int count )
{
	int len;
	window* pw = (window*)w;
	len = strlen( pw->name );
	if( count < len ) len = count;
	strcpy( text, pw->name );
	return len;
}

int get_message( msg* m, hwnd w )
{
	msg* _m = (msg*)msgs.head;
	if( _m )
	{
		blist_remove( &msgs, _m );
		*m = *_m;
		k_free( _m );
	}
	else
	{
		// empty
		m->wnd = 0;
		m->message = wm_idle;
	}

	return 1;
}

long dispatch_message( msg* m )
{
	if( !m || !m->wnd )
		return 0;
	return send_message( m->wnd, m->message, m->wparam, m->lparam );
}

int _nc_hit_test( hwnd hw, int x, int y )
{
	window* w = (window*)hw;
	int width = w->rt.right-w->rt.left;
	int height = w->rt.bottom-w->rt.top;

	if( x < 0 || y < 0 || x > width || y > height )
		return 0;

	if( x < 4 && y < 4 )
		return ht_topleft;
	if( x > width-4 && y < 4 )
		return ht_topright;
	if( x < 4 && y > height-4 )
		return ht_bottomleft;
	if( x > width-4 && y > height-4 )
		return ht_bottomright;

	if( x < 4 )
		return ht_left;
	if( x > width-4 )
		return ht_right;
	if( y < 4 )
		return ht_top;
	if( y > height-4 )
		return ht_bottom;

	if( w->ws_style & ws_caption )
	{
		int ht;
		ht = hittest_caption_buttons( hw, x, y );
		if( ht )
			return ht;

		if( rect_in( &w->rtcaption, x, y ) )
		{
			return ht_caption;
		}
	}

	if( rect_in( &w->rtclient, x, y ) )
		return ht_client;
	return 0;
}

int default_window_proc( hwnd wnd, unsigned long message, unsigned long wparam, unsigned long lparam )
{
	window* w = (window*)wnd;
	switch( message )
	{
	case wm_erasebkgnd:
		draw_window_client( wnd );
		break;
	case wm_nchittest:
		return _nc_hit_test( wnd, LOWORD(lparam), LOWORD(lparam) );
	case wm_ncpaint:
		// draw non-client area
		draw_window_frame( wnd );
		break;
	case wm_syscommand:
		{
		switch( wparam )
		{
		case sc_close:
			destroy_window(wnd);
			break;
		}
		}
		break;
	case wm_nclbuttondown:
		{
			if( gui.pressed_hitest & ht_close )
			{
				hdc dc = get_window_dc( wnd );
				w->close_pressed = 1;
				draw_caption_buttons( wnd, dc );
				release_dc( dc );
			}
			else
			if( gui.pressed_hitest & ht_minimize )
			{
				hdc dc = get_window_dc( wnd );
				w->minimize_pressed = 1;
				draw_caption_buttons( wnd, dc );
				release_dc( dc );
			}
		}
		break;
	case wm_nclbuttonup:
		{
			window* pressed_wnd = (window*)gui.pressed_wnd;
			int up_hitest;
			int dx,dy;
			point pt;
			pt.x = LOWORD(wparam);
			pt.y = HIWORD(wparam);
			screen_to_client( gui.pressed_wnd, &pt );
			pt.x += pressed_wnd->rtclient.left;
			pt.y += pressed_wnd->rtclient.top;

			dx = pt.x - gui.pressed_pt.x;
			dy = pt.y - gui.pressed_pt.y;

			up_hitest = _nc_hit_test( gui.pressed_wnd, pt.x, pt.y );
			if( gui.pressed_hitest & ht_close )
			{
				hdc dc = get_window_dc( wnd );
				w->close_pressed = 0;
				draw_caption( wnd, dc, &w->rtcaption );
				if( up_hitest & ht_close )
				{
					post_message( wnd, wm_syscommand, sc_close, wparam );
				}
				release_dc( dc );
			}
			if( gui.pressed_hitest & ht_minimize )
			{
				hdc dc = get_window_dc( wnd );
				w->minimize_pressed = 0;
				draw_caption( wnd, dc, &w->rtcaption );
				if( up_hitest & ht_minimize )
				{
					show_window( wnd, 0 );
				}
			}
			else
			if( dx || dy )
			{
				if( gui.pressed_hitest & ht_caption )
					move_window( wnd, w->rt.left+dx, w->rt.top+dy, w->rt.right-w->rt.left, w->rt.bottom-w->rt.top, 1 );
				else 
				if( gui.pressed_hitest & ht_top )
					move_window( wnd, w->rt.left, w->rt.top+dy, w->rt.right-w->rt.left, w->rt.bottom-w->rt.top-dy, 1 );
				else 
				if( gui.pressed_hitest & ht_left )
					move_window( wnd, w->rt.left+dx, w->rt.top, w->rt.right-w->rt.left-dx, w->rt.bottom-w->rt.top, 1 );
				else 
				if( gui.pressed_hitest & ht_topleft )
					move_window( wnd, w->rt.left+dx, w->rt.top+dy, w->rt.right-w->rt.left-dx, w->rt.bottom-w->rt.top-dy, 1 );
				else
				if( gui.pressed_hitest & ht_topright )
					move_window( wnd, w->rt.left, w->rt.top+dy, w->rt.right-w->rt.left+dx, w->rt.bottom-w->rt.top-dy, 1 );
				else
				if( gui.pressed_hitest & ht_right )
					move_window( wnd, w->rt.left, w->rt.top, w->rt.right-w->rt.left+dx, w->rt.bottom-w->rt.top, 1 );
				else
				if( gui.pressed_hitest & ht_bottomright )
					move_window( wnd, w->rt.left, w->rt.top, w->rt.right-w->rt.left+dx, w->rt.bottom-w->rt.top+dy, 1 );
				else
				if( gui.pressed_hitest & ht_bottomleft )
					move_window( wnd, w->rt.left+dx, w->rt.top, w->rt.right-w->rt.left-dx, w->rt.bottom-w->rt.top+dy, 1 );
				else
				if( gui.pressed_hitest & ht_bottom )
					move_window( wnd, w->rt.left, w->rt.top, w->rt.right-w->rt.left, w->rt.bottom-w->rt.top+dy, 1 );
			}
		}
		break;
	}
	
	return 1;
}


void destroy_window( hwnd w )
{
	//window* wnd = (window*)w;
	
	send_message( gui.taskbar, wm_taskcommand, 0, w );
	show_window( w, 0 );
	
/*	
	_detach_window( wnd );
	if( wnd->extra )
		free( wnd->extra );
	free(wnd);
*/
}

void get_client_rect( hwnd w, rect* rt )
{
	window* wnd = (window*)w;
	rt->left = 0;
	rt->top = 0;
	rt->right = wnd->rtclient.right - wnd->rtclient.left;
	rt->bottom = wnd->rtclient.bottom - wnd->rtclient.top;
}

void get_window_rect( hwnd w, rect* rt )
{
	window* wnd = (window*)w;
	*rt = wnd->rt;
	client_to_screen_rt( w, rt );
}

void draw_window( hwnd w )
{
	window* wnd = (window*)w;
	window* child;
	
	if( wnd->showhide == 0 ) // hide
		return;
	
	// draw non-client area
	post_message( w, wm_ncpaint, 0, 0 );
	// erase client area
	post_message( w, wm_erasebkgnd, 0, 0 );
	// draw non-client area
	post_message( w, wm_paint, 0, 0 );
	
	// draw child window
	child = wnd->child;
	while( child )
	{
		draw_window( (hwnd)child );

		//draw_window_client( (hwnd)child );
		child = child->next;
	}
}

void draw_window_client( hwnd w )
{
	rect rt;
	window* wnd = (window*)w;
	hdc dc = get_window_dc( w );
	rt = wnd->rtclient;
	// draw client
	gdi_rect( dc, &rt, _RGB(255,255,255) , 1 );
	release_dc( dc );
}

void draw_rect( rect rt, COLOR cr, int fill )
{
	int y;
	if( fill )
	{
		for( y = rt.top ; y < rt.bottom ; y++ )
			gui.line( rt.left, y, rt.right, y, cr );
	}
	else
	{
		gui.line( rt.left, rt.top, rt.right, rt.top, cr );
		gui.line( rt.left, rt.bottom, rt.right, rt.bottom, cr );
		gui.line( rt.left, rt.top, rt.left, rt.bottom, cr );
		gui.line( rt.right, rt.top, rt.right, rt.bottom, cr );
	}
}

void gdi_draw_frame( hdc dc, rect rt, COLOR cr1, COLOR cr2, int up )
{
	if( !up )
	{
		int tmp = cr1;
		cr1 = cr2;
		cr2 = tmp;
	}
	gdi_line( dc, rt.left, rt.top, rt.right, rt.top, cr1 );
	gdi_line( dc, rt.left, rt.bottom, rt.right, rt.bottom, cr2 );
	gdi_line( dc, rt.left, rt.top, rt.left, rt.bottom, cr1 );
	gdi_line( dc, rt.right, rt.top, rt.right, rt.bottom, cr2 );

/*
	COLOR crBright, crDark, cr1, cr2;

	crBright = _RGB( COLOR_R(cr)+30, COLOR_G(cr)+30, COLOR_B(cr)+30 );
	crDark = _RGB( COLOR_R(cr)-30, COLOR_G(cr)-30, COLOR_B(cr)-30 );

	if( up )
	{
		cr1 = cr_1;
		cr2 = cr_2;
	}
	else
	{
		cr1 = cr_2;
		cr2 = c;
	}

	gdi_line( dc, rt.left, rt.top, rt.right, rt.top, cr1 );
	gdi_line( dc, rt.left, rt.bottom, rt.right, rt.bottom, cr2 );
	gdi_line( dc, rt.left, rt.top, rt.left, rt.bottom, cr1 );
	gdi_line( dc, rt.right, rt.top, rt.right, rt.bottom, cr2 );	
	*/
}

void draw_window_frame( hwnd w )
{
	window* wnd = (window*)w;
	hdc dc = get_window_dc( w );

	rect rt;
	get_window_rect( w, &rt );
	rt.right = rt.right - rt.left;
	rt.bottom = rt.bottom - rt.top;
	rt.left = 0;
	rt.top = 0;

	gdi_draw_frame( dc, rt, _RGB(212,212,212), _RGB(64,64,64), 1 );
	rect_smaller( &rt, 1 );
	gdi_draw_frame( dc, rt, _RGB(255,255,255), _RGB(128,128,128), 1 );
	rect_smaller( &rt, 1 );
	gdi_rect( dc, &rt, _RGB(212,208,200), 0 );
	rect_smaller( &rt, 1 );
	gdi_rect( dc, &rt, _RGB(212,208,200), 0 );
	rect_smaller( &rt, 1 );

	// calc client rect
	wnd->rtclient = rt;

	if( wnd->ws_style & ws_caption )
	{
		// draw caption
		wnd->rtcaption = rt;
		wnd->rtcaption.bottom = wnd->rtcaption.top+20;
		//wnd->rtcaption.right;// -= 18;
		draw_caption( w, dc, &wnd->rtcaption );
		wnd->rtclient.top += 20;
	}
	
	release_dc( dc );
}

int hittest_caption_buttons( hwnd w, int x, int y )
{
	window* pw = (window*)w;
	rect rtCtl;
	point pt;
	
	rtCtl = pw->rtcaption;
	rtCtl.left = pw->rtcaption.right-2 - 16*2;
	rtCtl.right = rtCtl.left + 16;
	rtCtl.top = pw->rtcaption.top+2;
	rtCtl.bottom = rtCtl.top+16;

	if( pw->ws_style & ws_minimize )
	{
		if( rect_in( &rtCtl, x, y ) )
			return ht_minimize;
		
		pt.x = 16; pt.y = 0;
		rect_move( &rtCtl, &pt );	
		
		if( rect_in( &rtCtl, x, y ) )
			return ht_close;
	}
	return 0;
}

void draw_caption_buttons( hwnd w, hdc dc )
{
	window* pw = (window*)w;
	rect rtCtl;
	
	rtCtl = pw->rtcaption;
	rtCtl.left = pw->rtcaption.right-2 - 16*2;
	rtCtl.right = rtCtl.left + 16;
	rtCtl.top = pw->rtcaption.top+2;
	rtCtl.bottom = rtCtl.top+16;

	if( pw->ws_style & ws_minimize )
	{
		// draw control
		draw_button( w, dc, "-", &rtCtl, pw->minimize_pressed );
	}

	if( pw->ws_style & ws_close )
	{
		// draw control
		point pt;
		pt.x = 16; pt.y = 0;
		rect_move( &rtCtl, &pt );
		draw_button( w, dc, "X", &rtCtl, pw->close_pressed );
	}
}

void draw_caption( hwnd w, hdc dc, rect* rt )
{
	window* wnd = (window*)w;
	COLOR crText, crCaption;
	rect rtText;

	if( wnd->showhide == 0 )
		return;
	if( w == gui.active )
		crCaption = _RGB(20,20,160);
	else
		crCaption = _RGB(100,100,100);

	gdi_rect( dc, rt, crCaption, 1 );


	rtText = *rt;
	rtText.right -= (16*2+2);

	// draw caption text
	crText = _RGB( 255,255,255 );
	gdi_draw_text( dc, wnd->name, &rtText, &crText, 0 );

	// draw buttons
	draw_caption_buttons( w, dc );
}

typedef struct _font_data
{
	hdc dc;
	rect* prtClip;
	COLOR* pcrText;
	COLOR* pcrBack;
}font_data;

void gui_font_setpixel( int x, int y, int put, void* data )
{
	font_data* fd = (font_data*)data;
	//device_context* pdc = (device_context*)fd->dc;

	// clipping
	if( fd->prtClip && rect_in( fd->prtClip, x, y ) == 0 )
		return;

	if( put )
	{
		if( fd->pcrText )
			gdi_setpixel( fd->dc, x, y, *fd->pcrText );
	}
	else
	{
		if( fd->pcrBack ) 
			gdi_setpixel( fd->dc, x, y, *fd->pcrBack );
	}
}

void gdi_draw_text( hdc dc, char* s, rect* rt, COLOR* pcrText, COLOR* pcrBack )
{
	font_data fd;
	fd.dc = dc;
	fd.prtClip = rt;
	fd.pcrText = pcrText;
	fd.pcrBack = pcrBack;

	gputs( s, rt->left, rt->top, &fd );
}

void gdi_circle( hdc dc, int xc, int yc, int r, COLOR cr )
{
	int xk = 0;
	int yk = 0 + r;
	int prev = 3 - 2*r;
	int next = 0;
	
	gdi_setpixel( dc, xc+xk, yc+yk, cr );
	gdi_setpixel( dc, xc+xk, yc-yk, cr );
	gdi_setpixel( dc, xc-xk, yc-yk, cr );
	gdi_setpixel( dc, xc-xk, yc+yk, cr );
	gdi_setpixel( dc, xc+yk, yc+xk, cr );
	gdi_setpixel( dc, xc+yk, yc-xk, cr );
	gdi_setpixel( dc, xc-yk, yc-xk, cr );
	gdi_setpixel( dc, xc-yk, yc+xk, cr );
	while(xk < yk)
	{
		if(prev < 0)
		{
			next = prev + 4*xk + 6;
			xk++;
		}
		else
		{
			next = prev + 4*(xk-yk) + 10;
			xk++;
			yk--;
		}
		gdi_setpixel( dc, xc+xk, yc+yk, cr );
		gdi_setpixel( dc, xc+xk, yc-yk, cr );
		gdi_setpixel( dc, xc-xk, yc-yk, cr );
		gdi_setpixel( dc, xc-xk, yc+yk, cr );
		gdi_setpixel( dc, xc+yk, yc+xk, cr );
		gdi_setpixel( dc, xc+yk, yc-xk, cr );
		gdi_setpixel( dc, xc-yk, yc-xk, cr );
		gdi_setpixel( dc, xc-yk, yc+xk, cr );

		prev = next;
	}
}

/////////////////////////////////////////////////////////////////////////////
// common window function

void show_window( hwnd w, int sw )
{
	window* wnd = (window*)w;
	
	wnd->showhide = sw;

	if( sw == 1 )
	{
		set_active_window( w );
	}
	else
	if( sw == 0 )
	{
		w = get_top_window( w );
		if( gui.active == w )
		{
			// active low z-order window
			if( wnd->prev )
				set_active_window( (hwnd)wnd->prev );
			else
				set_active_window( gui.desktop );
		}
		
		draw_window( gui.desktop );
	}
}

void _taskbar_paint_buttons( hwnd taskbar );

void set_active_window( hwnd w )
{
	hwnd tw, prev;

	prev = gui.active;

	if( w != gui.desktop )
	{
		tw = get_top_window( w );
		if( gui.active == tw )
			return;
		gui.active = tw;
		_set_last_sibling_window( (window*)tw );
	}
	else
		gui.active = w;

	if( prev != gui.desktop && prev != gui.taskbar )
	{
		window* wnd_prev = (window*)prev;
		hdc dc = get_window_dc( prev );
		draw_caption( prev, dc, &wnd_prev->rtcaption );
	}

	if( gui.active != gui.desktop && gui.active != gui.taskbar )
	{
		draw_window( gui.active );
		_taskbar_paint_buttons( gui.taskbar );
	}
}

////////////////////////////////////////////////////////////////////////////
// menu
hmenu create_menu(void);
hmenu create_menu()
{
	menu* pm = (menu*)k_malloc( sizeof(menu) );
	memset( pm, 0, sizeof(menu) );
	return (hmenu)pm;
}

int append_menu( hmenu m, unsigned long flag, int command, char* name )
{
	menu* pm = (menu*)m;
	
	menuitem* mi = k_malloc( sizeof(menuitem) );
	if( mi == 0 )
		return 0;
	memset( mi, 0, sizeof(menuitem) );
	mi->command = command;
	mi->flag = flag;
	strcpy( mi->name, name );
	
	blist_add_tail( &pm->menuitems, &mi->node );
	return 1;
}

void destroy_menu( hmenu m )
{
	if( m )
	{
		menu* pm = (menu*)m;
		menuitem* mi = (menuitem*)pm->menuitems.head;
		while( mi )
		{
			blist_remove( &pm->menuitems, mi );
			k_free( mi );
			mi = (menuitem*)pm->menuitems.head;
		}
	
		k_free( (void*)m );
	}
}

void _menu_paint( hwnd hw, hdc dc )
{
	window* pw = (window*)hw;
	menu* pm = (menu*)pw->menu;
	rect rt = pw->rtclient;

	COLOR cr;
	rect rtitem;
	point pt;

	menuitem* mi = (menuitem*)pm->menuitems.head;
	set_rect( &rtitem, rt.left, rt.top, rt.right, 20 );
	pt.x = 0; pt.y = 20;
	cr = _RGB(0,0,0 );
	while( mi )
	{
		gdi_draw_text( dc, mi->name, &rtitem, &cr, 0 );
		rect_move( &rtitem, &pt );
		mi = (menuitem*)mi->node.next;
	}
}

int _menu_hittest( hwnd hw, int x, int y, rect* prtitem, int* pnItem, menuitem** ppmi )
{
	window* pw = (window*)hw;
	menu* pm = (menu*)pw->menu;
	rect rt = pw->rtclient;
	int nItem = 0;

	COLOR cr;
	rect rtitem;
	point pt;

	menuitem* mi = (menuitem*)pm->menuitems.head;
	set_rect( &rtitem, rt.left, rt.top, rt.right, 20 );
	pt.x = 0; pt.y = 20;
	cr = _RGB(0,0,0 );
	while( mi )
	{
		if( rect_in( &rtitem, x, y ) )
		{
			*prtitem = rtitem;
			*pnItem = nItem;
			*ppmi = mi;
			return 1;
		}
		rect_move( &rtitem, &pt );
		mi = (menuitem*)mi->node.next;
		nItem++;
	}
	return 0;
}

int menu_window_proc( hwnd wnd, unsigned long message, unsigned long wparam, unsigned long lparam )
{
	switch( message )
	{
	case wm_ncpaint:
		{
		window* w = (window*)wnd;
		//menu* pm = (menu*)w->menu;
		hdc dc = get_window_dc( wnd );

		rect rt;
		get_window_rect( wnd, &rt );
		rt.right = rt.right - rt.left;
		rt.bottom = rt.bottom - rt.top;
		rt.left = 0;
		rt.top = 0;

		gdi_draw_frame( dc, rt, _RGB(212,212,212), _RGB(64,64,64), 1 );
		rect_smaller( &rt, 1 );
		gdi_draw_frame( dc, rt, _RGB(255,255,255), _RGB(128,128,128), 1 );
		rect_smaller( &rt, 1 );
		gdi_rect( dc, &rt, _RGB(212,208,200), 0 );
		rect_smaller( &rt, 1 );
		gdi_rect( dc, &rt, _RGB(212,208,200), 0 );
		rect_smaller( &rt, 1 );

		gdi_rect( dc, &rt, _RGB(212,208,200), 1 );
		// calc client rect
		w->rtclient = rt;

		_menu_paint( wnd, dc );
		release_dc( dc );
		}
		break;
	case wm_lbuttonup:
		{
		rect rtitem;
		int item;
		menuitem* mi;
		point pt;
		window* pw = (window*)gui.menu;
		pt.x = LOWORD(wparam);
		pt.y = HIWORD(wparam);
		
		if( _menu_hittest( wnd, pt.x, pt.y, &rtitem, &item, &mi ) )
		{
			menu* pm = (menu*)pw->menu;
			post_message( (hwnd)pm->owner, wm_command, MAKEWPARAM(0,mi->command), 0 );
			show_window( wnd, 0 );
			gui.menu = 0;
		}
		}
		break;
	case wm_paint:
		{
		}
		break;
	}

	return 0;
}

int _menu_size( hmenu m, point* pt )
{
	menu* pm = (menu*)m;
	int len, max_len = 0;
	int i = 0;

	menuitem* mi = (menuitem*)pm->menuitems.head;
	while( mi )
	{
		i++;
		len = strlen( mi->name );
		if( max_len < len ) 
			max_len = len;
		mi = (menuitem*)mi->node.next;
	}

	pt->x = max_len*16;
	pt->y = i*20;
	return 1;
}

void set_menu( hwnd hw, hmenu hm )
{
	window* pw = (window*)hw;
	pw->menu = hm;
}

void track_popup_menu( hmenu m, int sx, int sy, hwnd hw )
{
	menu* pm = (menu*)m;
	point pt;
	_menu_size( m, &pt );
	if( sx + pt.x > gui.width )
		sx -= pt.x;
	if( sy + pt.y > gui.height )
		sy -= pt.y;

	if( !pm->hw_menu )
		pm->hw_menu = create_window("menu", "menu", 0, sx, sy, pt.x, pt.y, gui.desktop, 0 );
	show_window( pm->hw_menu, 1 );
	gui.menu = pm->hw_menu;
	set_menu( gui.menu, m );
	pm->owner = hw;
}

/////////////////////////////////////////////////////////////////////////////
// common window

void draw_button( hwnd w, hdc dc, char* text, rect* rt, int press )
{
	rect rtbt = *rt;
	COLOR crText = _RGB( 0,0,0 );

	gdi_draw_frame( dc, rtbt, _RGB(212,212,212), _RGB(64,64,64), press?0:1 );
	rect_smaller( &rtbt, 1 );
	gdi_draw_frame( dc, rtbt, _RGB(255,255,255), _RGB(128,128,128), press?0:1 );
	rect_smaller( &rtbt, 1 );

	gdi_rect( dc, &rtbt, _RGB(212,208,200), 1 );
	gdi_draw_text( dc, text, &rtbt, &crText, 0 );
}

extern unsigned short g_wallpaper[];
void put_image( unsigned short* image );

int desktop_window_proc( hwnd wnd, unsigned long message, unsigned long wparam, unsigned long lparam )
{
	//window* w = (window*)wnd;

	switch( message )
	{
	case wm_erasebkgnd:
		{
			
		 put_image( g_wallpaper );

			/*
		hdc dc = get_dc( wnd );
		rect rt;
		get_client_rect( wnd, &rt );
		gdi_rect( dc, &rt, _RGB(58,110,165), 1 );
		release_dc( dc );
		*/
		
		}
		break;
	}
	
	return 1;
}

int _create_desktop_window( struct _gui* gui )
{
	wndclass wc;
	memset( &wc, 0, sizeof(wndclass) );
	strcpy( wc.name, "desktop" );
	wc.winproc = desktop_window_proc;
	register_class( &wc );

	gui->desktop = create_window( "desktop", "my desktop", 0, 0, 0, gui->width, gui->height, -1, 0 );
	if( !gui->desktop )
		return 0;
	
	return 1;
}

void _taskbar_draw_buttons( hwnd taskbar, rect* rt )
{
	window* pw = (window*)taskbar;
	taskbar_extra* data = (taskbar_extra*)pw->extra;
	taskinfo* ti = (taskinfo*)data->taskinfos.head;
	int task_count, task_button_width;
	rect rtTask;
	hdc dc;
	point pt;

	task_count = blist_count( &data->taskinfos );
	if( task_count == 0 )
		return;
	dc = get_dc( taskbar );
	task_button_width = (rt->right-rt->left)/task_count;
	if( task_button_width > 150 )
		task_button_width = 150;
	set_rect( &rtTask, rt->left, rt->top, rt->left+task_button_width ,rt->bottom );
	pt.x = task_button_width; pt.y = 0;
	while( ti )
	{
		window* ptask = (window*)ti->hw;
		
		draw_button( taskbar, dc, ptask->name, &rtTask, ti->hw==gui.active );
		rect_move( &rtTask, &pt );
		ti = (taskinfo*)ti->node.next;
	}
	
	release_dc( dc );
}

void _taskbar_paint_buttons( hwnd taskbar )
{
	rect rt;
	get_client_rect( taskbar, &rt );
	rt.left += 60;
	_taskbar_draw_buttons( taskbar, &rt );
}

int _taskbar_hittest_buttons( hwnd taskbar, rect* rt, point* pthit, rect* rtbt, hwnd* pwtask )
{
	window* pw = (window*)taskbar;
	taskbar_extra* data = (taskbar_extra*)pw->extra;
	taskinfo* ti = (taskinfo*)data->taskinfos.head;
	int task_count, task_button_width;
	rect rtTask;
	point pt;

	task_count = blist_count( &data->taskinfos );
	if( task_count == 0 )
		return 0;
	task_button_width = (rt->right-rt->left)/task_count;
	if( task_button_width > 150 )
		task_button_width = 150;
	set_rect( &rtTask, rt->left, rt->top, rt->left+task_button_width ,rt->bottom );
	pt.x = task_button_width; pt.y = 0;
	while( ti )
	{
		//window* ptask = (window*)ti->hw;
		if( rect_in( &rtTask, pthit->x, pthit->y ) )
		{
			*rtbt = rtTask;
			*pwtask = ti->hw;
			return 1;
		}
		rect_move( &rtTask, &pt );
		ti = (taskinfo*)ti->node.next;
	}
	
	return 0;
}


hwnd create_sensor_window(void );
hwnd create_control_window(void);
hwnd create_power_window(void);

int taskbar_window_proc( hwnd wnd, unsigned long message, unsigned long wparam, unsigned long lparam )
{
	window* w = (window*)wnd;
	taskbar_extra* data = (taskbar_extra*)w->extra;

	switch( message )
	{
	case wm_create:
		{
			create_window("button","시작",ws_visible|ws_child, 0,0,50,20, wnd, 0 );
			w->menu = create_menu();
			append_menu( w->menu, 0, 11, "센서 App" );
			append_menu( w->menu, 0, 12, "전력 제어 App" );
			append_menu( w->menu, 0, 13, "IrDA 제어 App" );
			append_menu( w->menu, 0, 14, "터치 설정" );
			append_menu( w->menu, 0, 10, "시스템 종료..." );
		}
		break;
	case wm_destroy:
		{
			if(w->menu)
				destroy_menu(w->menu);
		}
		break;
	case wm_command:
		{
		int ctl_id = LOWORD( wparam );
		int cmd = HIWORD( wparam );
		switch( cmd )
		{
		case 11:
			kernel_printf("create_sensor_wndow()\n");
//			create_sensor_window();
			break;
		case 12:
			kernel_printf("create_power_wndow()\n");
//			create_power_window();
			break;
		case 13:
			kernel_printf("create_control_wndow()\n");
//			create_control_window();
			break;
		case bn_clicked:
			if( ctl_id == 0 ) // 시작
			{
				rect rt;
				get_window_rect( wnd, &rt );
				
				track_popup_menu( w->menu, 0, rt.top, wnd );
			}
			break;
		}	
		}
		break;
	case wm_ncpaint:
		{
		window* w = (window*)wnd;
		hdc dc = get_window_dc( wnd );

		rect rt;
		get_window_rect( wnd, &rt );
		rt.right = rt.right - rt.left;
		rt.bottom = rt.bottom - rt.top;
		rt.left = 0;
		rt.top = 0;

		gdi_draw_frame( dc, rt, _RGB(212,212,212), _RGB(64,64,64), 1 );
		rect_smaller( &rt, 1 );
		gdi_draw_frame( dc, rt, _RGB(255,255,255), _RGB(128,128,128), 1 );
		rect_smaller( &rt, 1 );
		gdi_rect( dc, &rt, _RGB(212,208,200), 0 );
		rect_smaller( &rt, 1 );
		gdi_rect( dc, &rt, _RGB(212,208,200), 0 );
		rect_smaller( &rt, 1 );

		gdi_rect( dc, &rt, _RGB(212,208,200), 1 );
		// calc client rect
		w->rtclient = rt;
		release_dc( dc );
		}
		break;
	case wm_lbuttondown:
		{
			rect rtbt;
			hwnd task;
			point pt;
			rect rt;
			get_client_rect( wnd, &rt );
			rt.left += 60;
			pt.x = LOWORD(wparam);
			pt.y = HIWORD(wparam);

			if( _taskbar_hittest_buttons( wnd, &rt, &pt, &rtbt, &task ) )
			{
				window* ptask = (window*)task;
				hdc dc = get_dc( wnd );
				draw_button( wnd, dc, ptask->name, &rtbt, 1 );
				release_dc( dc );
			}
		}
		break;
	case wm_lbuttonup:
		{
			rect rtbt;
			hwnd task;
			point pt;
			rect rt;
			get_client_rect( wnd, &rt );
			rt.left += 60;
			pt.x = LOWORD(wparam);
			pt.y = HIWORD(wparam);

			if( _taskbar_hittest_buttons( wnd, &rt, &pt, &rtbt, &task ) )
			{
				show_window( task, 1 );
				//set_active_window( task );
			}
			else
				_taskbar_paint_buttons( wnd );
		}
		break;
	case wm_paint:
		{
			_taskbar_paint_buttons( wnd );
		}
		break;
	case wm_taskcommand:
		{
		hwnd hw = lparam;
		window* pw = (window*)hw;
		
		if( pw->wndcls->winproc == menu_window_proc )
			break;
			
		if( wparam == 1 ) // create
		{
			taskinfo* ti;
			ti = (taskinfo*)k_malloc( sizeof(taskinfo) );
			if( ti )
			{
				memset( ti, 0, sizeof(taskinfo) );
				ti->hw = hw;
				blist_add_tail( &data->taskinfos, ti );
			}
		}
		else// destroy
		{
			taskinfo* ti = (taskinfo*)data->taskinfos.head;
			while( ti )
			{
				if( ti->hw == hw )
					break;
				ti = (taskinfo*)ti->node.next;
			}
			//if( blist_exist( &data->taskinfos, pw ) )
			if( ti )
				blist_remove( &data->taskinfos, ti );
		}
		_taskbar_paint_buttons( wnd );
		}
		break;
	}
	
	return 1;
}

int _create_taskbar_window( struct _gui* gui )
{
	wndclass wc;
	memset( &wc, 0, sizeof(wndclass) );
	strcpy( wc.name, "taskbar" );
	wc.winproc = taskbar_window_proc;
	wc.cb_wnd_extra = sizeof(taskbar_extra);
	register_class( &wc );

	gui->taskbar = create_window( "taskbar", "my taskbar", 0, 0, gui->height-26, gui->width, 26, gui->desktop, 0 );
	if( !gui->taskbar )
		return 0;
	
	return 1;
}

void _button_paint( hwnd w, int press )
{
	window* wnd = (window*)w;
	hdc dc = get_dc( w );
	rect rt;
	get_client_rect( w, &rt );
	draw_button( w, dc, wnd->name, &rt, press );
	release_dc( dc );
}

int button_window_proc( hwnd wnd, unsigned long message, unsigned long wparam, unsigned long lparam )
{
	window* w = (window*)wnd;
	button_extra* data = (button_extra*)w->extra;

	switch( message )
	{
	case wm_paint:
		_button_paint( wnd, data->pressed );
		break;
	case wm_nclbuttondown:
	case wm_lbuttondown:
		data->pressed = 1;
		send_message( wnd, wm_paint, 0, 0 );
		break;
	case wm_nclbuttonup:
		{
		window* pressed_wnd = (window*)gui.pressed_wnd;
		int up_hitest;
		point pt;
		pt.x = LOWORD(wparam);
		pt.y = HIWORD(wparam);
		screen_to_client( gui.pressed_wnd, &pt );
		pt.x += pressed_wnd->rtclient.left;
		pt.y += pressed_wnd->rtclient.top;

		up_hitest = _nc_hit_test( gui.pressed_wnd, pt.x, pt.y );
		
		data->pressed = 0;
		send_message( wnd, wm_paint, 0, 0 );

		if( up_hitest )
			post_message( (hwnd)w->parent, wm_command, MAKEWPARAM(w->wid,bn_clicked), (unsigned long)wnd );
		}
		break;
	case wm_lbuttonup:
		{
		int up_hitest;
		point pt;
		pt.x = LOWORD(wparam);
		pt.y = HIWORD(wparam);
		up_hitest = _nc_hit_test( gui.pressed_wnd, pt.x, pt.y );
		
		data->pressed = 0;
		send_message( wnd, wm_paint, 0, 0 );

		if( up_hitest )
			post_message( (hwnd)w->parent, wm_command, MAKEWPARAM(w->wid,bn_clicked), (unsigned long)wnd );
		}
		break;
	}
	
	return 1;
}

int static_window_proc( hwnd wnd, unsigned long message, unsigned long wparam, unsigned long lparam )
{
	window* w = (window*)wnd;
	
	switch( message )
	{
	case wm_paint:
		{
		COLOR cr;
		hdc dc = get_dc( wnd );
		rect rt;
		get_client_rect( wnd, &rt );
		gdi_rect( dc, &rt, _RGB(255,255,255), 1 );
		cr = _RGB(0,0,0);
		gdi_draw_text( dc, w->name, &rt, &cr, 0 );
		release_dc( dc );
		}
		break;
	}
	
	return 1;
}

void gui_mouse_event_lb_down( int x, int y )
{
	hwnd w;
	window* wnd;
	// find window and active it
	point pt, ptc;
	pt.x = x; pt.y = y;

	kernel_printf("button down x : %d , y : %d \n", x, y);
	// pre-process for menu
	if( gui.menu )
	{
		window* pw = (window*)gui.menu;
		if( !rect_in( &pw->rt, x, y ) )
		{
			show_window( gui.menu, 0 );
			gui.menu = 0;
		}
		else
			return;
	}

	w = window_from_point( pt );
	set_active_window( w );

	// l-button down event fire
	wnd = (window*)w;
	screen_to_client( w, &pt );
	ptc = pt;
	ptc.x += wnd->rtclient.left;
	ptc.y += wnd->rtclient.top;

	// gui pressed info
	gui.pressed_wnd = w;
	gui.pressed_hitest = _nc_hit_test( w, ptc.x, ptc.y );
	gui.pressed_pt = ptc;

	if( rect_in( &wnd->rtclient, ptc.x, ptc.y ) )
	{
		// client click
		post_message( w, wm_lbuttondown, MAKEWPARAM( ptc.x, ptc.y ), 0 );
	}
	else
	{
		// non-client click
		post_message( w, wm_nclbuttondown, MAKEWPARAM( x, y ), 0 );
	}
}

void gui_mouse_event_lb_up( int x, int y )
{
	window* wnd = (window*)gui.pressed_wnd;
	point pt;
	pt.x = x; pt.y = y;
	
	kernel_printf("button up x : %d , y : %d \n", x, y);
	// pre-process for menu
	if( gui.menu )
	{
		window* pw = (window*)gui.menu;
		if( rect_in( &pw->rt, pt.x, pt.y ) )
		{
			screen_to_client( gui.menu, &pt );
			pt.x += pw->rtclient.left;
			pt.y += pw->rtclient.top;
			send_message( gui.menu, wm_lbuttonup, MAKEWPARAM( pt.x, pt.y ), 0 );
		}
		return;
	}

	screen_to_client( gui.pressed_wnd, &pt );
	pt.x += wnd->rtclient.left;
	pt.y += wnd->rtclient.top;

	if( gui.pressed_hitest & ht_client )
	{
		// client click
		post_message( gui.pressed_wnd, wm_lbuttonup, MAKEWPARAM( pt.x, pt.y ), 0 );
	}
	else
	{
		// non-client click
		post_message( gui.pressed_wnd, wm_nclbuttonup, MAKEWPARAM( x, y ), 0 );
	}
}

void start_gui(void);
void start_gui()
{
	msg m;
	while( get_message( &m, 0 ) )
	{
		dispatch_message( &m );
	}
}

void init_gui( int screen_width, int screen_height, int bpp,
			  draw_setpixel dsp, draw_line line )
{
	memset( &msgs, 0, sizeof(msgs) );
	memset( &wndclss, 0, sizeof( wndclss) );
	memset( &gui, 0, sizeof(gui) );

	gui.width = screen_width;
	gui.height = screen_height;
	gui.bpp = bpp;
	
	gui.setpixel = dsp;
	if( line )
		gui.line = line;
	else
		gui.line = _gui_line;

	init_font( gui_font_setpixel );

	// common window class register
	{
	wndclass wc;
	memset( &wc, 0, sizeof(wndclass) );
	strcpy( wc.name, "default" );
	wc.winproc = default_window_proc;
	register_class( &wc );
	}

	{
	wndclass wc;
	memset( &wc, 0, sizeof(wndclass) );
	strcpy( wc.name, "button" );
	wc.winproc = button_window_proc;
	wc.cb_wnd_extra = sizeof(button_extra);
	register_class( &wc );
	}

	{
	wndclass wc;
	memset( &wc, 0, sizeof(wndclass) );
	strcpy( wc.name, "static" );
	wc.winproc = static_window_proc;
	register_class( &wc );
	}

	{
	wndclass wc;
	memset( &wc, 0, sizeof(wndclass) );
	strcpy( wc.name, "menu" );
	wc.winproc = menu_window_proc;
	register_class( &wc );
	}
	gui.menu = 0;

	// init desktop
	_create_desktop_window( &gui );
	// init taskbar
	_create_taskbar_window( &gui );

	gui.active = gui.desktop;
	gui.pressed_wnd = gui.desktop;
	gui.pressed_pt.x = gui.pressed_pt.y = 0;
	gui.pressed_hitest = ht_client;
}
