/*
*********************************************************************************************************
*                                   AESOP MP2530F Board Support Package
*
*  An Entertainment Solution On a Platform (AESOP) is a completely Open Source
*  based graphical user environment and suite of applications for PDAs and other
*  devices running Linux. It is included in various embedded Linux distributions
*  such as OpenZaurus - http://www.aesop-embedded.org
*
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  File       : BSP.H
*  By        	: jhkim(jhkim@libertron.com)
*  Version 		: v1.0
*********************************************************************************************************
*/
#define	BSP_MP2530F_CLK	    1000000L                         
                          
/*
*********************************************************************************************************
*                                  MP2530F Processor Interrupt Control Register
*********************************************************************************************************
*/

#define TYPE_IRQ 0
#define TYPE_FIQ 1

#define INTMODEL				(*(INT32U *)		0xC0000808)
#define INTMODEH				(*(INT32U *)		0xC000080C)
#define INTMASKL				(*(INT32U *)		0xC0000810)
#define INTMASKH				(*(INT32U *)		0xC0000814)
#define INTPRIORDER			(*(INT32U *)		0xC0000818)
#define INTPENDL				(*(INT32U *)		0xC0000820)
#define INTPENDH				(*(INT32U *)		0xC0000824)

#define INTL_DISPLAY		0
#define INTL_SDISPLAY		1
#define INTL_VIP				2
#define INTL_DMA				3
#define INTL_TIMER0			4
#define INTL_SYSCTRL		5
#define INTL_DUALCPUIF	6
#define INTL_MPEGIF			7
#define INTL_EXTINT0		8
#define INTL_EXTINT1		9
#define INTL_UART0			10
#define INTL_TIMER1			11
#define INTL_SSPSPI0		12
#define INTL_GPIO				13
#define INTL_SDMMC			14
#define INTL_TIMER2			15
#define INTL_H264				16
#define INTL_MPEGCODEC	17
#define INTL_GRP2D			18
#define INTL_VLC				19
#define INTL_UDC				20
#define INTL_TIMER3			21
#define INTL_DEINTERLACE 22
#define INTL_PPM				23
#define INTL_AUDIOIF		24
#define INTL_ADC				25
#define INTL_MCUSTATIC	26
#define INTL_GRP3D			27
#define INTL_UHC				28
#define INTL_ROTATOR		29
#define INTL_SCALER			30
#define INTL_RTC				31
#define INTH_I2C0				32
#define INTH_I2C1				33
#define INTH_UART1			34
#define INTH_UART2			35
#define INTH_UART3			36
#define INTH_UART4			37
#define INTH_UART5			38
#define INTH_SSPSPI1		39
#define INTH_SSPSPI2		40
#define INTH_CSC				41


#define PRIORITY_REQ0		0x0
#define PRIORITY_REQ1		0x1
#define PRIORITY_REQ2		0x2
#define PRIORITY_REQ3		0x3

/*
*********************************************************************************************************
*                                              UARTS
*********************************************************************************************************
*/

#define SYSTEM_CLOCK_UART_DIV		26
#define SYSTEM_DEBUG_UART_BAUD	115200

#define UART0_BASE				0xC0016000
#define UART1_BASE				0xC0016080
#define UART2_BASE				0xC0016800
#define UART3_BASE				0xC0016880
#define UART4_BASE				0xC0017000
#define UART5_BASE				0xC0017080

#define UART_BASE					0xC0016880

#define LCON					(*(volatile INT16U *) (UART_BASE + 0x00))
#define UCON					(*(volatile INT16U *) (UART_BASE + 0x02)) 
#define FCON					(*(volatile INT16U *) (UART_BASE + 0x04)) 
#define MCON 					(*(volatile INT16U *) (UART_BASE + 0x06)) 
#define TRSTATUS			(*(volatile INT16U *) (UART_BASE + 0x08)) 
#define ESTATUS				(*(volatile INT16U *) (UART_BASE + 0x0A)) 
#define FSTATUS				(*(volatile INT16U *) (UART_BASE + 0x0C)) 
#define MSTATUS				(*(volatile INT16U *) (UART_BASE + 0x0E)) 
#define THB						(*(volatile INT16U *) (UART_BASE + 0x10)) 
#define RHB						(*(volatile INT16U *) (UART_BASE + 0x12)) 
#define BRD						(*(volatile INT16U *) (UART_BASE + 0x14)) 
#define TIMEOUT				(*(volatile INT16U *) (UART_BASE + 0x16)) 
#define INTR					(*(volatile INT16U *) (UART_BASE + 0x18)) 
#define FCLK					(*(volatile INT16U *) (UART_BASE + 0x40)) 
#define UCLK					(*(volatile INT16U *) (UART_BASE + 0x44)) 

/*
*********************************************************************************************************
*                                              TIMER REGISTER
*********************************************************************************************************
*/
#define TMRCOUNT0		    (*(INT32U *)(0xC0001800))	// timer 0 counter register
#define TMRMATCH0		    (*(INT32U *)(0xC0001804))	// timer 0 match register
#define TMRCONTROL0		  (*(INT32U *)(0xC0001808))	// timer 0 control register
#define TMRCLKENB0		  (*(INT32U *)(0xC0001840))	// timer 0 clock generation enable register
#define TMRCLKGEN0		  (*(INT32U *)(0xC0001844))	// timer 0 clock generation control register

#define TMRCOUNT1		    (*(INT32U *)(0xC0001880))	// timer 1 counter register
#define TMRMATCH1		    (*(INT32U *)(0xC0001884))	// timer 1 match register
#define TMRCONTROL1		  (*(INT32U *)(0xC0001888))	// timer 1 control register
#define TMRCLKENB1		  (*(INT32U *)(0xC00018C0))	// timer 1 clock generation enable register
#define TMRCLKGEN1		  (*(INT32U *)(0xC00018C4))	// timer 1 clock generation control register

#define TMRCOUNT2		    (*(INT32U *)(0xC0001900))	// timer 2 counter register
#define TMRMATCH2		    (*(INT32U *)(0xC0001904))	// timer 2 match register
#define TMRCONTROL2		  (*(INT32U *)(0xC000190)8)	// timer 2 control register
#define TMRCLKENB2		  (*(INT32U *)(0xC0001940))	// timer 2 clock generation enable register
#define TMRCLKGEN2		  (*(INT32U *)(0xC0001944))	// timer 2 clock generation control register

#define TMRCOUNT3		    (*(INT32U *)(0xC0001980))	// timer 3 counter register
#define TMRMATCH3		    (*(INT32U *)(0xC0001984))	// timer 3 match register
#define TMRCONTROL3		  (*(INT32U *)(0xC0001988))	// timer 3 control register
#define TMRCLKENB3		  (*(INT32U *)(0xC00019C0))	// timer 3 clock generation enable register
#define TMRCLKGEN3		  (*(INT32U *)(0xC00019C4))	// timer 3 clock generation control register

// Timer Control Register( TMRCONTROL )
#define TMRCONTROL_READ_COUNTER	(1<<6)  
#define TMRCONTROL_INTPEND			(1<<5)  
#define TMRCONTROL_INTENB				(1<<4)
#define TMRCONTROL_RUN					(1<<3)
#define TMRCONTROL_WATCHDOG_ENB	(1<<2)
#define TMRCONTROL_SET_TCLOCK		(3<<0)
#define TMRCONTROL_TCLOCK_DIV2	(0<<0)  // TCLK = Source clock / 2
#define TMRCONTROL_TCLOCK_DIV4	(1<<0)  // TCLK = Source clock / 4
#define TMRCONTROL_TCLOCK_DIV8	(2<<0)  // TCLK = Source clock / 8
#define TMRCONTROL_TCLOCK_DIV1	(3<<0)  // TCLK = Source clock

// Timer Clock Gerneration Enable Register ( TMRCLKENB )
#define TMRCLKENB_TCLKMODE			(1<<3) 	
#define TMRCLKENB_CLKGENENB			(1<<2) 	

#define TMRCLKGEN_CLKDIV  			(0xff<<4)
#define TMRCLKGEN_CLKDIV_SHIFT	(4)

#define TMRCLKGEN_CLKDIV0				(32)

#define TMRCLKGEN_CLKSRCSEL			(7<<1)
#define TMRCLKGEN_CLKSRCSEL_SHIFT	(1)
#define TMRCLKGEN_CLKSRC_PLL0		(0<<1)
#define TMRCLKGEN_CLKSRC_PLL1		(1<<1)
#define TMRCLKGEN_CLKSRC_PLL2		(2<<1)

#define PLLSET_PDIV							(0xff<<16)
#define PLLSET_PDIV_SHIFT				(16)
#define PLLSET_MDIV							(0xff<<8)
#define PLLSET_MDIV_SHIFT				(8)
#define PLLSET_SDIV							(0xff<<0)
#define PLLSET_SDIV_SHIFT				(0)

#define PLLSETREG2			(*(INT32U *)(0xC000F00C))	


/*
*********************************************************************************************************
*                                          FUNCTION PROTOTYPES
*********************************************************************************************************
*/

void     BSP_Init(void);

/*
*********************************************************************************************************
*                                              INTERRUPTS
*********************************************************************************************************
*/

void     BSP_IntDisAll(void);

/*
*********************************************************************************************************
*                                             TICK SERVICES
*********************************************************************************************************
*/

void     Tmr_Init(void);
void     Tmr_TickISR(void);
void     Tmr_TickISR_Handler(void);
