#ifndef _LCD_H
#define _LCD_H

#include <types.h>

#define CKEN		__REG(0x41300004)  /* Clock Enable Register */
#define CKEN16_LCD	(1 << 16)	/* LCD Unit Clock Enable */

#define LCCR0		__REG(0x44000000)  /* LCD Controller Control Register 0 */
#define LCCR1		__REG(0x44000004)  /* LCD Controller Control Register 1 */
#define LCCR2		__REG(0x44000008)  /* LCD Controller Control Register 2 */
#define LCCR3		__REG(0x4400000C)  /* LCD Controller Control Register 3 */
#define DFBR0		__REG(0x44000020)  /* DMA Channel 0 Frame Branch Register */
#define DFBR1		__REG(0x44000024)  /* DMA Channel 1 Frame Branch Register */
#define LCSR		__REG(0x44000038)  /* LCD Controller Status Register */
#define LIIDR		__REG(0x4400003C)  /* LCD Controller Interrupt ID Register */
#define TMEDRGBR	__REG(0x44000040)  /* TMED RGB Seed Register */
#define TMEDCR		__REG(0x44000044)  /* TMED Control Register */

#define FDADR0		__REG(0x44000200)  /* DMA Channel 0 Frame Descriptor Address Register */
#define FSADR0		__REG(0x44000204)  /* DMA Channel 0 Frame Source Address Register */
#define FIDR0		__REG(0x44000208)  /* DMA Channel 0 Frame ID Register */
#define LDCMD0		__REG(0x4400020C)  /* DMA Channel 0 Command Register */
#define FDADR1		__REG(0x44000210)  /* DMA Channel 1 Frame Descriptor Address Register */
#define FSADR1		__REG(0x44000214)  /* DMA Channel 1 Frame Source Address Register */
#define FIDR1		__REG(0x44000218)  /* DMA Channel 1 Frame ID Register */
#define LDCMD1		__REG(0x4400021C)  /* DMA Channel 1 Command Register */

#define LCCR0_ENB		(1 << 0)


#define PXAFB_BPP		16
#define PXAFB_BPP_BITS		0x04
#define FB_SYNC_HOR_HIGH_ACT	1
#define FB_SYNC_VERT_HIGH_ACT	2



/*
// 800 X 600 X 65535 ==============================

#define LCD_PIXCLOCK			25000000
#define LCD_BPP				PXAFB_BPP
#define LCD_XRES			800
#define LCD_YRES			600
#define LCD_HORIZONTAL_SYNC_PULSE_WIDTH	2
#define LCD_VERTICAL_SYNC_PULSE_WIDTH	2
#define LCD_BEGIN_OF_LINE_WAIT_COUNT	10
#define LCD_BEGIN_FRAME_WAIT_COUNT      29	
#define LCD_END_OF_LINE_WAIT_COUNT	10
#define LCD_END_OF_FRAME_WAIT_COUNT	29
#define LCD_SYNC			(FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT)
#define LCD_LCCR0			0x003008F8
#define LCD_LCCR3 			(0x0440FF02 | (PXAFB_BPP_BITS << 24))
*/

// 1024 X 768 X 65535 ==============================
/*
#define LCD_PIXCLOCK			66000000
#define LCD_BPP				PXAFB_BPP
#define LCD_XRES			1024
#define LCD_YRES			768
#define LCD_HORIZONTAL_SYNC_PULSE_WIDTH	8
#define LCD_VERTICAL_SYNC_PULSE_WIDTH	2
#define LCD_BEGIN_OF_LINE_WAIT_COUNT	180
#define LCD_BEGIN_FRAME_WAIT_COUNT  	1
#define LCD_END_OF_LINE_WAIT_COUNT	200
#define LCD_END_OF_FRAME_WAIT_COUNT	1
#define LCD_SYNC			(FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT)
#define LCD_LCCR0			0x003008F8
#define LCD_LCCR3 			(0x0440FF01 | (PXAFB_BPP_BITS << 24))
*/

// 1024 X 768 X 65535
/*
#define LCD_PIXCLOCK			27000000
#define LCD_BPP				PXAFB_BPP
#define LCD_XRES			1024
#define LCD_YRES			768
#define LCD_HORIZONTAL_SYNC_PULSE_WIDTH	6
#define LCD_VERTICAL_SYNC_PULSE_WIDTH	2
#define LCD_BEGIN_OF_LINE_WAIT_COUNT	1
#define LCD_BEGIN_FRAME_WAIT_COUNT  	0
#define LCD_END_OF_LINE_WAIT_COUNT	1
#define LCD_END_OF_FRAME_WAIT_COUNT	0
#define LCD_SYNC			(FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT)
#define LCD_LCCR0			0x003008F8
#define LCD_LCCR3 			(0x0470FF02 | (PXAFB_BPP_BITS << 24))
*/

// 640 X 480 X 65535
#define LCD_PIXCLOCK			25000000
#define LCD_BPP				PXAFB_BPP
#define LCD_XRES			640
#define LCD_YRES			480
#define LCD_HORIZONTAL_SYNC_PULSE_WIDTH	2 
#define LCD_VERTICAL_SYNC_PULSE_WIDTH	2 
#define LCD_BEGIN_OF_LINE_WAIT_COUNT	10
#define LCD_BEGIN_FRAME_WAIT_COUNT 	29
#define LCD_END_OF_LINE_WAIT_COUNT	10
#define LCD_END_OF_FRAME_WAIT_COUNT	29
#define LCD_SYNC			(FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT)
#define LCD_LCCR0			0x003008F8
#define LCD_LCCR3 			(0x0040FF02 | (PXAFB_BPP_BITS << 24))


#define LCD_WIDTH		LCD_XRES
#define LCD_HEIGHT		LCD_YRES

//#define LCD_BASE_ADDR		0xA2800000 
//#define LCD_BASE_ADDR		0xA1E01000
#define LCD_BASE_ADDR		0xA1401000


#define LCCR0_ENB       (1 << 0)        /* LCD Controller enable */
#define LCCR0_CMS       (1 << 1)        /* Color = 0, Monochrome = 1 */
#define LCCR0_SDS       (1 << 2)        /* Single Panel = 0, Dual Panel = 1 */
#define LCCR0_LDM       (1 << 3)        /* LCD Disable Done Mask */
#define LCCR0_SFM       (1 << 4)        /* Start of frame mask */
#define LCCR0_IUM       (1 << 5)        /* Input FIFO underrun mask */
#define LCCR0_EFM       (1 << 6)        /* End of Frame mask */
#define LCCR0_PAS       (1 << 7)        /* Passive = 0, Active = 1 */
#define LCCR0_BLE       (1 << 8)        /* Little Endian = 0, Big Endian = 1 */
#define LCCR0_DPD       (1 << 9)        /* Double Pixel mode, 4 pixel value = 0, 8 pixle values = 1 */
#define LCCR0_DIS       (1 << 10)       /* LCD Disable */
#define LCCR0_QDM       (1 << 11)       /* LCD Quick Disable mask */
#define LCCR0_PDD       (0xff << 12)    /* Palette DMA request delay */
#define LCCR0_PDD_S     12 
#define LCCR0_BM        (1 << 20)       /* Branch mask */
#define LCCR0_OUM       (1 << 21)       /* Output FIFO underrun mask */


struct pxafb_dma_desc 
{
	int32u fdadr;
	int32u fsadr;
	int32u fidr;
	int32u ldcmd;
};

int init_lcd( void );

#define RGB(r,g,b)	( ((r/8)<<11)|((g/4)<<5)|(b/8) )
#define GetR( rgb )	(rgb>>11)
#define GetG( rgb )     ((rgb<<5)>>11)
#define GetB( rgb )     (rgb&0x001F)

void lcd_clear( int16u col );
void put_image( int16u* image );
void put_pixel_16( int x, int y, int16u col );
void put_line_16( int x, int y, int x2, int y2, int16u col );
void put_text_16( int x, int y, char* s, int16u* col_text, int16u* col_back );
void put_rect_image( int x, int y, int x2, int y2, int16u* img );

#endif //  _LCD_H
