/*////////////////////////////////////////////////////////////////////////

  mini-FAT library
  by KyungMin, Cho.  bro(bro@shinbiro.com)
  -----------------------------------------------------------------------

 2004-07-19 | initial code ( romfs_path with root path )
 2004-07-20 | romfs_path 
 2004-07-21 | init code support FAT12/FAT16
 2004-07-26 | fatfs_file functions
              limit file size = FAT12:<20M FAT16:<500MB  logical:<2097MB

/////////////////////////////////////////////////////////////////////// */
#ifndef __FAT_H__
#define __FAT_H__

#include <types.h>

#define MAX_FSPATH					255

// fat12 = 0x0FFF fat16 = 0xFFFF fat32 = 0xFFFFFFFF
#define END_OF_CLUSTER				(-1)
// root cluster is virtual cluster by bro
#define ROOT_CLUSTER				0
// not found cluster is virtual cluster by bro
#define NOT_FOUND_CLUSTER			0

#define EMPTY_CLUSTER				0

// fat12 = 0x0FF7 fat16 = 0xFFF7 fat32 = 0xFFFFFFF7
// but current version ignore bad cluster number...
#define BAD_CLUSTER					0xFFFFFFF7

///////////////////////////////////////////////////////////////////////////
// FAT RAW IMPLEMENTATION
//
//  FAT12/FAT16  Ѵ.

#pragma pack(1)
typedef struct _BPB {

	int8u	BS_jmpBoot[3]; // ڵ 0xEB,0x??,0x90 0xE9,0x??,0x??
	int8u	BS_OEMName[8]; // MSWIN4.1 MSDOS5.0
	int8u	BPB_BytesPerSec[2]; //  ȣȯ ؼ 512 (512)
	int8u	BPB_SecPerClus; // Ŭʹ ͼ (2)
	int16u	BPB_RsvdSecCnt; // ༽ (1)
	int8u	BPB_NumFATs;	// FAT  (2)
	int8u	BPB_RootEntCnt[2]; // Ʈ丮 丮׸ (FAT32 0)
	int16u	BPB_TotSec16;	//  ͼ (FAT32 0)
	int8u	BPB_Media;		// ĸü 0xF8  ̵ĸü 0xF0
	int16u	BPB_FATSz16;	//  FAT   ͼ (FAT32 0)
	int16u	BPB_SecPerTrk;	// Ʈ ͼ (ũ˱ ش)
	int16u	BPB_NumHeads;	//  (ũ˱ ش)
	int32u	BPB_HiddSec;	// ش FAT  ƼǾտ ִ  ͼ (ƼǾȵ ü 0)
	int32u	BPB_TotSec32;	// FAT32  ͼ (FAT12/16 0)
	
	// FAT12/16  
	int8u	BS_DrvNum;		// ̺ȣ ÷ 0x00 , ϵ 0x80
	int8u	BS_Reserved1;	// NT Ǵ   (0)
	int8u	BS_BootSig;		// Ȯ Ʈ (0x29). Ʒ ׸  ǥ
	int32u	BS_VolID;		//  ø ȣ ( ¥ð յ )
	int8u	BS_VolLab[11];	//   (NO NAME      ) 
	int8u	BS_FilSysType[8];	// "FAT12    ", "FAT16    ", "FAT      " (̰ɷ FS¸ ؼ ȵȴ.)
	
} BPB;
#pragma pack()

#define FILE_ATTR_READ_ONLY				0x01
#define FILE_ATTR_HIDDEN				0x02
#define FILE_ATTR_SYSTEM				0x04
#define FILE_ATTR_VOLUME_ID				0x08
#define FILE_ATTR_DIRECTORY				0x10
#define FILE_ATTR_ARCHIVE				0x20
#define FILE_ATTR_LONG_NAME				(FILE_ATTR_READ_ONLY | FILE_ATTR_HIDDEN | FILE_ATTR_SYSTEM | FILE_ATTR_VOLUME_ID)

#pragma pack(push,1)
typedef struct _directory_entry {
	//int8u	DIR_Name[11];	//  ( a.z => "a       z  " ⺻  ä)
	int8u	DIR_FileName[8];// ϸ 
	int8u	DIR_Extension[3];// Ȯ
	int8u	DIR_Attr;		//  Ӽ 
	int8u	DIR_NTRes;		// NT Ǵ   ( dot: 0x00 dir:0x08 file:0x18)
	int8u	DIR_CrtTimeTenth;// 1/10    ð
	int16u	DIR_CrtTime;	//   ð
	int16u	DIR_CrtDate;	//   ð
	int16u	DIR_LstAccDate;	//   ð
	int16u	DIR_FstClusHI;	// ù° Ŭ   (FAT12/16 ׻ 0)
	int16u	DIR_WrtTime;	// ֱ  ð
	int16u	DIR_WrtDate;	// ֱ  ¥
	int16u	DIR_FstClusLO;	// ù° Ŭ  
	int32u	DIR_FileSize;	//  ũ
} directory_entry;
#pragma pack(pop)

void raw2BPB( char* data, BPB* bpb );
int is_bootsector( char* bootsector );

typedef enum _fs_type
{
	FS_UNKNOWN,
	FS_FAT12,
	FS_FAT16,
	FS_FAT32
}fs_type;

fs_type get_fs_type( BPB* bpb );

int get_root_dir_sector( BPB* bpb );
int get_root_dir_sectors( BPB* bpb );

int get_first_sector_of_cluster( BPB* bpb, int cluster );
int get_first_data_sector( BPB* bpb );

int get_next_cluster_in_fat12( BPB* bpb, int8u* fat, int cluster );
int get_next_cluster_in_fat16( BPB* bpb, int8u* fat, int cluster );

int get_find_cluster_in_fat12( BPB* bpb, int8u* fat, int cluster );
int get_find_cluster_in_fat16( BPB* bpb, int8u* fat, int cluster );

//////////////////////////////////////////////////////////////////////////////////
// FAT FILE SYSTEM IMPLEMENTATION

// sector i/o for virtual filesystem
typedef struct _sector_io
{
	void* storage;		// storage object

	// additional information
	int bytes_per_sector;

	int (*read)( struct _sector_io* sio, int sector, int offset, int8u* buf, int size );
	int (*write)( struct _sector_io* sio, int sector, int offset, int8u* buf, int size );
}sector_io;

typedef struct _fatfs
{
	// virtual i/o
	sector_io	sio;

	// FAT navtive
	BPB			bpb;
	fs_type		type;
	int8u*		fat;
	
	// FAT function
	int (*get_next_cluster)( struct _fatfs* fs, int cluster );
	int (*get_prev_cluster)( struct _fatfs* fs, int cluster );
}fatfs;

int get_next_cluster_fat12( fatfs* fs, int cluster );
int get_prev_cluster_fat12( fatfs* fs, int cluster );
int get_next_cluster_fat16( fatfs* fs, int cluster );
int get_prev_cluster_fat16( fatfs* fs, int cluster );

/////////////////////////////////////////////////////////////////////////
// FAT Implementation
//  Ŭ  ͸ ϰ ־ ҵϴ.
//  ۾ Ŭ ۿ  directory_entry  

#define FATFS_MAX_PATH_BUF_SECTOR		4
typedef struct _fatfs_path
{
	int		cluster;			// fat  cluster
	int8u	buf[ FATFS_MAX_PATH_BUF_SECTOR*512 ];		// image ؿ work ram buffer memory

	directory_entry*	de;		//  directory entry

	int		cur_dir_cluster;	// ڽ directory entry ã  ִ 
								// folder cluster (directory's cluster)

	int8u	FullPath[MAX_FSPATH];	// full path cache
	int8u	FileName[13];	// ϸ 
	int8u	Attr;			//  Ӽ
	int16u	CrtTime;		//   ð
	int16u	CrtDate;		//   ð
	int16u	LstAccDate;		//   ð	int16u	DIR_FstClusHI;	// ù° Ŭ   (FAT12/16 ׻ 0)
	int16u	WrtTime;		// ֱ  ð
	int16u	WrtDate;		// ֱ  ¥
	int32u	FileSize;		//  ũ

}fatfs_path;

int fatfs_read_cluster( fatfs* fs, fatfs_path* wc, int sector, int sector_count );
int fatfs_read_work_cluster( fatfs* fs, fatfs_path* wc, int cluster );
int fatfs_read_root_cluster( fatfs* fs, fatfs_path* wc );
int fatfs_read_next_cluster( fatfs* fs, fatfs_path* wc );
int fatfs_read_prev_cluster( fatfs* fs, fatfs_path* wc );

int fatfs_read_parent_cluster( fatfs* fs, int cur_dir_cluster, fatfs_path* p );

int fatfs_cluster_set_directory_entry( fatfs* fs, fatfs_path* wc );
fatfs_path* fatfs_path_next_internal( fatfs* fs, fatfs_path* wc );


int fatfs_is_equal_fname( char* DIR_FName, char* filename );
char* fatfs_get_last_char( char* psz, int len );
void fatfs_get_file_full_name( char* fullname, char* filename, char* extension );

char* get_path_prev( char* cur, char* start );
char* get_parent_path( char* path, char* parent );

void fatfs_set_file_full_name( char* DIR_FileName, char* filefullname );

// internal FAT(faile allocation table) function
int fatfs_get_fat_count( fatfs* fs );

int fatfs_get_fat( fatfs* fs, int cluster );
void fatfs_set_fat( fatfs* fs, int cluster, int value );
int fatfs_get_last_cluster( fatfs* fs, int cluster );
int fatfs_get_first_cluster( fatfs* fs, int cluster );
int fatfs_get_next_cluster( fatfs* fs, int cluster );
int fatfs_get_prev_cluster( fatfs* fs, int cluster );

int fatfs_find_free_cluster( fatfs* fs );
int fatfs_alloc_cluster( fatfs* fs );
int fatfs_realloc_cluster( fatfs* fs, int cluster );
int fatfs_free_cluster_list( fatfs* fs, int cluster );


typedef struct _fatfs_file
{
	int8u	FileName[13];	// ϸ 
	int8u	Attr;			//  Ӽ
	int16u	CrtTime;		//   ð
	int16u	CrtDate;		//   ð
	int16u	LstAccDate;		//   ð	int16u	DIR_FstClusHI;	// ù° Ŭ   (FAT12/16 ׻ 0)
	int16u	WrtTime;		// ֱ  ð
	int16u	WrtDate;		// ֱ  ¥
	int32u	FileSize;		//  ũ

	int8u	FullPath[MAX_FSPATH];	// full path cache
	int		first_cluster;	//  Ÿ ó Ŭ
	int		cur_cluster;	//  Ŭ ġ
	int		cur_sector;		//  Ŭ   ġ
	int		cur_offset;		//    ġ
}fatfs_file;


////////////////////////////////////////////////////////////////////////////
// public fatfs api

// fat file system
int init_fatfs( fatfs* fs, sector_io sio );
void free_fatfs( fatfs* fs );


// list file/dir 
int fatfs_open_path( fatfs* fs, char* path, fatfs_path* p );
fatfs_path* fatfs_path_next( fatfs* fs, fatfs_path* wc );
int fatfs_is_directory( fatfs_path* wc );

// directory i/o
int fatfs_create_directory( fatfs* fs, char* path );
int fatfs_rename_directory( fatfs* fs, char* path, char* newpath );

int fatfs_set_unused_de( fatfs* fs, char* path, directory_entry* pde );
int fatfs_create_de( fatfs* fs, char* path, directory_entry* psde );

int fatfs_delete_directory( fatfs* fs, char* path );


// utility (not used)
int fatfs_get_full_path( fatfs* fs, fatfs_path* p, char* buf, int len );
int fatfs_fix_full_path( char* path, char* fixbuf, int size );


// file i/o
int fatfs_create_file( fatfs* fs, char* path );
int fatfs_rename_file( fatfs* fs, char* path, char* newpath );
int fatfs_delete_file( fatfs* fs, char* path );

#define FATFS_FSEEK_SET			0
#define FATFS_FSEEK_CUR			1
#define FATFS_FSEEK_END			2

int fatfs_fopen( fatfs* fs, char* path, fatfs_file* file );
int fatfs_fread( fatfs* fs, fatfs_file* file, void* buf, int size );
int fatfs_fwrite( fatfs* fs, fatfs_file* file, void* buf, int size );
int fatfs_fseek( fatfs* fs, fatfs_file* file, int offset, int orgin );
int	fatfs_ftell( fatfs* fs, fatfs_file* file );
void fatfs_fclose( fatfs* fs, fatfs_file* file );

#endif // __FAT_H__
