/*/////////////////////////////////////////////////////////////////////////
 mini-GUI
 by bro ( bro@shinbiro.com )
 2004/10/18

/////////////////////////////////////////////////////////////////////////*/
#ifndef __GUI_H__
#define __GUI_H__

#include <blist.h>
///////////////////////////////////////////////////////////////////////////
// primitive graphic library
///////////////////////////////////////////////////////////////////////////

/*
typedef unsigned long unsigned long;
typedef unsigned short unsigned short;
typedef unsigned char u_char;
*/


// 0x00bbggrr
#define RGB32(r,g,b)  ( ((unsigned long)(((unsigned char)(r) | ((unsigned char)(g)<<8)) | (((unsigned long)(unsigned char)(b))<<16)))&0x00FFFFFF ) 
//#define RGB16(r,g,b)	 (unsigned short)( ((r&0x1F)<<8) | ((g&0x3F)<<5) | ((b&0x1F) ))

#define RGB16(r,g,b)      ( ((r/8)<<11)|((g/4)<<5)|(b/8) )

#undef GUI_BPP_32		1
#define GUI_BPP_16		1		


#ifdef GUI_BPP_32
typedef unsigned long COLOR;
#define COLOR_R(cr) ((unsigned char)cr)
#define COLOR_G(cr) ((unsigned char)(cr>>8))
#define COLOR_B(cr) ((unsigned char)(cr>>16))
#define _RGB(r,g,b)		RGB32((r),(g),(b))
#endif

#ifdef GUI_BPP_16 
typedef unsigned short COLOR;
#define COLOR_R(cr) ((unsigned char)(cr>>11)*4)
#define COLOR_G(cr) ((unsigned char)((cr<<5)>>11)*8)
#define COLOR_B(cr) ((unsigned char)(cr&0x001F)*4)
#define _RGB(r,g,b)		RGB16(r,g,b)
#endif


typedef struct _bitmap
{
	int width;
	int height;
	int bpp;
	int byte_per_pixel;
	
	char* data;
	char* line[1];
}bitmap;

bitmap* create_bitmap( int width, int height, int bpp );
void destory_bitmap( bitmap* bm );

void putpix( bitmap* bm, int x, int y, COLOR col );
COLOR getpix( bitmap* bm, int x, int y );

void line( bitmap* bm, int x, int y, int x2, int y2, COLOR col );

void blit( bitmap* des, int desx, int desy, bitmap* src, int x, int y, int w, int h );



/////////////////////////////////////////////////////////////////////////
// windows system library
/////////////////////////////////////////////////////////////////////////
typedef unsigned long hwnd;
typedef unsigned long hdc;
typedef unsigned long hmenu;

#ifndef MAKEWPARAM
#define MAKEWPARAM(low,high)	( (unsigned long) ( (( (unsigned long)(unsigned short)high<<16) | ((unsigned short)low)) ) )
#endif
#ifndef HIWORD
#define HIWORD(ul)			((unsigned short)(ul>>16))
#endif
#ifndef LOWORD
#define LOWORD(ul)			((unsigned short)ul)
#endif

typedef struct _point
{
	int x;
	int y;
}point;

typedef struct _rect
{
	int top;
	int left;
	int right;
	int bottom;
}rect;

void set_rect( rect* rt, int l, int t, int r, int b );
void rect_smaller( rect* rt, int i );
void rect_move( rect* rt, point* org );
int rect_in( rect* rt, int x, int y );

void client_to_screen( hwnd w, point* pt );
void client_to_screen_rt( hwnd w, rect* rt );

void screen_to_client( hwnd w, point* pt );

////////////////////////////////////////////////////////////////////////////
// GDI

#define MAX_CLIP_RECT		20
typedef struct _device_context
{
	hwnd w;
	
	rect clip[MAX_CLIP_RECT]; // dc의 로직 좌표계를 기준으로 하는 rectangle
	int clip_count;

	point org;				// dc의 로직 좌표계 기준

}device_context;

hdc get_dc( hwnd w );
hdc get_window_dc( hwnd w );
void release_dc( hdc dc );

void gdi_setpixel( hdc dc, int x, int y, COLOR cr );
void gdi_line( hdc dc, int x1, int y1, int x2, int y2, COLOR cr );
void gdi_rect( hdc dc, rect* rt, COLOR cr, int fill );
void gdi_circle( hdc dc, int xc, int yc, int r, COLOR cr );
void gdi_draw_text( hdc dc, char* s, rect* rt, COLOR* pcrText, COLOR* pcrBack );

void draw_caption_buttons( hwnd w, hdc dc );
int hittest_caption_buttons( hwnd w, int x, int y );
void draw_caption( hwnd h, hdc dc, rect* rt );
void draw_button( hwnd w, hdc dc, char* text, rect* rt, int press );

/////////////////////////////////////////////////////////////////////////////////
// message

#define wm_idle						0
#define wm_create					1
#define wm_destroy					2
#define wm_ncpaint					3
#define wm_paint					4
#define wm_showwindow				5
#define wm_erasebkgnd				6
#define wm_lbuttondown				7
#define wm_lbuttonup				8
#define wm_nclbuttondown			9
#define wm_nclbuttonup				10
#define wm_command					11
#define wm_nchittest				12
#define wm_syscommand				13
#define wm_taskcommand				14

// wm_syscommand
#define sc_close					1

typedef struct _msg
{
	blist_node node;

	hwnd wnd;	
	long message;
	long wparam;
	long lparam;
}msg;

long send_message( hwnd w, unsigned long message, unsigned long wparam, unsigned long lparam );
long post_message( hwnd w, unsigned long message, unsigned long wparam, unsigned long lparam );
long dispatch_message( msg* m );
int get_message( msg* m, hwnd w );

////////////////////////////////////////////////////////////////////////////////// 
// window class

typedef int (*WINPROC)( hwnd wnd, unsigned long message, unsigned long wparam, unsigned long lparam );

typedef struct _wndclass
{
	blist_node node;

	char name[64];
	WINPROC winproc;

	int cb_wnd_extra;
}wndclass;

int register_class( wndclass* wndcls );

////////////////////////////////////////////////////////////////////////////////
// window
// window style
#define ws_caption				0x00000001
#define ws_close				0x00000002
#define ws_visible				0x00000008
#define ws_minimize				0x00000010
#define ws_overlappedwindow		0x000000FF
#define ws_child				0x00000100

typedef struct _window
{
	char name[32];
	
	// rt = (100,100,200,200) 이면 rtcaption = (3,3,94,18) rtclient = (3,19,94,94)
	rect rt;				// 부모윈도 rt에 대한 상대 좌표 rt
	rect rtcaption;			// rt에 대한 상대 좌표 캡션 영역 
	rect rtclient;			// rt에 대한 상대 좌표 클라이언트 영역
	
	int wid;				// window identifier
	int showhide;			// 0 : hide 1: show 

	wndclass* wndcls;

	struct _window* parent;
	struct _window* next;
	struct _window* prev;
	struct _window* child;

	unsigned long ws_style;
	hmenu	menu;
	int	close_pressed;
	int	minimize_pressed;
	void* extra;			// window's extra data
}window;

hwnd create_window( char* clsname, char* name, unsigned long style, int x, int y, int w, int h, hwnd parent, int wid );
void destroy_window( hwnd w );

void draw_window( hwnd w );
void draw_window_frame( hwnd h );
void draw_window_client( hwnd w );

void get_client_rect( hwnd w, rect* rt );
void get_window_rect( hwnd w, rect* rt );

hwnd window_from_point( point pt );

void set_parent( hwnd w, hwnd c );
hwnd get_top_window( hwnd w );

void set_window_text( hwnd w, char* text );
int get_window_text( hwnd w, char* text, int count );

void show_window( hwnd w, int sw );
void move_window( hwnd hw, int x, int y, int w, int h, int repaint );

// focus
void set_active_window( hwnd w );

// hit-test
#define ht_caption				0x00000001
#define ht_client				0x00000002
#define ht_close				0x00000004
#define ht_minimize				0x00000008
#define ht_top					0x00000010
#define ht_left					0x00000020
#define ht_right				0x00000040
#define ht_bottom				0x00000080
#define ht_topleft				0x00000100
#define ht_topright				0x00000200
#define ht_bottomleft			0x00000400
#define ht_bottomright			0x00000800
#define ht_border				0x00000FF0


////////////////////////////////////////////////////////////////////////////
// common window
int default_window_proc( hwnd wnd, unsigned long message, unsigned long wparam, unsigned long lparam );


// button wm_command
#define bn_clicked				1
typedef struct _button_extra
{
	int pressed;
}button_extra;

typedef struct _taskinfo
{
	blist_node node;
	hwnd hw;
}taskinfo;

typedef struct _taskbar_extra
{
	blist taskinfos;
}taskbar_extra;


////////////////////////////////////////////////////////////////////////////
// menu
typedef struct _menuitem
{
	blist_node node;

	char name[32];
	int command;
	unsigned long flag;
}menuitem;

typedef struct _menu
{
	blist menuitems;
	
	hwnd owner;
	hwnd hw_menu; // cache
}menu;

hmenu create_menu();
int append_menu( hmenu m, unsigned long flag, int command, char* name );
void destroy_menu( hmenu m );
void track_popup_menu( hmenu m, int sx, int sy, hwnd hw );
void set_menu( hwnd hw, hmenu hm );

////////////////////////////////////////////////////////////////////////////
// mini gui system
typedef void (*draw_setpixel)( int x, int y, COLOR cr );
typedef void (*draw_line)( int x, int y, int x2, int y2, COLOR cr );

typedef void (*mouse_lb_down)( int x, int y );
typedef void (*mouse_lb_up)( int x, int y );

void init_gui( int screen_width, int screen_height, int bpp,
			  draw_setpixel dsp, draw_line line );
void start_gui();

void gui_mouse_event_lb_down( int x, int y );
void gui_mouse_event_lb_up( int x, int y );

// mini gui system
struct _gui
{
	// screen
	int width;
	int height;
	int bpp;

	// draw primitive setting
	draw_setpixel	setpixel;
	draw_line		line;

	hwnd desktop;		// desktop window
	hwnd taskbar;		// taskbar window
	hwnd active;		// current activated window

	hwnd menu;			// current popup menu

	// for mouse click
	hwnd pressed_wnd;
	int pressed_hitest;
	point pressed_pt;	// point in window
};

extern struct _gui gui;
#endif //__GUI_H__
