#ifndef _VFS_H
#define _VFS_H

#include<kernel/resource.h>
#include<list.h>
#include<types.h>

#define MAX_VFS_OBJECT_NUM	(20)

/* ID numbers for external devices */
#define ID_FAT		(0)
#define ID_SERIAL 	(1)
#define ID_LCD 		(2)
#define ID_LED 		(3)


#define	TYPE_FS		(0x01)
#define	TYPE_DEVICE	(0x02)
#define	TYPE_OTHERS	(0x04)

/* For file usage control */
#define NOT_USE	(0)
#define IN_USE	(1)


#define	SET_POS		(0x01)
#define	CUR_POS		(0x02)
#define	END_POS		(0x04)



// Temporary use



struct file 
{
	int fd;
	struct file_op *operator;
	struct list_head file_list;
	struct process *owner;
	int use_count;
	int offset;
	int size;
	int state;
	void *private;
	lock_t lock;
		
};



struct vfs_object 
{
	int object_id;
	char *name;
	struct resource *data;
	struct file_op *operator;
	struct list_head object_list;
	struct process *owner;
	fp_t initialize;
	lock_t lock;
};


struct file_op 
{
	int device_type;
	fp_t init;
	int (*open)(struct file *,char *name,int mode);
	int (*read)(struct file *,char *,int);
	int (*lseek)(struct file *,int,int);
	int (*write)(struct file *,char *,int);
	int (*close)(int fd);
	fp_t destroy;
	fp_t ioctl;
};





int init_vfs(void);
int open(int type, char *name, char mode);
int read(int fd, char *buffer, int size);
int vfs_register_device(int request_id,struct resource *resource_to_add);




#endif // _VFS_H
