/**
 * \defgroup uipopt Configuration options for uIP
 * @{
 *
 * uIP is configured using the per-project configuration file
 * "uipopt.h". This file contains all compile-time options for uIP and
 * should be tweaked to match each specific project. The uIP
 * distribution contains a documented example "uipopt.h" that can be
 * copied and modified for each project.
 */

/**
 * \file
 * Configuration options for uIP.
 * \author Adam Dunkels <adam@dunkels.com>
 *
 * This file is used for tweaking various configuration options for
 * uIP. You should make a copy of this file into one of your project's
 * directories instead of editing this example "uipopt.h" file that
 * comes with the uIP distribution.
 */

/*
 * Copyright (c) 2001-2003, Adam Dunkels.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.  
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  
 *
 * This file is part of the uIP TCP/IP stack.
 *
 * $Id: uipopt.h,v 1.5 2004/11/22 21:31:01 bckddn Exp $
 *
 */

#ifndef __UIPOPT_H__
#define __UIPOPT_H__

typedef unsigned char u8_t;
typedef unsigned short u16_t;
typedef unsigned short uip_stats_t;

/*
 * ip address config!
 */
#define UIP_FIXEDADDR    1
#define UIP_PINGADDRCONF 0


#if 1
#define UIP_IPADDR0     211
#define UIP_IPADDR1     189
#define UIP_IPADDR2     19
#define UIP_IPADDR3     222

#define IPADDR		"211.189.19.222"
#else
#define UIP_IPADDR0     192
#define UIP_IPADDR1     168
#define UIP_IPADDR2     77
#define UIP_IPADDR3     222

#define IPADDR		"192.168.77.222"
#endif

/*
 * netmake and bit config!
 */
#define UIP_NETMASK0    255
#define UIP_NETMASK1    255
#define UIP_NETMASK2    255
#define UIP_NETMASK3    0

/*
 * gateway address config!
 */
#if 1
#define UIP_DRIPADDR0   211
#define UIP_DRIPADDR1   189
#define UIP_DRIPADDR2   19
#define UIP_DRIPADDR3   1
#else
#define UIP_DRIPADDR0   192
#define UIP_DRIPADDR1   168
#define UIP_DRIPADDR2   77
#define UIP_DRIPADDR3   254
#endif


/*
 * mac address config!
 */
#define UIP_FIXEDETHADDR 1


#define UIP_ETHADDR0    0x00
#define UIP_ETHADDR1    0xa2
#define UIP_ETHADDR2    0x55
#define UIP_ETHADDR3    0xf2
#define UIP_ETHADDR4    0x26
#define UIP_ETHADDR5    0x25



#define UIP_TTL         255
#define UIP_REASSEMBLY 0
#define UIP_REASS_MAXAGE 40
#define UIP_UDP           0
#define UIP_UDP_CHECKSUMS 0
#define UIP_UDP_CONNS    10
#define UIP_UDP_APPCALL  udp_appcall
#define UIP_ACTIVE_OPEN 1
#define UIP_CONNS       10
#define UIP_LISTENPORTS 10
#define UIP_RECEIVE_WINDOW   32768
#define UIP_URGDATA      1
#define UIP_RTO         3000
#define UIP_MAXRTX      8000
#define UIP_MAXSYNRTX      3000
#define UIP_TCP_MSS     (UIP_BUFSIZE - UIP_LLH_LEN - 40)
#define UIP_TIME_WAIT_TIMEOUT 120
#define UIP_ARPTAB_SIZE 16
#define UIP_ARP_MAXAGE 120
#define UIP_BUFSIZE     1500
#define UIP_STATISTICS  1
#define UIP_LOGGING     0
void uip_log(char *msg);
#define UIP_LLH_LEN     14
#ifndef LITTLE_ENDIAN
#define LITTLE_ENDIAN  3412
#endif /* LITTLE_ENDIAN */
#ifndef BIG_ENDIAN
#define BIG_ENDIAN     1234
#endif /* BIGE_ENDIAN */

#ifndef BYTE_ORDER
#define BYTE_ORDER     LITTLE_ENDIAN
#endif /* BYTE_ORDER */


#endif /* __UIPOPT_H__ */



