/*
*********************************************************************************************************
*                                            AESOP MP2530F Board Support Package
*
*  An Entertainment Solution On a Platform (AESOP) is a completely Open Source
*  based graphical user environment and suite of applications for PDAs and other
*  devices running Linux. It is included in various embedded Linux distributions
*  such as OpenZaurus - http://www.aesop-embedded.org
*
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  File       : BSP.C
*  By        : jhkim(jhkim@libertron.com)
*  Version : v1.0
*********************************************************************************************************
*/


#define PSR_READ(arg)       __asm{MRS (arg), CPSR}
#define PSR_WRITE(arg)      __asm{MSR CPSR_c, (arg)}

#define NO_IF_INTERRUPTS_IMPLEMENTED_LOW 	32
#define NO_IF_INTERRUPTS_IMPLEMENTED_HIGH	16
/*
*********************************************************************************************************
*                                               CONSTANTS                                                
*********************************************************************************************************
*/

#define  BSP_UNDEF_INSTRUCTION_VECTOR_ADDR   (*(INT32U *)        0x00000004L)
#define  BSP_SWI_VECTOR_ADDR          				(*(INT32U *)        0x00000008L)
#define  BSP_PREFETCH_ABORT_VECTOR_ADDR      	(*(INT32U *)        0x0000000CL)
#define  BSP_DATA_ABORT_VECTOR_ADDR          	(*(INT32U *)        0x00000010L)
#define  BSP_IRQ_VECTOR_ADDR                 			(*(INT32U *)        0x00000018L)
#define  BSP_FIQ_VECTOR_ADDR                 			(*(INT32U *)        0x0000001CL)

#define  BSP_IRQ_ISR_ADDR                   			(*(INT32U *)        0x00000038L)
#define  BSP_FIQ_ISR_ADDR                    			(*(INT32U *)        0x0000003CL)

/*
*********************************************************************************************************
*                                               DATA TYPES
*********************************************************************************************************
*/

typedef  void (*PFNCT)(void);

static PFNCT BSP_Vectors_IRQ[NO_IF_INTERRUPTS_IMPLEMENTED_LOW] = {0,};

/*
*********************************************************************************************************
*                                            GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                               PROTOTYPES
*********************************************************************************************************
*/

extern void  BSP_InitIntCtrl(void);
static void  Tmr_TickInit(void);
//static  void  BSP_IntDisAll (void);
extern void  BSP_IntPendClearAll (void);
extern void  BSP_IntPendClear(INT32U PendNum);
extern void BSP_VectSet_IRQ(INT32U irq, PFNCT isr,INT32U type_irq);
extern void  BSP_IntEnAll (void);
extern void BSP_IntMasking(INT32U IntNum);
extern void BSP_IntUnMasking(INT32U IntNum);


//static 	void  Ext_ISRInit (void);

/*
*********************************************************************************************************
*                                           BSP INITIALIZATION
*
* Description : This function should be called by your application code before you make use of any of the
*               functions found in this module.
*
* Arguments   : none
*********************************************************************************************************
*/

void  BSP_Init (void)
{
    //PICIntEnable  = 0;                                /* Disable ALL interrupts                        */                                          
    //PICIntEnClear = 0xFFFFFFFFL;                      /* Clear ALL external interrupts                 */ 

    BSP_InitIntCtrl();                                /* Initialize the interrupt controller           */
    Tmr_Init();                                       /* Initialize the timers                         */
    Tmr_TickInit();                                   /* Initialize uC/OS-II's tick ISR                */
}

/*
*********************************************************************************************************
*                                    INITIALIZE INTERRUPT CONTROLLER
*
* Description : This function should be called by your application code before you make use of any of the
*               functions found in this module.
*
* Arguments   : none
*
* Note(s)     : 1) 0xE59FF020 is opcode of:  ldr pc,[pc,#+0x20]
*********************************************************************************************************
*/

extern void  BSP_InitIntCtrl (void)
{
	BSP_IntDisAll();
	BSP_IntPendClearAll();

	INTMODEL = 0x0;
	INTMODEH = 0x0;
					
	BSP_IRQ_VECTOR_ADDR               = 0xE59FF018;                /* LDR PC,[PC,#0x18] instruction    */
	BSP_IRQ_ISR_ADDR                  = (INT32U)OS_CPU_IRQ_ISR;    /* IRQ exception vector address     */

	BSP_FIQ_VECTOR_ADDR               = 0xE59FF018;                /* LDR PC,[PC,#0x18] instruction    */
	BSP_FIQ_ISR_ADDR                  = (INT32U)OS_CPU_FIQ_ISR;    /* IRQ exception vector address     */

	BSP_UNDEF_INSTRUCTION_VECTOR_ADDR = 0xEAFFFFFE;                /* Jump to itself                   */
	BSP_SWI_VECTOR_ADDR               = 0xEAFFFFFE;                /* Jump to itself                   */
	BSP_PREFETCH_ABORT_VECTOR_ADDR    = 0xEAFFFFFE;                /* Jump to itself                   */
 	BSP_DATA_ABORT_VECTOR_ADDR        = 0xEAFFFFFE;                /* Jump to itself                   */
	BSP_FIQ_VECTOR_ADDR               = 0xEAFFFFFE;                /* Jump to itself                   */

	BSP_VectSet_IRQ(INTL_TIMER0, Tmr_TickISR_Handler, TYPE_IRQ);

	BSP_IntEnAll();
	
}

/*
*********************************************************************************************************
*                                     DISABLE ALL INTERRUPTS
*
* Description : This function disables all interrupts from the interrupt controller.
*               
* Arguments   : none
*********************************************************************************************************
*/

extern void  BSP_IntDisAll (void)
{
	INTMASKL |= 0xFFFFFFFF;  /* Masking ALL interrupts                          */ 
	INTMASKH |= 0xFFFFFFFF;
}

extern void  BSP_IntEnAll (void)
{
	INTMASKL &= ~(0xFFFFFFFF);  /* Masking ALL interrupts                          */ 
	INTMASKH &= ~(0xFFFFFFFF);
}


extern void  BSP_IntPendClearAll (void)
{

	INTPENDL |= 0xFFFFFFFF; /* Clear pending interrupts                        */
	INTPENDH |= 0xFFFFFFFF;	
}

extern void BSP_IntPendClear(INT32U PendNum)
{
	if(PendNum > 31)

	{
		INTPENDH |= (0x1 << (PendNum - 32));
	}
	else
	{
		INTPENDL |= (0x1 << PendNum);

	}
}


extern void BSP_IntMasking(INT32U IntNum)
{
	if(IntNum > 31)

	{
		INTMASKH |= (0x1 << (IntNum - 32));
	}
	else
	{
		INTMASKL |= (0x1 << IntNum);

	}
}


extern void BSP_IntUnMasking(INT32U IntNum)
{
	if(IntNum > 31)

	{
		INTMASKH &= ~(0x1 << (IntNum - 32));
	}
	else
	{
		INTMASKL &= ~(0x1 << IntNum);

	}
}

extern void BSP_VectSet_IRQ(INT32U irq, PFNCT isr, INT32U type_irq)
{
	if (irq < 63) {
		BSP_Vectors_IRQ[irq] = isr;
	}

	if(type_irq == 0){
		if(irq > 31) {
			INTMODEL &= ~(0x1 << (irq - 32));
		}
		else
		{
			INTMODEL &= ~(0x1 << irq);			
		}
	}
	else 
	{
		if(irq > 31) {
			INTMODEL |= (0x1 << (irq - 32));
		}
		else
		{
			INTMODEH |= (0x1 << irq);
		}
	}
}

/*
*********************************************************************************************************
*                                           IRQ ISR HANDLER
*
* Description : This function is called by OS_CPU_IRQ_ISR() to determine the source of the interrupt
*               and process it accordingly.
*
* Arguments   : none
*********************************************************************************************************
*/

void  OS_CPU_IRQ_ISR_Handler (void)
{
 	PFNCT  pfnct;

	INT32U base_irq_nr, irq_nr, end_irq_nr, pend;
	
	if( INTPENDL != 0 ) 
	{
		pend = INTPENDL;
	
		base_irq_nr = 0;
		end_irq_nr = 32;
	}
	else if ( INTPENDH !=0 )
	{
		pend = INTPENDH;

		base_irq_nr = 32;
		end_irq_nr = 10;
	}

	for ( irq_nr = base_irq_nr; irq_nr < base_irq_nr + end_irq_nr; irq_nr++)
	{
		if ( pend == 1 ) break;
		pend = pend >> 1;
	}

	pfnct = BSP_Vectors_IRQ[irq_nr];

	if(pfnct != (PFNCT)0){
		(*pfnct)();
	}

	BSP_IntPendClear(irq_nr);
}

/*
*********************************************************************************************************
*                                           FIQ ISR HANDLER
*
* Description : This function is called by OS_CPU_FIQ_ISR() to determine the source of the interrupt
*               and process it accordingly.
*
* Arguments   : none
*********************************************************************************************************
*/

void  OS_CPU_FIQ_ISR_Handler (void)
{
 	PFNCT  pfnct;

	INT32U base_irq_nr, irq_nr, end_irq_nr, pend;
	
	if( INTPENDL != 0 ) 
	{
		pend = INTPENDL;
	
		base_irq_nr = 0;
		end_irq_nr = 32;
	}
	else if ( INTPENDH !=0 )
	{
		pend = INTPENDH;

		base_irq_nr = 32;
		end_irq_nr = 10;
	}

	for ( irq_nr = base_irq_nr; irq_nr < base_irq_nr + end_irq_nr; irq_nr++)
	{
		if ( pend == 1 ) break;
		pend = pend >> 1;
	}

	pfnct = BSP_Vectors_IRQ[irq_nr];

	if(pfnct != (PFNCT)0){
		(*pfnct)();
	}

	BSP_IntPendClear(irq_nr);
}

/*
*********************************************************************************************************
*                          READ TIMER USED TO MEASURE INTERRUPT DISABLE TIME
*
* Description : This function is called to read the current counts of the interrupt disable time 
*               measurement timer.  It is assumed that the timer used for this measurement is a free-running
*               16 bit timer.  
*
*               Timer #1 of the LH79520 is used.  This is a down-timer and the counts are inverted to make
*               the timer look as an up counter.
*               
* Arguments   : none
*
* Returns     ; The 16 bit counts of the timer assuming the timer is an UP counter.
*********************************************************************************************************
*/

#if OS_CPU_INT_DIS_MEAS_EN > 0
INT16U  OS_CPU_IntDisMeasTmrRd (void)
{
    INT16U  cnts;

    cnts = (INT16U)(Timer1Value & 0xFFFF);
    return (cnts);
}
#endif

/*
*********************************************************************************************************
*                                     INITIALIZE TIMER FOR uC/OS-View
*
* Description : This function is called to by uC/OS-View to initialize the free running timer that is
*               used to make time measurements.
*
* Arguments   : none
*
* Returns     ; none
*
* Note(s)     : This function is EMPTY because the timer is initialized elsewhere.
*********************************************************************************************************
*/

#if OS_VIEW_MODULE > 0
void  OSView_TmrInit (void)
{
}
#endif

/*
*********************************************************************************************************
*                                     READ TIMER FOR uC/OS-View
*
* Description : This function is called to read the current counts of a 16 bit free running timer.
*
*               Timer #1 of the LH79520 is used.  This is a down-timer and the counts are inverted to make
*               the timer look as an up counter.
*               
* Arguments   : none
*
* Returns     ; The 16 bit counts of the timer assuming the timer is an UP counter.
*********************************************************************************************************
*/

#if OS_VIEW_MODULE > 0
INT32U  OSView_TmrRd (void)
{
    INT32U  cnts;


    cnts = (INT32U)(~TIMER1->value & 0xFFFF);
    return (cnts);
}
#endif


/*
*********************************************************************************************************
*                                         TIMER1 ENABLE INTERRUPTS
*
* Description : This function is enable to timer interrupt.
*
* Arguments   : none
*
* Notes       :
*********************************************************************************************************
*/

void Tmr_Enable_Intr(void)
{
	INT32U    	regvalue = 0;
	
	regvalue		=	TMRCONTROL0;
	regvalue 		|= 	TMRCONTROL_INTENB;
}

/*
*********************************************************************************************************
*                                         TIMER1 DISABLE INTERRUPTS
*
* Description : This function is disable to timer interrupt.
*
* Arguments   : none
*
* Notes       :
*********************************************************************************************************
*/

void Tmr_Disable_Intr(void)
{
	INT32U    	regvalue = 0;
	
	regvalue		=	TMRCONTROL0;
	regvalue 		&= ~TMRCONTROL_INTENB;
}

/*
*********************************************************************************************************
*                                        TIMER1 CLEAR INTERRUPTS SOURCE
*
* Description : This function is clear to timer interrupt source.
*
* Arguments   : none
*
* Notes       :
*********************************************************************************************************
*/
void Tmr_Clear_Intr(void)
{
	INT32U    	regvalue = 0;
	
	regvalue		=	TMRCONTROL0;
	regvalue 		&= ~TMRCONTROL_INTPEND;
 	regvalue 		|=  ( TMRCONTROL_INTPEND );

	TMRCONTROL0 =  regvalue; 	
}

/*
*********************************************************************************************************
*                             TIMER1 DISABLE INTERRUPTS & TURN OFF VECTOR INTERRUPTS
*
* Description : This function is disable to timer and vector  interrupt source.
*
* Arguments   : none
*
* Notes       :
*********************************************************************************************************
*/
void Tmr_Disable(void)
{
	INT32U    	regvalue = 0;
	
	regvalue		=	TMRCONTROL0;
	regvalue 		&= ~TMRCONTROL_RUN;
	
	Tmr_Clear_Intr();                            /* Clear timer0 interrupt source                      */
}

void Tmr_Enable(void)
{
	INT32U    	regvalue = 0;
	
	regvalue		=	TMRCONTROL0;
	regvalue 		|=  TMRCONTROL_RUN;
	
	Tmr_Clear_Intr();                            /* Clear timer0 interrupt source                      */
}


/*
*********************************************************************************************************
*                                         TICKER INITIALIZATION
*
* Description : This function is called to initialize uC/OS-II's tick source (typically a timer generating
*               interrupts every 1 to 100 mS).
*
*               We decided to use Timer #0 as the tick interrupt source.
*
* Arguments   : none
*
* Notes       :
*********************************************************************************************************
*/

void  Tmr_Init (void)
{

	INT32U regvalue	= 0;
	INT32U pllvalue, v, p, m, s;
	
	
	regvalue	=	TMRCLKGEN0;
	regvalue	&= ~( TMRCLKGEN_CLKSRCSEL << 1 );   
 	regvalue  |=  ( TMRCLKGEN_CLKSRC_PLL2    );	 		
 	TMRCLKGEN0	=  regvalue; 	
 	
 	regvalue	=	TMRCLKGEN0;
 	regvalue 	&= ~( TMRCLKGEN_CLKDIV         );       
	regvalue	|=  ( TMRCLKGEN_CLKDIV0 - 1 ) << ( TMRCLKGEN_CLKDIV_SHIFT );
	TMRCLKGEN0	=  regvalue;


	regvalue	=	TMRCLKENB0;
	regvalue 	&=  ~( TMRCLKENB_TCLKMODE);
	regvalue 	|=   ( TMRCLKENB_TCLKMODE ); 			
	TMRCLKENB0	=	 regvalue;
	
	regvalue	=	TMRCLKENB0;
	regvalue	 &= ~( TMRCLKENB_CLKGENENB ); 			
	regvalue	 |=  ( TMRCLKENB_CLKGENENB );
	TMRCLKENB0	 =	 regvalue;
	
	regvalue	=	TMRCONTROL0;
	regvalue 	&= ~( TMRCONTROL_INTPEND | TMRCONTROL_SET_TCLOCK );
	regvalue 	|=  ( TMRCONTROL_TCLOCK_DIV1                     );
	TMRCONTROL0  =	regvalue;
				
	TMRCOUNT0	=   (INT32U)0;

	v = PLLSETREG2;
	p = (INT32U)((v>>PLLSET_PDIV_SHIFT)&0xff);
	m = (INT32U)((v>>PLLSET_MDIV_SHIFT)&0xff);
	s = (INT32U)((v)&0xff);
	pllvalue = ( (m+8) * 27000000) / ( (p+2) * (1<<s) );
						
	regvalue 	=	(INT32U)( ( pllvalue / (INT32U)32 ) / (INT32U)(10) );
	TMRMATCH0	=   (INT32U)regvalue;
 	
 	regvalue	=	TMRCONTROL0;
 	regvalue	&= ~( TMRCONTROL_INTENB | TMRCONTROL_INTPEND );
 	regvalue 	|=  ( TMRCONTROL_INTENB );
	TMRCONTROL0 =   regvalue;											


	regvalue	=	TMRCONTROL0;
	regvalue &= ~( TMRCONTROL_INTPEND );								
	regvalue |=  ( TMRCONTROL_RUN     );
	TMRCONTROL0 = regvalue;
	
}

/*
*********************************************************************************************************
*                                       TICKER INITIALIZATION
*
* Description : This function is called to initialize uC/OS-II's tick source (typically a timer generating
*               interrupts every 1 to 100 mS).
*               
* Arguments   : none
*********************************************************************************************************
*/

static  void  Tmr_TickInit (void)
{
	;
}

/*
*********************************************************************************************************
*                                          TIMER #0 IRQ HANDLER
*
* Description : This function handles the timer interrupt that is used to generate TICKs for uC/OS-II.
*
* Arguments   : none
*********************************************************************************************************
*/

void  Tmr_TickISR_Handler (void)
{
	//CPU_INT32U cpu_sr;
	
	//CPU_CRITICAL_ENTER();
	
    	OSTimeTick();                                  /* Call uC/OS-II's OSTimeTick() to signal a tick   */
	Tmr_Clear_Intr();                                /* Clear timer1 interrupt source                   */

	//CPU_CRITICAL_EXIT();
}
