		REQUIRE8       	 
		PRESERVE8

		AREA   CPU, CODE, READONLY                     ; Name this block of code                                   ;

		GLOBAL load_context
		GLOBAL context_switch
		GLOBAL save_cpsr
		GLOBAL restore_cpsr
		GLOBAL enable_interrupt
		GLOBAL disable_interrupt
		
		IMPORT irq_sp


NO_INT		EQU     0xC0                         ; Mask used to disable interrupts (Both FIR and IRQ)
SVC32_MODE  EQU     0x13

		ENTRY


save_context FUNCTION
		MSR     CPSR_c, #(SVC32_MODE) ; Change to SVC mode
		ENDFUNC

		
context_switch FUNCTION
		;LDR		SP, [R0]				;Load a new stack pointer

		LDR		R4, =irq_sp
		LDR		SP,	[R4]

		MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; Change to SVC mode

		LDMFD	SP!, {R4-R12}
		STMFD	R0!, {R4-R12}
		LDMFD	SP!, {R4-R6}
		STMFD	R0!, {R4-R6}

;		MSR     CPSR_c, #(NO_INT | 32_MODE) ; Change to SVC mode

		MOV		SP, R1				;Load a new stack pointer

		; RESTORE NEW TASK'S CONTEXT
		LDMFD   SP!, {R4}               ;    Pop new task's CPSR
		MSR     SPSR_cxsf, R4


		LDMFD   SP!, {R0-R12,LR,PC}^    ;    Pop new task's context
		ENDFUNC



load_context FUNCTION
		MOV		SP, R0				;Load a new stack pointer

		; RESTORE NEW TASK'S CONTEXT
		LDMFD   SP!, {R4}               ;    Pop new task's CPSR
		MSR     SPSR_cxsf, R4


		LDMFD   SP!, {R0-R12,LR,PC}^    ;    Pop new task's context
		ENDFUNC

save_cpsr
        MRS     R0,CPSR                     ; Set IRQ and FIQ bits in CPSR to disable all interrupts
        ORR     R1,R0,#NO_INT
        MSR     CPSR_c,R1
        MRS     R1,CPSR                     ; Confirm that CPSR contains the proper interrupt disable flags
        AND     R1,R1,#NO_INT
        CMP     R1,#NO_INT
        BNE     save_cpsr					; Not properly disabled (try again)
        ;BX      LR                          ; Disabled, return the original CPSR contents in R0
        MOV     PC, LR



restore_cpsr
        MSR     CPSR_c,R0
        ;BX      LR
        MOV     PC, LR


;----------------- Enable I-bit & F-bit
enable_interrupt FUNCTION
	stmfd	sp!, {r0, lr}

	mrs	r0, cpsr
	bic	r0, r0, #0xc0
	msr	cpsr_cxsf, r0
	ldmfd	sp!, {r0, pc}

	ENDFUNC


;----------------- Disable I-bit & F-bit
disable_interrupt FUNCTION
	stmfd	sp!, {r0, lr}

	mrs	r0, cpsr
	orr	r0, r0, #0xc0
	msr	cpsr_cxsf, r0

	ldmfd	sp!, {r0, pc}

	ENDFUNC



	END
