		REQUIRE8       	 
		PRESERVE8


		 
		 AREA   INT_HANDLER, CODE, READONLY                     ; Name this block of code                                   ;
                
         GLOBAL interrupt_handler
		 GLOBAL __yield_thread
		 IMPORT	irq_handler
		 IMPORT schedule
		 IMPORT current_thread
		 IMPORT	irq_sp

NO_INT		EQU     0xC0                         ; Mask used to disable interrupts (Both FIR and IRQ)
NO_IRQ		EQU     0x80                         ; Mask used to disable interrupts (Both FIR and IRQ)
NO_FIQ		EQU     0x40                         ; Mask used to disable interrupts (Both FIR and IRQ)

SYS32_MODE EQU	0x1F
UND32_MODE EQU	0x1B
ABT32_MODE EQU	0x17
SVC32_MODE  EQU     0x13
FIQ32_MODE  EQU     0x11
IRQ32_MODE  EQU     0x12
USR32_MODE EQU	0x10

I_Bit           EQU     0x80 ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40 ; when F bit is set, FIQ is disabled


		ENTRY





interrupt_handler
		; Disable FIQ for a moment 
		;MSR     CPSR_c, #(NO_INT | IRQ32_MODE) ; Change to IRQ mode (to use the IRQ stack to handle interrupt)
		STMFD   SP!, {R1-R3}                   ; PUSH WORKING REGISTERS ONTO IRQ STACK
		MOV     R1, SP                         ; Save   IRQ stack pointer
		ADD     SP, SP,#12                     ; Adjust IRQ stack pointer
		SUB     R2, LR,#4                      ; Adjust PC for return address to task
		MRS     R3, SPSR                       ; Copy SPSR (i.e. interrupted task's CPSR) to R3
		MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; Change to SVC mode

		; SAVE TASK'S CONTEXT ONTO TASK'S STACK
		STMFD   SP!, {R2}                      ;    Push task's Return PC
		STMFD   SP!, {LR}                      ;    Push task's LR
		STMFD   SP!, {R4-R12}                  ;    Push task's R12-R4

		LDMFD   R1!, {R4-R6}                   ;    Move task's R1-R3 from IRQ stack to SVC stack
		STMFD   SP!, {R4-R6}
		STMFD   SP!, {R0}                      ;    Push task's R0    onto task's stack
		STMFD   SP!, {R3}                      ;    Push task's CPSR (i.e. IRQ's SPSR)

		LDR		R4, __current_thread
		LDR		R5, [R4]
		STR		SP, [R5]
		

OS_CPU_IRQ_ISR_1
		MSR     CPSR_c, #(NO_IRQ | IRQ32_MODE) ; Re-enable FIQ, Change to IRQ mode (to use the IRQ stack to handle interrupt)

		LDR		R4, __irq_sp
		STR		SP, [R4]


		;	// IRQ check
;		ldr	r2, =0x04000000		; timer interrupt check??
;		ldr	r0, [r1, #0]		; INT_ICIP
;		tst	r0, r2

		ldr	r0, =0x40D00000
		ldr r0, [r0]
		BL      irq_handler 

		MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; Change to SVC mode


		; RESTORE NEW TASK'S CONTEXT
		LDMFD   SP!, {R4}                      ;    Pop new task's CPSR
		MSR     SPSR_cxsf, R4

		LDMFD   SP!, {R0-R12,LR,PC}^           ;    Pop new task's context







__yield_thread
;		STMFD	SP!, {LR}
;		MSR     CPSR_c, #(NO_INT | IRQ32_MODE) ; Change to SVC mode
;		STMFD   SP!, {R1-R2}                   ; PUSH WORKING REGISTERS ONTO IRQ STACK
;		MOV     R1, SP                         ; Save   IRQ stack pointer
;		ADD     SP, SP,#12                     ; Adjust IRQ stack pointer
;		MOV     R2, LR                      ; Adjust PC for return address to task
;		MRS     R2, SPSR                       ; Copy SPSR (i.e. interrupted task's CPSR) to R3
;		MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; Change to SVC mode

		STMFD	SP!, {R1 - R3}
		MOV		R1, SP
		ADD		SP, SP, #12
		MOV     R2, LR                      ; Adjust PC for return address to task
		MRS		R3, CPSR
		MSR		CPSR_c, #(NO_INT | SVC32_MODE)



		; SAVE TASK'S CONTEXT ONTO TASK'S STACK
		STMFD   SP!, {R2}                      ;    Push task's Return PC
		STMFD   SP!, {LR}                      ;    Push task's LR
		STMFD   SP!, {R4-R12}                  ;    Push task's R12-R4

		LDMFD   R1!, {R4-R6}                   ;    Move task's R1-R3 from IRQ stack to SVC stack
		STMFD   SP!, {R4-R6}
		STMFD   SP!, {R0}                      ;    Push task's R0    onto task's stack
		STMFD   SP!, {R3}                      ;    Push task's CPSR (i.e. IRQ's SPSR)

		LDR		R4, __current_thread
		LDR		R5, [R4]
		STR		SP, [R5]
		
		MSR     CPSR_c, #(NO_IRQ | IRQ32_MODE) ; Re-enable FIQ, Change to IRQ mode (to use the IRQ stack to handle interrupt)

		LDR		R4, __irq_sp
		STR		SP, [R4]

		BL      schedule

		MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; Change to SVC mode


		; RESTORE NEW TASK'S CONTEXT
		LDMFD   SP!, {R4}                      ;    Pop new task's CPSR
		MSR     SPSR_cxsf, R4

		LDMFD   SP!, {R0-R12,LR,PC}^           ;    Pop new task's context



__current_thread
		DCD		current_thread
__irq_sp
		DCD		irq_sp

		END

