
#include <migos_stdio.h>
#include <string.h>
#include <mem.h>

#include <util.h>

#include <fs/nand.h>
#include <fs/romfs.h>

fatfs		g_fatfs;
rom_storage 	g_st;



int init_fs( void )
{
	char	buf[512], *cur;
	char	mb[20] = {0,};
	int i, sectors = uSYS_FS_IMAGE_SIZE / 512;
	if( uSYS_FS_IMAGE_SIZE % 512 ) sectors ++;
	kernel_printf("\r\ninit_fs() ...\r\n");

	g_fatfs.type = FS_UNKNOWN;

	itomb( uSYS_FS_IMAGE_SIZE, mb );
	kernel_printf("  uSYS_FS_RAM_BASE : 0x%x\r\n  uSYS_FS_IMAGE_SIZE : %s\r\n  uSYS_FS_NAND_BASE_SECTOR : %d\r\n", uSYS_FS_RAM_BASE, mb, uSYS_FS_NAND_BASE_SECTOR );
	kernel_printf("  Nand Rom Image Copy ( %d sectors ) ", sectors );
	cur = (BYTE*)uSYS_FS_RAM_BASE;               //0xA0800000
	// inspect bootsector firstly
	NAND_ReadSector( &Nand1, uSYS_FS_NAND_BASE_SECTOR, buf );
	if( is_bootsector( buf ) == 0 )
	{
		kernel_printf("\r\n  Invalid bootsector signature... init_fs() [fail]\r\n");
		return 0;
	}

	for( i = 0 ; i < sectors; i ++ )
	{ 
		NAND_ReadSector( &Nand1, i+uSYS_FS_NAND_BASE_SECTOR, buf );
		memcpy( cur, buf, 512 );
		cur+=512;
		if( (i % 1024) == 0) kernel_printf("."); 
	}
	kernel_printf("[ok]\r\n");

	init_rom_storage( &g_st, (void*)uSYS_FS_RAM_BASE, uSYS_FS_IMAGE_SIZE ); 
	
	init_fatfs( &g_fatfs, g_st.sio );	
	
	if( g_fatfs.type == FS_FAT12 )
		kernel_printf("  Filesystem type : FAT12 \r\n" );
	else
	if( g_fatfs.type == FS_FAT16 )
		kernel_printf("  Filesystem type : FAT16 \r\n" );
	else
	{
		kernel_printf("  Unkouwn Filesystem ... init_fs() [fail] \r\n" );
		return 0;
	}

	return 1;
}

int rom_storage_read_sector( sector_io* sio, int sector, int offset, BYTE* buf, int size )
{
	rom_storage* st = sio->storage;
	char *src = (char *)st->address + (sector *sio->bytes_per_sector );
	int read_size = sio->bytes_per_sector - offset;

	src += offset;
	if( size < read_size )
		read_size = size;

	memcpy( buf, src, read_size );

	return read_size;
}

int rom_storage_write_sector( sector_io* sio, int sector, int offset, char* buf, int size )
{
	rom_storage* st = sio->storage;
	char* src = (BYTE*)st->address + (sector * sio->bytes_per_sector);
	int write_size = sio->bytes_per_sector - offset;

	src += offset;
	if( size < sio->bytes_per_sector )
		write_size = size;

	memcpy( src, buf, write_size );

	return write_size;
}

int init_rom_storage( rom_storage* st, void* address, int size )
{
	st->address = address;
	st->size = size;
	st->sio.storage = st;
	st->sio.read = rom_storage_read_sector;
	st->sio.write = rom_storage_write_sector;
	return 1;
}


