/*Headers for MIGOS Kernel */
#include <migos_stdio.h>
#include<kernel/thread.h>
#include <kernel/process.h>
#include <kernel/schedule.h>
#include <types.h>
#include<kernel/jvm.h>

/*Headers for Java Virtual Machine  */
#include <global.h>




struct jproc_exec *current_jvm_env_g = NULL;


int init_jvm (INSTANCE_CLASS main_class, int argc, char* argv[],ARRAY *arguments) {
    int result;

	kernel_printf("Initializing Java Virtual Machine \n");
	kernel_printf("init_jvm : argc - %d , argv[0] - \n" , argc);

    JamEnabled = FALSE;
    JamRepeat = FALSE;

        
			/* Set classpath */
#ifdef ENABLE_FS
	else if ((strcmp(argv[1], "-classpath") == 0) && argc > 2) {
            if (JamEnabled) {
                fprintf(stderr, KVM_MSG_CANT_COMBINE_CLASSPATH_OPTION_WITH_JAM_OPTION);
                exit(1);
            }
            UserClassPath = argv[2];
            argv+=2; argc -=2;
#endif


    /* Skip program name */
    argc--;
    argv++;

    if (UserClassPath == NULL && !JamEnabled) { 
        UserClassPath = getenv("classpath");
        if (UserClassPath == NULL) { 
            /* Just in case environment variable reading is case sensitive */
            UserClassPath = getenv("CLASSPATH");
            /* Use "." as the default classpath */
            if (UserClassPath == NULL) { 
                UserClassPath = ".";
            }
        }
    }


    {
 


			/* In principle, KVM can run with just a few kilobytes */
			/* of heap space.  We use 16 kilobytes as the minimum  */
			/* as that value is useful for some test cases.        */
			/* The maximum heap size allowed by the KVM garbage    */
			/* collector is 64 megabytes.  In practice, the        */
			/* collector is optimized only for small heaps, and    */
			/* is likely to have noticeable GC pauses with heaps   */
			/* larger than a few megabytes */
			RequestedHeapSize = DEFAULTHEAPSIZE;

			/* Make sure the heap size is divisible by four */
			RequestedHeapSize -= RequestedHeapSize%CELL;



//		RequestedHeapSize = 16 * 1024 - ((16 % 1024 )%CELL);
        /* Call the portable KVM startup routine */
        result = init_migvm(main_class, argc, argv,arguments);

        /* If no classfile was provided, print help text */
        if (result == -1) printHelpText();
    }

    return result;
}



int start_jvm_interpreter(void)
{
	int returnValue ;
	struct process *current = get_current_process();
	kernel_printf("Running the interpreter\n");
    TRY {
        VM_START {
			if(current->state == INTERPRETING){
					kernel_printf("Restoring java execution environment........\n");
					loadExecutionEnvironment(CurrentThread);
					FastInterpret();
			}else {
					/*It may be first java thread interpreting. */
					kernel_printf("This is the first java thread interpreting....\n");
					current->state = INTERPRETING;
					/* Start the interpreter */
					Interpret();
			}
        } VM_FINISH(value) {
            returnValue = value;
        } VM_END_FINISH
    } CATCH(e) {
        /* Any uncaught C-level exception above will transfer control here */
			kernel_printf("Exception occured during interpretation \n");
    } END_CATCH

    return returnValue;
}

struct thread *build_java_thread_system(INSTANCE_CLASS main_class, ARRAY argumentsArg)
{
		struct thread *main_phy_thread = NULL;
		kernel_printf("Building java thread system...........\n");

		InitializeThreading(main_class, argumentsArg, &main_phy_thread);

		if(main_phy_thread == NULL){
				kernel_printf("build_java_thread_system : cannot create a physical thread \n");
				goto fail_create_thread;
		}


		return main_phy_thread;
fail_create_thread:
		return NULL;

}


THREAD
create_java_thread(JAVATHREAD_HANDLE java_thread_H, struct thread **phy_thread)
{
		kernel_printf("Creating java thread\n");
		*phy_thread = (struct thread *)k_malloc(sizeof(struct thread));
		THREAD new_java_thread;

		if(*phy_thread == NULL){
				errno = -ENOMEM;
				goto out_of_memory;
		}
		kernel_printf("Building java thread\n");
		new_java_thread = BuildThread(java_thread_H);

		(*phy_thread)->java_thread = new_java_thread;
		new_java_thread->phy_thread = *phy_thread;	/*Set physical thread performing this java thread */

		initialize_jthread(0, start_jvm_interpreter, 0, get_current_process(), (*phy_thread));
		

//		strcpy(new_thread->name, name);
//		initialize_thread(thread_id, entry, priority, new_thread);
		add_thread_to_rq(*phy_thread, get_rq());
		
		return new_java_thread;

out_of_memory:
		return NULL;

}




