/***************************************************************************** 
 * File        : mm.c
 * Date        : 2007/04/04
 * Author      : Sun Joon Gyu
 * Modified    : 2007/04/04
 * Description : Rutines for memory management
 * ***************************************************************************/


#ifndef MM_TEST
#include<migos_stdio.h>
#include<arch/cpu.h>
#include<kernel/mm.h>
#include<kernel/process.h>
//#include<kernel/thread.h>
#include<kernel/error.h>
#include<list.h>
//#include<drivers/serial/serial.h>


#else
#include"mm.h"
#include"types.h"
#include"list.h"
struct mm_clt *pThreadMemList_g;	/* a first block for heap spaces */
#endif //MM_TEST


struct mm_clt *pHeapListHead_g;	/* a head for memory controller list*/
struct mm_clt *pFirstHeapHead_g;	/* a first block for heap spaces */



struct mm_clt *pHeapListTail_g;	/* It indicates the end of heap */

//int8s heap[SIZE_OF_HEAP] __attribute__((aligned(64))) ;		/* only for testing this rutines */




/******************************************************************
 * Function : Kern_MemInit()
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Operation : It simply initializes memory control block for heap.
 *             At first, it creates a header controlling memory spaces.
 *             And it puts the header into the top of the heap.
 *****************************************************************/

bool kmem_init(void)
{
	kernel_printf("Initializing kernel memory.......... \n");

	pHeapListHead_g = (struct mm_clt *)START_HEAP_ADDR;

	kernel_printf("Start heap address : %x \n", START_HEAP_ADDR);
	kernel_printf("End heap address : %x \n", END_HEAP_ADDR);

	//pHeapListTail_g = ((struct mm_clt *)END_HEAP_ADDR )-1;
	INIT_LIST_HEAD(&pHeapListHead_g->memLink);
	INIT_LIST_HEAD(&pHeapListHead_g->allocLink);
	INIT_LIST_HEAD(&pHeapListHead_g->freeMemLink);

	pHeapListHead_g->status = INVALID;

	struct mm_clt *memHeader = pFirstHeapHead_g = 
			(struct mm_clt *)(((int8u *)pHeapListHead_g) + BLOCK_SIZE);	/*Next block */
	/* Initializing a header of heap*/
	INIT_LIST_HEAD(&memHeader->memLink);
	INIT_LIST_HEAD(&memHeader->allocLink);
	INIT_LIST_HEAD(&memHeader->freeMemLink);

	memHeader->status = AVAILABLE;

	/* should subtract the size of one memory block and a list header*/

	memHeader->nblocks = (int32s) (((int8s *)END_HEAP_ADDR - 
							(int8s *)pFirstHeapHead_g) >> BLOCK_SIZE_SHIFT) - 1; 	

	kernel_printf("Total heap size: %d blocks \n", memHeader->nblocks);


	if(memHeader->nblocks == 0)
		return -ENOMEM;

	list_add(&memHeader->freeMemLink, &pHeapListHead_g->freeMemLink);
	list_add(&memHeader->memLink, &pHeapListHead_g->memLink);

#ifdef MM_TEST
	pThreadMemList_g = malloc(sizeof(struct mm_clt));
	INIT_LIST_HEAD(&pThreadMemList_g->memLink);
	INIT_LIST_HEAD(&pThreadMemList_g->allocLink);
#endif
	return SUCCESS;

}



/***********************************************************************************************
 * Function : find_wanted_blk(size_t size)
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Arguments : size_t size - the number of bytes desired
 * Return type : struct mm_clt *- it returns an address indicating specific memory which is available.
 * Operation : Sequencially, It goes around memory control blocks until it finds a memory control 
 *             block which can be allowed to put a desired size of bytes
 ***********************************************************************************************/


static struct mm_clt * 
find_wanted_blk(size_t size)
{
	int32u nblocks = 0;
	struct mm_clt *iter = NULL;
	struct mm_clt *ret = NULL;
	int cpu_sr;

	struct mm_clt * pHead = (struct mm_clt *)pHeapListHead_g;

	size += (BLOCK_SIZE-1);
	nblocks = size >> BLOCK_SIZE_SHIFT;

	enter_critical_section();
	list_for_each_entry(iter,&pHead->freeMemLink,freeMemLink){
	
			if(iter->nblocks >= nblocks){
					ret = iter;
					break;
			}
			

	}list_end_for;
	exit_critical_section();

	return ret;
		
}





/*************************************************************************************************
 * Function : get_mem_blk()
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Arguments : struct mm_clt * pBlkAddr - the address picked in the function find_wanted_blk() 
 *             size_t size - the number of bytes desired
 * Return type : int8s - It returns a condition value indicating if this rutines completed successfully or not
 * Operation : It adjust neibor's links and create new memory control block to control extra memory spaces
 * ************************************************************************************************/


static int8s 
get_mem_blk(struct mm_clt * pBlkAddr,size_t size)
{

	int32s nblocks;
	size += (BLOCK_SIZE-1);
	nblocks = size >> BLOCK_SIZE_SHIFT;
	int8s ret;
	int cpu_sr;
	
	enter_critical_section();
	/* Just in case there is something wrong with memory block pointer */
	if((pBlkAddr->nblocks) == nblocks){

		pBlkAddr->status = ALLOCATED;
		pBlkAddr->threadID = get_current_process()->process_id;

#ifndef MM_TEST
		list_del_init(&pBlkAddr->freeMemLink);
		list_del_init(&pBlkAddr->allocLink);
		list_add(&pBlkAddr->allocLink, &get_current_process()->mem_header.allocLink);

#else
		list_del(&pBlkAddr->freeMemLink);
		list_add(&pBlkAddr->allocLink, &pThreadMemList_g->allocLink);
#endif
		ret = SUCCESS;
	}
	else if((pBlkAddr->nblocks) > nblocks){
		pBlkAddr->status = ALLOCATED;
		size_t extraBlkSize = pBlkAddr->nblocks - nblocks - 1;  
			/* should subtract an additional memory control header */

		struct mm_clt * pExtraMemBlk = (struct mm_clt *)((int8s *)pBlkAddr
				+ ((nblocks + 1) << BLOCK_SIZE_SHIFT)); 

		INIT_LIST_HEAD(&pExtraMemBlk->memLink);
		INIT_LIST_HEAD(&pExtraMemBlk->allocLink);
		INIT_LIST_HEAD(&pExtraMemBlk->freeMemLink);
		
		pExtraMemBlk->nblocks = extraBlkSize;
		pExtraMemBlk->status = AVAILABLE;


		pBlkAddr->nblocks = nblocks;
		pBlkAddr->status = ALLOCATED;

		list_add(&pExtraMemBlk->memLink,&pBlkAddr->memLink);
		list_add(&pExtraMemBlk->freeMemLink,&pBlkAddr->freeMemLink);
		list_del_init(&pBlkAddr->freeMemLink);

#ifndef MM_TEST
		list_del_init(&pBlkAddr->allocLink);
		list_add(&pBlkAddr->allocLink, &get_current_process()->mem_header.allocLink);
		pBlkAddr->threadID = get_current_process()->process_id;
#else
		list_add(&pBlkAddr->allocLink, &pThreadMemList_g->allocLink);
#endif
		ret = SUCCESS;

	}
	else{

		ret = -ENOMEM;
	}

	exit_critical_section();
//CRITICAL_SECTION_LEAVE();

	return ret;	

}	




#if 0

/*****************************************************************************
 * Function : Kern_BlkTblRegister()
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Arguments : int32u *pReqVar - a pointer variable which is asking the 
 * 					bytes of memory spaces
 *             struct mm_clt * - A address pointing at the location which is just 
 *             		    allocated
 *
 * Return type : int8s - It returns a condition value indicating if this rutines 
 * 			completed successfully of not
 * Operation : It registers a address of a desired pointer variable into memory 
 * 		control block and also put the control block into the list in 
 * 		thread control block
 ******************************************************************************/

int8s 
Kern_BlkTblRegister(struct mm_clt * pBlkAddr)
{
CRITICAL_SECTION_ENTER();
	if(pCurrThread_g!= NULL){
		if(pCurrThread_g->pMemList != NULL){
			pBlkAddr->pAllocNext = pCurrThread_g->pMemList;
			pCurrThread_g->pMemList = pBlkAddr;
		}else{
			pBlkAddr->pAllocNext = NULL;
			pCurrThread_g->pMemList = pBlkAddr;

		}
	}

	pBlkAddr->pVar = pReqVar;
CRITICAL_SECTION_LEAVE();
	return SUCCESS;
}


#endif


/******************************************************************************
 * Function :  k_free()
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Arguments : void *p - the address of memory wanted to be released
 * Return type : int8s - It returns a condition value indicating if this rutines 
 			completed successfully of not
 * Operation : it simply makes the bunch of memory blocks owned 
 		by the thread free. 
 ******************************************************************************/

int8s k_free(void *p)
{

	bool found = 0;
	int cpu_sr;
	int8s ret;


	struct mm_clt *release = (struct mm_clt *) 
			(( ((int32u)p) - (BLOCK_SIZE - 1)) >> BLOCK_SIZE_SHIFT);
	release = (struct mm_clt *) (((int32u)release) << BLOCK_SIZE_SHIFT);
#ifdef MM_TEST
	struct mm_clt * mem_head = pThreadMemList_g;
#else
	struct mm_clt * mem_head = &get_current_process()->mem_header;
#endif


	struct mm_clt *iter;

	list_for_each_entry(iter,&mem_head->allocLink,allocLink){
			if(iter == release){
					found=1;
					list_del_init(&release->allocLink);
					goto do_free;
			}
	}list_end_for;

	/*This is worst case. We should search all the process' memory header. */
	mem_head = pHeapListHead_g;

	list_for_each_entry(iter,&mem_head->memLink,memLink){
			if(iter == release){
					found=1;
					list_del_init(&release->allocLink);
					break;
			}
	}list_end_for;

do_free:

	enter_critical_section();

	if(found){
			struct mm_clt *prev_mem = list_entry(release->memLink.prev, struct mm_clt, memLink);
			struct mm_clt *next_mem = list_entry(release->memLink.next, struct mm_clt, memLink);



			if(prev_mem->status == AVAILABLE){
					merge_mem_blk(prev_mem, release , BACKWARD);

					if(next_mem->status == AVAILABLE){
							merge_mem_blk(prev_mem, next_mem, FORWARD);
					}
			}else if(next_mem->status == AVAILABLE){
							merge_mem_blk(release, next_mem, FORWARD);
			}else{

					release->status = AVAILABLE;
					list_del_init(& release->freeMemLink );
					list_add(&release->freeMemLink,&pHeapListHead_g->freeMemLink);




#if 0
					release->status = AVAILABLE;
					struct mm_clt *iter;
					list_for_each_entry(iter, &release->memLink, memLink){
							list_del_init(&release->freeMemLink);
							list_add_tail(&release->freeMemLink,&iter->freeMemLink);
							break;

					}list_end_for;
#endif

			}


			ret = SUCCESS;
	} else
			ret = 0;

	exit_critical_section();

	return ret;
	
}



/*************************************************************************************************
 * Function :  merge_mem_blk()
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Arguments : struct mm_clt * pPrevBlk - 
 * Return type : int8s - It returns a condition value indicating if this rutines completed successfully of not
 * Operation : it simply makes the bunch of memory blocks owned by the thread free. 
 ***************************************************************************************************/
#if 1
static bool 
merge_mem_blk(struct mm_clt * pPrevBlk, struct mm_clt * pNextBlk,int8u dir)
{

		int32u mergedSize = (pPrevBlk->nblocks + pNextBlk->nblocks) + 1;	/* include one block used as a header of memory blocks */

		list_del_init(&pNextBlk->freeMemLink);
		list_del_init(&pNextBlk->memLink);
		pPrevBlk->nblocks= mergedSize;
		pPrevBlk->status = AVAILABLE;

		list_del_init(&pPrevBlk->freeMemLink);
		list_add(&pPrevBlk->freeMemLink,&pHeapListHead_g->freeMemLink);

#if 0
	int32u mergedSize = (pPrevBlk->nblocks + pNextBlk->nblocks) + 1;	/* include one block used as a header of memory blocks */

	struct list_head *next_blk_list_free = &pNextBlk->freeMemLink;
	struct list_head *next_blk_list_mem = &pNextBlk->memLink;

	struct list_head *prev_blk_list_free = &pPrevBlk->freeMemLink;
	struct list_head *prev_blk_list_mem = &pPrevBlk->memLink;

	next_blk_list_free->next->prev = prev_blk_list_free;
	prev_blk_list_free->next = next_blk_list_free->next;

	if(dir == BACKWARD){
			struct mm_clt *pprev = list_entry(prev_blk_list_mem->prev, struct mm_clt, memLink);
			list_del_init(prev_blk_list_free);
			list_add(prev_blk_list_free, &pprev->freeMemLink);
	}

	list_del_init(next_blk_list_mem);
	//list_add(next_blk_list_free, prev_blk_list_free);
	pPrevBlk->nblocks= mergedSize;
	pPrevBlk->status = AVAILABLE;

#endif
#ifndef MM_TEST
	pPrevBlk->threadID = get_current_process()->process_id;	
#endif

	return SUCCESS;	

}
#endif 

int32u *k_malloc(size_t size)
{
		struct mm_clt *pBlkAddr;
		int cpu_sr;

	if( (pBlkAddr = find_wanted_blk(size)) == NULL){
			return NULL;
		/* The thread tried to get memory from heap area should be blocked!! */
		/* DbgPrint("Not enough memory in Heap area\n"); */
	}
	if(get_mem_blk(pBlkAddr, size) != SUCCESS){
			return NULL;
		/* DbgPrint("Something wrong!!\n"); */
	}

	return (int32u *)((int8s *)pBlkAddr + BLOCK_SIZE);	/* return the address except memory header */
}

#if 1

int32u
get_free_mem_info()
{
		int32u sum = 0;
		struct mm_clt *head = NULL;
		struct mm_clt *iter;

		head = pHeapListHead_g;
		
		if(!head)
				return 0;

		list_for_each_entry(iter, &(head->freeMemLink),freeMemLink){
				kernel_printf("iter : %d  proc : %d \n", iter->nblocks, iter->threadID);
				sum += iter->nblocks;
		}list_end_for;


		return sum;
}


int32u
get_alloc_mem_info(struct mm_clt *head)
{
		int32u sum = 0;
		struct mm_clt *iter;

		if(!head)
				return 0;

		list_for_each_entry(iter, &(head->allocLink),allocLink){
				sum += iter->nblocks;
		}list_end_for;

		return sum;
}

int32u
get_total_mem_info(struct mm_clt *head)
{
		int32u sum = 0;
		struct mm_clt *iter;

		if(!head)
				return 0;

		list_for_each_entry(iter, &(head->memLink),memLink){
				sum += iter->nblocks;
		}list_end_for;


		return sum;
}
#endif


#ifdef MM_TEST
int32s 
main()
{
	bool k = Kern_MemInit();

	int *a1 = k_malloc(10);
	int *a2 = k_malloc(100);
	int *a3 = k_malloc(1000);
	int *a4 = k_malloc(10000);
	int *a5 = k_malloc(100000);
	int *a7 = k_malloc(100000);
	int *a8 = k_malloc(100000);
	int *a9 = k_malloc(100000);
	int *a10 = k_malloc(100000);

	struct mm_clt *iter;
	k_free(a4);
	k_free(a2);
	k_free(a1);
	k_free(a3);
	k_free(a5);



	int *b1 = k_malloc(30);
	int *b2 = k_malloc(2048);
	k_free(b1);
	b1 = k_malloc(10000);
	k_free(b2);
	k_free(b1);


	
	list_for_each_entry(iter, &pHeapListHead_g->memLink, memLink){
			kernel_printf("memLink\n");

			kernel_printf("iter nblocks : %d \n", iter->nblocks);
	}list_end_for;
	list_for_each_entry(iter, &pHeapListHead_g->allocLink, allocLink){
			kernel_printf("allocLink\n");
			kernel_printf("iter nblocks : %d \n", iter->nblocks);
	}list_end_for;
#if 1
	list_for_each_entry(iter, &pHeapListHead_g->freeMemLink, freeMemLink){
			kernel_printf("freeMemLink\n");
			kernel_printf("iter nblocks : %d \n", iter->nblocks);
	}list_end_for;
#endif 


}

#endif //MM_TEST
