#include<arch/cpu.h>
#include <migos_stdio.h>
#include <string.h>
#include <kernel/process.h>
#include <kernel/thread.h>
#include <kernel/mm.h>
#include <list.h>
#include <kernel/jvm.h>
#include <application.h>

#include <global.h>

struct process process_list_head_g;
struct process init_process;
struct process *current_process = &init_process;

/*init_jvm will be replaced with a Real JVM Initialization Routine */


/*************************************************************************** 
 * Function    : setup_swapper
 * Date        : 2008/08/23
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int setup_swapper(void)
{
		kernel_printf("Setting up swappers........ \n");

		INIT_LIST_HEAD(&process_list_head_g.thread_list);
		INIT_LIST_HEAD(&process_list_head_g.proc_list);
		INIT_LIST_HEAD(&process_list_head_g.mem_header.memLink);
		INIT_LIST_HEAD(&process_list_head_g.mem_header.freeMemLink);
		INIT_LIST_HEAD(&process_list_head_g.mem_header.allocLink);
		
		
		setup_swapper_process();
		setup_swapper_thread();

		return SUCCESS;
}




/*************************************************************************** 
 * Function    : setup_swapper_process
 * Date        : 2008/08/23
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int setup_swapper_process(void)
{
		kernel_printf("Initializing a swapper process ........ \n");

		init_process.process_id = 0;
		strcpy(init_process.name,"swapper process");
		init_process.process_type = KERNEL_PROCESS;
		INIT_LIST_HEAD(&init_process.thread_list);
		INIT_LIST_HEAD(&(init_process.file_table.file_list));
		init_process.fd_count = 0;
		INIT_LIST_HEAD(&init_process.proc_list);
		INIT_LIST_HEAD(&init_process.mem_header.memLink);
		INIT_LIST_HEAD(&init_process.mem_header.freeMemLink);
		INIT_LIST_HEAD(&init_process.mem_header.allocLink);
		current_process = &init_process;


		list_add(&(init_process.proc_list),&get_process_list_head()->proc_list);





		return SUCCESS;
}



/*************************************************************************** 
 * Function    : setup_exec_env
 * Date        : 2008/08/23
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int setup_exec_env(struct process *new_process, char *main_class)
{
		kernel_printf("current process addr : %x \n", get_current_process()); 
		struct jproc_exec *new_jexec_env = (struct jproc_exec *) k_malloc(sizeof(struct jproc_exec));
		if(new_jexec_env == NULL) {
				errno = -ENOMEM;
				goto no_mem;
		}
		kernel_printf("current process addr : %x \n", get_current_process()); 
		kernel_printf("new_jexec_env addr : %p \n",new_jexec_env);

		new_process->java_exec_env = new_jexec_env;
		new_jexec_env->Process = new_process;	

		/*TODO:
		 * initialize GlobalRoots, Class table, .... */

		return SUCCESS;

no_mem:
		return errno;


}


/*************************************************************************** 
 * Function    : create_jvm_process
 * Date        : 2008/08/23
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

struct process *create_jvm_process(int process_id, char *main_class, int priority)
{
		struct thread *application_thread = NULL;
		char argv[][10] = { 
				"kvm",
				"Hello"
		};

		struct process *new_process = (struct process *)k_malloc(sizeof(struct process));
		if(new_process == NULL){
				errno = -ENOMEM;
				goto no_mem;
		}




		new_process->process_id = process_id;
		new_process->process_type = JAVA_PROCESS;
		new_process->state = CREATED;
		new_process->alive_thread_count = 0;
		strcpy(new_process->name, main_class);

		INIT_LIST_HEAD(&new_process->thread_list);
		INIT_LIST_HEAD(&new_process->mem_header.allocLink);
		INIT_LIST_HEAD(&new_process->mem_header.memLink);
		INIT_LIST_HEAD(&new_process->mem_header.freeMemLink);
		INIT_LIST_HEAD(&new_process->thread_list);
		INIT_LIST_HEAD(&new_process->proc_list);
		INIT_LIST_HEAD(&new_process->file_table.file_list);


		INSTANCE_CLASS app_class = find_app_class_obj(main_class);

		if(app_class != NULL){
				int cpu_sr;
				ARRAY arguments;
				struct process *saved = current_process;
				
				enter_critical_section();
				kernel_printf("prev process addr : %x \n", get_current_process()); 
				current_process = new_process;
				kernel_printf("current process addr : %x \n", get_current_process()); 
				setup_exec_env(new_process,main_class);



				init_jvm(app_class, 2, argv, &arguments);
				application_thread = build_java_thread_system(app_class,arguments);

				current_process = saved;
				exit_critical_section();

				initializeClass(JavaLangOutOfMemoryError);
				initializeClass(JavaLangSystem);
				initializeClass(JavaLangString);
				initializeClass(JavaLangThread);
				initializeClass(JavaLangClass);



				if(application_thread == NULL){
						kernel_printf("create_jvm_process: can't create application thread \n");
						goto fail_create_thread;
				}
				new_process->main_thread = application_thread;
				list_add(&(new_process->proc_list),&(get_process_list_head()->proc_list));
				//start_jvm_interpreter(app_class);
		}




		return new_process;


fail_create_thread: k_free(new_process);
no_mem:
		return NULL;

}



struct mm_clt *
get_process_mm_info(struct process *proc)
{
		if(proc)
				return &(proc->mem_header);
}



