#include <migos_stdio.h>

#include <kernel/process.h>
#include <kernel/thread.h>
#include <kernel/schedule.h>
#include <kernel/jvm.h>

#include <arch/cpu.h>




#define SAVE_SVC_CONTEXT() __asm__ __volatile__( \
	"sub	sp, sp, #64\n\t" \
	"mrs	r0, cpsr\n\t" \
	"bic    r0, r0, #0x80\n\t" \
	"str	r0, [sp, #56]\n\t" \
	"stmia   sp, {r0 -  r12, lr}\n\t" \
	:::"r0","sp" \
)


#define RESTORE_SVC_CONTEXT() __asm__ __volatile__( \
	"ldr	r0, [sp, #56]\n\t" \
	"msr	cpsr, r0\n\t" \
	"ldmia	sp, {r0 - r12, lr}\n\t" \
	"add	sp, sp, #60\n\t" \
	:::"r0", "lr" \
)


#define ENABLE_IRQ() __asm__ __volatile__( \
	"mrs	r0, cpsr\n\t" \
	"bic	r0, r0, #0x80\n\t" \
	"msr	cpsr, r0\n" \
	:::"r0" \
)


#define DISABLE_IRQ() __asm__ __volatile__( \
	"mrs	r0, cpsr\n\t" \
	"orr	r0, r0, #0x80\n\t" \
	"msr	cpsr, r0\n" \
	:::"r0" \
)



struct thread rt_rqueue_g;	/* Ready queue for rt threads*/
struct thread rqueue_g;	/* Ready queue for threads*/


struct thread waitq_g;	/* Ready queue for threads*/






struct thread *find_next_thread()
{

		int found = 0;
		struct thread *next = NULL;

#if RR_SCHED
		/* Round Robin Scheduling */
		if(!list_empty(&get_rq()->rdy_list)){
				found = 1;
				next = list_entry(get_rq()->rdy_list.next, struct thread, rdy_list);
		}
		
		if(found){
				return next;
		}else{
				return NULL;
		}



#elif PRIO_SCHED
		/* Priority Scheduling */

		/*Real-time thread exist?? */
		if(!list_empty(&get_rt_rq()->rdy_list)){

				struct thread *iter;
				int max_priority = MAX_THREAD_PRIO;

				list_for_each_entry(iter, &get_rt_rq()->rdy_list, rdy_list){
						if(max_priority > iter->priority ){
								
								max_priority = iter->priority;
								next = iter;
								found = 1;

						}
				}list_end_for;

				if(found){

						if( get_current_thread()->priority >= next->priority){
								return next;

						}else 
								return NULL;
				}

		/*Normal thread exist?? */
		}
		
		if(!list_empty(&get_rq()->rdy_list)){
				found = 1;
				next = list_entry(get_rq()->rdy_list.next, struct thread, rdy_list);
		}
		
		if(found){
				return next;
		}else{
				return NULL;
		}



#else
		/*Panic */
		kernel_printf("Scheduling policy Not defined!!\n");
#endif

}

void sched_hook(struct thread *prev_thread)
{

		prev_thread->timeslice = DEFAULT_TIMESLICE; 
}


#include <arch/intgrt_struct.h>
extern struct irq irq0;
extern struct timer_struct timer1;
#define IRQEnableClear irq0.EnableClear
#define IRQEnableSet irq0.EnableSet
#define Timer1Clear timer1.Clear

void schedule(void)
{

		struct thread *prev_thread = get_current_thread();
		struct process *prev_proc = prev_thread->parent_proc;

		struct thread *next_thread = find_next_thread();
		struct process *next_proc = NULL;

		sched_hook(get_current_thread());

		if(next_thread == NULL){
				/*No thread to be runnable */
				kernel_printf("no schedule!! \n");
				goto no_sched;
		}else {

				if(prev_proc->process_type == JAVA_PROCESS){
						/*TODO : 
						 * Save Virtual Machine Execution Environment. */
						storeExecutionEnvironment(prev_thread->java_thread);
				}

				next_proc = next_thread->parent_proc;
		}

		if(prev_proc != next_proc){	
				/*Needs to switch process' and thread's context */

				switch(next_proc->process_type){
						case KERNEL_PROCESS:

								current_process = next_proc;

								add_thread_to_rq(next_thread,get_rq());
								current_thread = next_thread;

								if(current_thread->state == INITIAL){
										current_thread->state = RUNNING;
								}

								context_switch(prev_thread->stack_top, next_thread->stack_top);

								break;

						case JAVA_PROCESS:
								current_process = next_proc;
								current_jvm_env_g = next_proc->java_exec_env;
								add_thread_to_rq(next_thread,get_rq());
								current_thread = next_thread;
								/*TODO :
								 * Do java thread context switching */

								CurrentThread = current_thread->java_thread;	/*Set java thread to run */
								if(current_thread->state == INITIAL){
										current_thread->state = RUNNING;
								}else {
										loadExecutionEnvironment(CurrentThread);
								}
								context_switch(prev_thread->stack_top, next_thread->stack_top);

								break;

				}


		}else {
				/* Only need to do is to switch thread's context */
				add_thread_to_rq(next_thread,get_rq());
				current_thread = next_thread;

				if(current_thread->thread_type == JAVA_THREAD){
						if(current_thread->state == INITIAL){
								current_thread->state = RUNNING;
						}else {
//								kernel_printf("Restoring previous java execution context.\n");
								loadExecutionEnvironment(current_thread->java_thread);
						}
						CurrentThread = current_thread->java_thread;	/*Set java thread to run */
				}else{
						if(current_thread->state == INITIAL){
								current_thread->state = RUNNING;
						}
				}



				context_switch(prev_thread->stack_top, next_thread->stack_top);
		}
no_sched:

}

void init_sched(void)
{
		migos_printf("Initializing scheduler.......\n");

		INIT_LIST_HEAD(&rt_rqueue_g.rdy_list);
		INIT_LIST_HEAD(&rqueue_g.rdy_list);
		INIT_LIST_HEAD(&waitq_g.wait_list);

}




void start_sched(void)
{
		struct thread *thread_to_run = NULL;
		struct process *proc_to_run = NULL;
		if(current_process != NULL){
				thread_to_run = find_next_thread();
				if(!thread_to_run){
						goto fail_start_sched;
				}
				
				proc_to_run = thread_to_run->parent_proc;



				switch(proc_to_run->process_type){
						case KERNEL_PROCESS:
								current_process = proc_to_run;
								current_thread = thread_to_run;
								add_thread_to_rq(thread_to_run,get_rq());
//								list_del(&thread_to_run->rdy_list);
//								list_add_tail(&thread_to_run->rdy_list, &get_rq()->rdy_list);
								load_context(thread_to_run->stack_top);

								break;

						case JAVA_PROCESS:
								current_process = proc_to_run;
								current_jvm_env_g = proc_to_run->java_exec_env;
								/*TODO :
								 * Do java thread context switching */
								current_thread = thread_to_run;
								add_thread_to_rq(thread_to_run,get_rq());
								CurrentThread = current_thread->java_thread;	/*Set java thread to run */
								current_process->state = PREPARE_INTERPRET;
								load_context(thread_to_run->stack_top);

								break;

				}

				
		}

fail_start_sched:
		kernel_printf("failed to start scheduling. It may have no thread to run\n");
}

#if 0
void do_scheduling(void)
{
	struct task_struct *old = current;

	current = select_task();
	
	if (current == old)
		return;


	DISABLE_IRQ();
	SAVE_SVC_CONTEXT();
	
	__asm__ __volatile__(
		"str	sp, %0\n\t"
		"ldr	sp, %1\n"
		:
		: "m" (old->stack_top), "m" (current->stack_top)
		: "sp"
	);

	
	__asm__ __volatile__(
		"ldr	r0, %0\n\t"
		"add	r1, pc, #28\n\t"
		"str	r1, [r0, #60]\n\t"
		"ldr    r0, [sp, #56]\n\t"
	        "bic    r0, r0, #0x80\n\t"
		"msr    cpsr, r0\n\t"
		"ldmia  sp, {r0 - r12, lr}\n\t"
		"add    sp, sp, #60\n\t"
		"ldr	pc, [sp], #4\n\t"
		"nop\n\t"
		:
		: "m" (old->stack_top)
		: "r0", "r1", "lr", "sp"
	);

	lightchu();
	if (current->rt_signal)
		do_rt_signal();

	if (current->signal)
		do_signal();
}



void do_first_scheduling() 
{
	current = task_list;

	
	__asm__ __volatile__(
		"ldr	sp, %0\n"
		:
		: "m" (current->stack_top)
	);

	RESTORE_SVC_CONTEXT();


	__asm__ __volatile__(
		"ldr	pc, [sp], #4\n" 
	);
}


#endif
