#include <migos_stdio.h>
#include <string.h>

#include <kernel/thread.h>
#include <kernel/mm.h>
#include <kernel/process.h>
#include <kernel/schedule.h>
#include <kernel/error.h>
#include <arch/cpu.h>
#include <types.h>
#include <list.h>



struct thread init_thread;
struct thread *current_thread= &init_thread;




/*************************************************************************** 
 * Function    : initialize_thread
 * Date        : 2008/08/23
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int initialize_jthread(int thread_id, fp_t entry, int priority, 
				struct process *parent, struct thread *new_thread)
{

		new_thread->thread_id = thread_id;
		new_thread->thread_type = JAVA_THREAD;
		new_thread->state = INITIAL;
		new_thread->java_exec_env = NULL;
		new_thread->timeslice = DEFAULT_TIMESLICE;
		new_thread->parent_proc = parent;
		new_thread->priority = MAX_THREAD_PRIO;
		new_thread->rt_fact = NORMAL;

		/*Prepare stack entries of a thread for first scheduling. */
		unsigned int *stack_pointer = ((unsigned int *)&new_thread->stack_bottom) - 1;
		*(--stack_pointer) = (unsigned int)entry;		/*K thread entry point */
		*(--stack_pointer) = 0x0E0E0E0EL;		/*K R14 */
		*(--stack_pointer) = 0x0C0C0C0CL;		/*K R12 */
		*(--stack_pointer) = 0x0B0B0B0BL;		/*K R11 */
		*(--stack_pointer) = 0x0A0A0A0AL;		/*K R10 */
		*(--stack_pointer) = 0x09090909L;		/*K R9 */
		*(--stack_pointer) = 0x08080808L;		/*K R8 */
		*(--stack_pointer) = 0x07070707L;		/*K R7 */
		*(--stack_pointer) = 0x06060606L;		/*K R6 */
		*(--stack_pointer) = 0x05050505L;		/*K R5 */
		*(--stack_pointer) = 0x04040404L;		/*K R4 */
		*(--stack_pointer) = 0x03030303L;		/*K R3 */
		*(--stack_pointer) = 0x02020202L;		/*K R2 */
		*(--stack_pointer) = 0x01010101L;		/*K R1 */
		*(--stack_pointer) = 0x00000000L;		/*K R0 */
		*(--stack_pointer) = 0x00000013L;		/*K CPSR : Enable IRQ, FIQ  && ARM Mode*/

		/*Store a top of stack */
		new_thread->stack_top = (unsigned int) stack_pointer;


		INIT_LIST_HEAD(&new_thread->thread_list);
		INIT_LIST_HEAD(&new_thread->rdy_list);
		INIT_LIST_HEAD(&new_thread->wait_list);
		list_add_tail(&new_thread->thread_list, &parent->thread_list);
		parent->alive_thread_count++;
		
		
		/*TODO:
		 * set priority for scheduling*/

		return SUCCESS;

}

int initialize_kthread(int thread_id, fp_t entry, int priority, struct thread *new_thread, emergency rt_fact)
{
		void thread_exit(void);

		new_thread->thread_id = thread_id;
		new_thread->thread_type = KERNEL_THREAD;
		new_thread->state = INITIAL;
		new_thread->java_exec_env = NULL;
		new_thread->timeslice = DEFAULT_TIMESLICE;
		new_thread->parent_proc = get_current_process();
		new_thread->rt_fact = rt_fact;
		if(rt_fact == NORMAL){
				new_thread->priority = MAX_THREAD_PRIO;

		}else {
				new_thread->priority = priority;
		}

		/*Prepare stack entries of a thread for first scheduling. */
		unsigned int *stack_pointer = ((unsigned int *)&new_thread->stack_bottom) - 1;
		*(--stack_pointer) = (unsigned int)entry;		/*K thread entry point */
//		*(--stack_pointer) = 0x0E0E0E0EL;		/*K R14 */
		*(--stack_pointer) = thread_exit;		/*K R14 */
		*(--stack_pointer) = 0x0C0C0C0CL;		/*K R12 */
		*(--stack_pointer) = 0x0B0B0B0BL;		/*K R11 */
		*(--stack_pointer) = 0x0A0A0A0AL;		/*K R10 */
		*(--stack_pointer) = 0x09090909L;		/*K R9 */
		*(--stack_pointer) = 0x08080808L;		/*K R8 */
		*(--stack_pointer) = 0x07070707L;		/*K R7 */
		*(--stack_pointer) = 0x06060606L;		/*K R6 */
		*(--stack_pointer) = 0x05050505L;		/*K R5 */
		*(--stack_pointer) = 0x04040404L;		/*K R4 */
		*(--stack_pointer) = 0x03030303L;		/*K R3 */
		*(--stack_pointer) = 0x02020202L;		/*K R2 */
		*(--stack_pointer) = 0x01010101L;		/*K R1 */
		*(--stack_pointer) = 0x00000000L;		/*K R0 */
		*(--stack_pointer) = 0x00000013L;		/*K CPSR : Enable IRQ, FIQ  && ARM Mode*/

		/*Store a top of stack */
		new_thread->stack_top = (unsigned int) stack_pointer;


		INIT_LIST_HEAD(&new_thread->thread_list);
		INIT_LIST_HEAD(&new_thread->rdy_list);
		INIT_LIST_HEAD(&new_thread->wait_list);
		list_add_tail(&new_thread->thread_list, &get_current_process()->thread_list);
		

		/*TODO:
		 * set priority for scheduling*/

		return SUCCESS;

}



/*************************************************************************** 
 * Function    : add_thread_to_queue
 * Date        : 2008/08/23
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : Add a thread to scheduling queue. It should be schduling policy
 * 				 specific.
 * ***************************************************************************/
int add_thread_to_waitq(struct thread *new_thread, struct thread *queue)
{
		int ret = SUCCESS;
		int cpu_sr;
		if( new_thread == NULL && queue == NULL){
				ret = FAILED;
				goto invalid_param;
		}

		enter_critical_section();
		list_del_init(&new_thread->wait_list);
		list_add_tail(&new_thread->wait_list,&queue->wait_list);
		exit_critical_section();

invalid_param:

		errno = -EINVARG;
		return ret;

}



/*TODO : 
 * remove the second parameter queue.*/
int add_thread_to_rq(struct thread *new_thread, struct thread *queue)
{
		int ret = SUCCESS;
		int cpu_sr;
		if( new_thread == NULL && queue == NULL){
				ret = FAILED;
				goto invalid_param;
		}

		enter_critical_section();
		if(new_thread->rt_fact == REALTIME){
				list_del_init(&new_thread->rdy_list);
				list_add_tail(&new_thread->rdy_list,&get_rt_rq()->rdy_list);

		}else {
				list_del_init(&new_thread->rdy_list);
				list_add_tail(&new_thread->rdy_list,&get_rq()->rdy_list);
		}
		exit_critical_section();

invalid_param:

		errno = -EINVARG;
		return ret;

}





/*************************************************************************** 
 * Function    : create_kernel_thread
 * Date        : 2008/08/23
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 커널 쓰레드를 생성한다. 
 ****************************************************************************/

struct thread *create_kernel_thread(int thread_id, char *name, fp_t entry, emergency rt_fact, int priority)
{
		struct thread *new_thread = (struct thread *)k_malloc(sizeof(struct thread));

		if(new_thread == NULL){
				errno = -ENOMEM;
				goto out_of_memory;
		}

		strcpy(new_thread->name, name);
		initialize_kthread(thread_id, entry, priority, new_thread,rt_fact);
		add_thread_to_rq(new_thread, get_rq());
		get_current_process()->alive_thread_count++;
		
		kernel_printf("Create thread completed!\n");
		
		return new_thread;

out_of_memory:
		return NULL;

}








/*************************************************************************** 
 * Function    : setup_swapper_thread
 * Date        : 2008/08/23
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int setup_swapper_thread(void)
{
		kernel_printf("Initializing a swapper thread........ \n");

		init_thread.thread_id= 0;
		strcpy(init_thread.name,"swapper thread");
		init_thread.thread_type = KERNEL_THREAD;
		init_thread.state = INITIAL;
		init_thread.java_exec_env = NULL;
		init_thread.timeslice = DEFAULT_TIMESLICE;
		init_thread.parent_proc = get_current_process();
		init_thread.priority = MAX_THREAD_PRIO;
		current_thread = &init_thread;
		INIT_LIST_HEAD(&init_thread.thread_list);
		INIT_LIST_HEAD(&init_thread.rdy_list);
		INIT_LIST_HEAD(&init_thread.mem_header.memLink);
		INIT_LIST_HEAD(&init_thread.mem_header.freeMemLink);
		INIT_LIST_HEAD(&init_thread.mem_header.allocLink);



		//list_add_tail(&init_thread.thread_list, &get_current_process()->thread_list);
		

		return SUCCESS;

}



/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

void wait_thread(struct thread *block)
{
		int cpu_sr;

		if(block){
				enter_critical_section();
				block->state = BLOCK;
				list_del_init(&block->rdy_list);
				kernel_printf("waitting!!!\n");
				add_thread_to_waitq(block,get_waitq());
				if(block == get_current_thread()){
						yield_thread();
				}

				exit_critical_section();
		}
}




void __sleep_thread(struct thread *block)
{
		int cpu_sr;
		block->state = SLEEP;

		enter_critical_section();

		list_del_init(&block->rdy_list);
		add_thread_to_waitq(block,get_waitq());
		if(block == get_current_thread()){
				yield_thread();
		}

		exit_critical_section();
}



/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

void wake_up_thread(struct thread *wake)
{
		if(wake->state == BLOCK || wake->state == SLEEP){
				int cpu_sr;
				enter_critical_section();
				list_del_init(&wake->wait_list);
				add_thread_to_rq(wake,get_rq());
				wake->state = RUNNING;
				exit_critical_section();
		}
}



extern void __yield_thread(void);	/* Assembly routine */



void yield_thread(void)
{
	__yield_thread();	

}


struct thread *find_thread_by_id(int id)
{
		struct thread *iter = NULL;
		int found = 0;
		int cpu_sr;

		enter_critical_section();

		list_for_each_entry(iter, &get_rq()->rdy_list,rdy_list){
				if(iter->thread_id == id){
						found = 1;
						goto thread_found;
				}

		}list_end_for;
		
		list_for_each_entry(iter, &get_waitq()->wait_list,wait_list){
				if(iter->thread_id == id){
						found = 1;
						goto thread_found;
				}

		}list_end_for;

thread_found:
		exit_critical_section();

		if(found)
				return iter;
		else
				return NULL;
}

void thread_exit(void)
{
		struct thread *del_thread = get_current_thread();
		int cpu_sr;
		del_thread->state = HALT;


		enter_critical_section();

		list_del_init(&del_thread->rdy_list);
		list_del_init(&del_thread->wait_list);
		list_del_init(&del_thread->thread_list);

		yield_thread();
		exit_critical_section();
		while(1){};


}


int kill(int id)
{
		int ret;
		struct thread *t = find_thread_by_id(id);
		if(t){
				int cpu_sr;
				t->state = HALT;

				enter_critical_section();

				list_del_init(&t->rdy_list);
				list_del_init(&t->wait_list);
				list_del_init(&t->thread_list);

				if(t == get_current_thread()){
						yield_thread();
				}
				exit_critical_section();


		}
}


/*************************************************************************** 
 * Function    : create_java_thread
 * Date        : 2008/09/26
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 커널 쓰레드를 생성한다. 
 ****************************************************************************/
#if 0
struct thread *create_java_thread(int thread_id, char *name, fp_t entry, int priority)
{
		struct thread *new_thread = (struct thread *)k_malloc(sizeof(struct thread));

		if(new_thread == NULL){
				errno = -ENOMEM;
				goto out_of_memory;
		}



		strcpy(new_thread->name, name);
		initialize_thread(thread_id, entry, priority, new_thread);
		add_thread_to_queue(new_thread, get_rq());
		
		return new_thread;

out_of_memory:
		return NULL;

}
#endif
