#include<global.h>
#include<migos_stdio.h>
#include<kernel/timer.h>
#include<kernel/thread.h>
#include<kernel/schedule.h>
#include<kernel/mm.h>
#include<kernel/error.h>
#include<arch/cpu.h>
#include<list.h>
#include<types.h>

struct timer timer_queue;

int init_timer(void)
{
		kernel_printf("Initializing soft timer........ \n");

		INIT_LIST_HEAD(&timer_queue.timer_list);
		timer_queue.timeout = 0;
		timer_queue.current = NULL;
		timer_queue.fn = (void (*)(struct thread *))NULL;

#if 0
		int temp = 0;

		__asm {
				mrs r0, cpsr;
				bic	r0, r0, #0xc0;
				msr	cpsr_cxsf, r0;
				mrs temp, cpsr;
		}


		printf("cpsr: %x \n", temp);

#endif

		return SUCCESS;

}


int request_timer(int timeout, void (*fn)(struct thread *))
{
		struct process *saved = NULL;
		if(fn == NULL){
				return -EINVARG;
		}

		struct timer *new_timer = (struct timer *)k_malloc(sizeof(struct timer));

		new_timer->timeout = timeout;
		new_timer->current = get_current_thread();
		INIT_LIST_HEAD(&new_timer->timer_list);

		if(get_current_thread()->thread_type == JAVA_THREAD){
				new_timer->type = JAVA_TIMER;
				
		}else if(get_current_thread()->thread_type == KERNEL_THREAD){
				new_timer->type = KERNEL_TIMER;

		}
		
		new_timer->fn = fn;
		list_add_tail(&new_timer->timer_list,&timer_queue.timer_list);

		return SUCCESS;
}


int request_timer_thread(struct thread *this_thread,int timeout, void (*fn)(struct thread *))
{
		if(fn == NULL){
				return -EINVARG;
		}
		struct timer *new_timer = (struct timer *)k_malloc(sizeof(struct timer));

		new_timer->timeout = timeout;
		new_timer->current = this_thread;
		INIT_LIST_HEAD(&new_timer->timer_list);

		if(this_thread->thread_type == JAVA_THREAD){
				new_timer->type = JAVA_TIMER;
				
		}else if(this_thread->thread_type == KERNEL_THREAD){
				new_timer->type = KERNEL_TIMER;

		}
		
		new_timer->fn = fn;
		list_add_tail(&new_timer->timer_list,&timer_queue.timer_list);

		return SUCCESS;
}

int cancel_timer(struct timer *del_timer)
{
		int ret = 0;
		if(del_timer== NULL){
				errno = -EINVARG;
				ret = errno;
				goto fail;
		}

		list_del(&del_timer->timer_list);
		k_free(del_timer);
		ret = SUCCESS;

fail:
		return ret;
}


void remove_reserved_alarm(struct thread *this_thread)
{
		struct timer *iter = NULL;

		list_for_each_entry(iter, &timer_queue.timer_list, timer_list){
				if(iter->current == this_thread){
						cancel_timer(iter);
						return ;
				}

		}list_end_for;
}



void update_timer(void)
{
		struct timer *iter = NULL;
		struct process *saved = NULL;

		list_for_each_entry(iter, &timer_queue.timer_list, timer_list){
				iter->timeout--;
				if(iter->timeout == 0){
						if(iter->type == JAVA_TIMER){

								saved = get_current_process();
								current_process = iter->current->parent_proc;

								if(iter->fn){
										iter->fn(iter->current->java_thread);
								}
								cancel_timer(iter);

								current_process = saved;

						}else if(iter->type == KERNEL_TIMER){

								saved = get_current_process();

								current_process = iter->current->parent_proc;
								if(iter->fn){
										iter->fn(iter->current);
								}
								cancel_timer(iter);

								current_process = saved;
						}

				}

		}list_end_for;
}


void sleep(int sec)
{
		request_timer(sec * 1000, wake_up_thread);
		__sleep_thread(get_current_thread());

}


int sleep_thread(struct thread *block, int time)
{
		int cpu_sr;
		if(!block){
				return FAILED;

		}
		enter_critical_section();

		request_timer_thread(block, time * 1000, wake_up_thread);
		block->state = SLEEP;
		list_del_init(&block->rdy_list);
		add_thread_to_waitq(block,get_waitq());
		if(block == get_current_thread()){
				yield_thread();
		}

		exit_critical_section();
}


void usleep(int usec)
{
		request_timer(usec,wake_up_thread);
		wait_thread(get_current_thread());
}



void delay(int time)
{
	volatile int i;

	for (i=0; i<time; i++){}
}



void udelay(int time)
{
		volatile int i, j;
		for(i = 0; i < (1012); i++){
				for(j = 0; j < (65); j++){
						;
				}
		}

}



void print_timer(void)
{
		struct timer *iter = NULL;

		list_for_each_entry(iter, &timer_queue.timer_list, timer_list){
			printf("timer list : %s \n",	iter->current->name);

		}list_end_for;
}



int is_timer_reserved(struct thread *this_thread)
{
		struct timer *iter = NULL;

		list_for_each_entry(iter, &timer_queue.timer_list, timer_list){
				if(iter == this_thread)
						return 1;

		}list_end_for;


		return 0;
}
