#include <migos_stdio.h>
#include <types.h>
#include <list.h>
#include <kernel/resource.h>
#include <kernel/vfs.h>
#include <kernel/process.h>
#include <kernel/mm.h>
#include <kernel/error.h>


#ifndef EZ_X5
#include <string.h>
#endif //EZ_X5




struct vfs_object **vfile_table = NULL;
struct vfs_object vfs_object_head_g;


int setup_vfs_object(struct vfs_object *new_vfs_object, struct resource *resource_to_add);





/*************************************************************************** 
 * Function    : init_vfs
 * Date        : 2008/08/15
 * Author      : Sun Joon Gyu
 * Parameter   : None
 * Return value: Success / Error number
 * Description : Initialize Virtual File System
 * ***************************************************************************/
int init_vfs(void)
{
		int i;
		
		kernel_printf("Initializing virtual file system.........\n");
		vfile_table = (struct vfs_object **)k_malloc(sizeof(struct vfs_object *) * MAX_VFS_OBJECT_NUM);


		for(i = 0; i < MAX_VFS_OBJECT_NUM ; i++){
				vfile_table[i] = NULL;
		}

		if(vfile_table==NULL)	{
				return -ENOMEM;
		}

		INIT_LIST_HEAD(&vfs_object_head_g.object_list);

		return SUCCESS;
}




/* Name : vfs_register_device
 * Parameter : 
 *		- request_id : ̽ ȣ
 *      - 
 * Return :
 *		- Ҵ vfs_object ȯѴ.
 * Description :
 *      - VFS ο ̽ Ѵ. 
 */

int vfs_register_device(int request_id,struct resource *resource_to_add)
{
		struct vfs_object *new_vfs_object = NULL;

		if(vfile_table[request_id] != NULL)
		{
				errno = -EEXIST;
				goto reg_fail1;
		}

		/* Do this to reserve the requesting id number */
		vfile_table[request_id] = (struct vfs_object *)1; 


		kernel_printf("1: \n");
		new_vfs_object = (struct vfs_object *)k_malloc(sizeof(struct vfs_object));
		if(!new_vfs_object)
		{	
				errno = -ENOMEM;
				goto reg_fail2;

		}

		kernel_printf("new_vfs_object addr %x: \n", new_vfs_object);
		kernel_printf("resource_to_add->private addr : %x \n", resource_to_add->private);

		new_vfs_object->object_id = request_id;
		if(setup_vfs_object(new_vfs_object,resource_to_add) != SUCCESS){
				/*TODO : Handle Errors... */

		}
		vfile_table[request_id] = new_vfs_object;
		kernel_printf("3: \n");

		/* TODO:
		 * Initialize semaphore for a new vfs_object */

		new_vfs_object->initialize();
		kernel_printf("4: \n");

		return SUCCESS;
		


reg_fail2:
reg_fail1:	

		return FAILED;
}


/*************************************************************************** 
 * Function    : setup_vfs_object
 * Date        : 2008/08/15
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * 				new_vfs_object
 * 				resource_to_add
 * Return value:
 * Description : 
 * ***************************************************************************/

int setup_vfs_object(struct vfs_object *new_vfs_object, struct resource *resource_to_add)
{

		kernel_printf("2-1: \n");
		if(resource_to_add == NULL || new_vfs_object == NULL){
				errno = -EINVARG;
				// goto ...;
		}

		kernel_printf("2-2: \n");
		new_vfs_object->data = (void *)resource_to_add;
		new_vfs_object->owner = NULL;

		if(resource_to_add->type & (TYPE_FS | TYPE_DEVICE)){
		kernel_printf("2-3: \n");
				if(resource_to_add->private == NULL){
						errno = -EINVARG;
						// goto ...;
				}

				struct file_op *operator = (struct file_op*) resource_to_add->private;
		kernel_printf("2-3: \n");
				new_vfs_object->operator = operator;
		kernel_printf("2-3: \n");
		kernel_printf("in setup_vfs_object - new_vfs_object address: %x \n", new_vfs_object);
		kernel_printf("in setup_vfs_object address: %x \n", operator);
				new_vfs_object->initialize = (fp_t) operator->init;
		kernel_printf("2-3: \n");
				new_vfs_object->name = resource_to_add->name;
		kernel_printf("2-3: \n");
				INIT_LIST_HEAD(&new_vfs_object->object_list);
		kernel_printf("2-3: \n");

		}else {
		kernel_printf("2-4: \n");
				/* Not supported device type */
				;
		}
		kernel_printf("2-5: \n");

		list_add(&new_vfs_object->object_list, &vfs_object_head_g.object_list);

		kernel_printf("2-6: \n");


		return errno;
};




/*************************************************************************** 
 * Function    : get_vfs_object
 * Date        : 2008/08/15
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

struct vfs_object *get_vfs_object(int type, const char *name)
{

		struct vfs_object *iter = NULL;
		struct vfs_object *found = NULL;

		if(type &  TYPE_DEVICE){

				list_for_each_entry(iter, &(vfs_object_head_g.object_list), object_list){
						if(!strcmp(iter->name,name)){
								found = iter;
								goto found;
						}

				}list_end_for;

				goto not_found;

		} else if(type & TYPE_FS){
				found = vfile_table[TYPE_FS];
				goto found;
		} else {

				goto not_found;
		}

found:
		return found;

not_found:
		return NULL;

}


/*************************************************************************** 
 * Function    : open
 * Date        : 2008/08/15
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/
int open(int type, char *name, char mode)
{

#if 1
		int new_fd;
		struct file *new_file = NULL;


		struct vfs_object *object = get_vfs_object(type,name);
		new_file = (struct file *)k_malloc(sizeof(struct file));
		if(new_file == NULL)
				goto fail2;

		switch(type)
		{
				case TYPE_FS : 
						{
								struct process *curr = get_current_process();
								new_file->operator = object->operator;
								if(new_file->operator->open(new_file,name,mode) != SUCCESS){
										goto open_fail;
								}

								new_fd = ++curr->fd_count;
								new_file->fd = new_fd;
								new_file->state = IN_USE;
								new_file->offset = 0;
								INIT_LIST_HEAD(&new_file->file_list);
							  /*TODO:
								new_file->private = find_file_by_name(name);
								if(!fat_file){
								goto fail1;
								}
								*/
								list_add(&new_file->file_list, &curr->file_table.file_list);

						}
						break;
				case TYPE_DEVICE :
						{
								struct process *curr = get_current_process();
								new_file->operator = object->operator;
								if(new_file->operator->open(new_file,name,mode) != SUCCESS){
										goto open_fail;
								}

								new_fd = ++curr->fd_count;
								new_file->fd = new_fd;
								new_file->state = IN_USE;
								new_file->offset = 0;
								INIT_LIST_HEAD(&new_file->file_list);
							  /*TODO:
								*/
								list_add(&new_file->file_list, &curr->file_table.file_list);

						}
						break;
				default : 
						;
		}




		/* Open succeed */
		return new_fd;

fail2:
open_fail: k_free(new_file);
		return -ENOMEM;
#endif


}


/*************************************************************************** 
 * Function    : get_file
 * Date        : 2008/08/15
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

struct file *get_file(int fd)
{

		struct file *iter = NULL;
		struct process *curr = get_current_process();
		struct file *ret_file = NULL;

		list_for_each_entry(iter, &(curr->file_table.file_list), file_list){
				if(fd == iter->fd){
						ret_file = iter;
						goto found;
				}

		}list_end_for;

		/* not_found */
		return NULL;

found:
		return ret_file; 
}

/*************************************************************************** 
 * Function    : get_file_op
 * Date        : 2008/08/15
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

inline struct file_op *get_file_op(struct file *this_file)
{
		if(this_file != NULL){
				if(this_file->operator == NULL){
						errno = -ENOVALID;
				}
				
				return this_file->operator;
		}else {
				return NULL;
		}


}



/*************************************************************************** 
 * Function    : read
 * Date        : 2008/08/15
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int read(int fd, char *buffer, int size)
{
#if 1
		lock_t sem;
		int ret=0;
		int n = 0;

#ifdef SEMAPHORE
		aquire_sem(&sem);
#endif
		if(!sem)
		{
				errno = -EBUSY;
				goto read_sem_fail;
		}

		if(sem)	{

				struct file *this_file = get_file(fd);
				if(this_file == NULL){
						errno = -ENOFILE;
						goto get_file_fail;
				}

				struct file_op *this_file_op = get_file_op(this_file);
				if(this_file_op == NULL){
						errno = -ENOVALID;
						goto get_file_fail;
				}



				n = this_file_op->read(this_file, buffer, size);
				this_file->offset += n;
				ret = n;
#ifdef SEMAPHORE		
				release_sem(&sem);
#endif // SEMAPHORE

		}

		/*read_success */
		return ret;

get_file_fail:
read_sem_fail:
		return NULL;
#endif
}






/*************************************************************************** 
 * Function    : write
 * Date        : 2008/08/15
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int write(int fd, char *buffer, int size)
{
#if 1
		lock_t sem;
		int ret=0;
		int n = 0;

#ifdef SEMAPHORE
		aquire_sem(&sem);
#endif
		if(!sem)
		{
				errno = -EBUSY;
				goto write_sem_fail;
		}

		if(sem)	{

				struct file *this_file = get_file(fd);
				if(this_file == NULL){
						errno = -ENOFILE;
						goto get_file_fail;
				}

				struct file_op *this_file_op = get_file_op(this_file);
				if(this_file_op == NULL){
						errno = -ENOVALID;
						goto get_file_fail;
				}



				n = this_file_op->write(this_file, buffer, size);
				this_file->offset += n;
				ret = n;
#ifdef SEMAPHORE		
				release_sem(&sem);
#endif // SEMAPHORE

		}

		/*write_success */
		return ret;

get_file_fail:
write_sem_fail:
		return NULL;
#endif
}





/*************************************************************************** 
 * Function    : lseek
 * Date        : 2008/08/15
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int lseek(int fd, int pos, int offset)
{
#if 1
		lock_t sem;
		int ret=0;

#ifdef SEMAPHORE
		aquire_sem(&sem);
#endif
		if(!sem)
		{
				errno = -EBUSY;
				goto lseek_sem_fail;
		}

		if(sem)	{

				struct file *this_file = get_file(fd);
				if(this_file == NULL){
						errno = -ENOFILE;
						goto get_file_fail;
				}

				struct file_op *this_file_op = get_file_op(this_file);
				if(this_file_op == NULL){
						errno = -ENOVALID;
						goto get_file_fail;
				}


				if(this_file_op->lseek != NULL){
						ret = this_file_op->lseek(this_file, pos, offset);
				}else {

						switch(pos){
								case SET_POS:
										{
												if(offset < 0 && offset >= this_file->size){
														errno = -EINVARG;
														ret = errno;
														goto invalid_arg;
												}

												this_file->offset = offset;
												break;
										}
								case CUR_POS:
										{
												if(this_file->offset + offset < 0 
														&& this_file->offset +offset >= this_file->size){
														errno = -EINVARG;
														ret = errno;
														goto invalid_arg;
												}
												this_file->offset = this_file->offset + offset;
												break;
										}
								case END_POS:
										{
												if(offset > 0){
														this_file->offset = this_file->size;
												}else if(offset + this_file->size < 0){
														this_file->offset = 0;
												}else {
														this_file->offset = this_file->size + offset;
												}
												break;
										}
								default:
										{
												errno = -EINVARG;
												goto invalid_arg;
										}

						}
				}

#ifdef SEMAPHORE		
				release_sem(&sem);
#endif // SEMAPHORE

		}

		/* seek_success */
		return ret;

invalid_arg:
get_file_fail:
lseek_sem_fail:
		return errno;
#endif
}






/*************************************************************************** 
 * Function    : close
 * Date        : 2008/08/15
 * Author      : Sun Joon Gyu
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/
int close(int fd)
{
#if 1
		lock_t sem;
		int ret=0;

#ifdef SEMAPHORE
		aquire_sem(&sem);
#endif
		if(!sem)
		{
				errno = -EBUSY;
				goto close_sem_fail;
		}

		if(sem)	{

				struct file *this_file = get_file(fd);
				if(this_file == NULL){
						errno = -ENOFILE;
						goto get_file_fail;
				}

				struct file_op *this_file_op = get_file_op(this_file);
				if(this_file_op == NULL){
						errno = -ENOVALID;
						goto get_file_fail;
				}



				ret = this_file_op->close(this_file->fd);
				this_file->use_count--;
				if(this_file->use_count == 0){
						k_free(this_file);
				}



#ifdef SEMAPHORE		
				release_sem(&sem);
#endif // SEMAPHORE

		}

		/* close_success */
		return ret;

get_file_fail:
close_sem_fail:
		return errno;
#endif


}
