/*
 * Copyright (c) 2001, Adam Dunkels.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.  
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  
 *
 * This file is part of the uIP TCP/IP stack.
 *
 * $Id: uip_arch.c,v 1.2 2004/11/09 02:56:43 palangse Exp $
 *
 */


#include "uip.h"
#include "uip_arch.h"

#define BUF ((uip_tcpip_hdr *)&uip_buf[UIP_LLH_LEN])
#define IP_PROTO_TCP    6

/*-----------------------------------------------------------------------------------*/
void
uip_add32(u8_t *op32, u16_t op16)
{
  
  uip_acc32[3] = op32[3] + (op16 & 0xff);
  uip_acc32[2] = op32[2] + (op16 >> 8);
  uip_acc32[1] = op32[1];
  uip_acc32[0] = op32[0];
  
  if(uip_acc32[2] < (op16 >> 8)) {
    ++uip_acc32[1];    
    if(uip_acc32[1] == 0) {
      ++uip_acc32[0];
    }
  }
  
  
  if(uip_acc32[3] < (op16 & 0xff)) {
    ++uip_acc32[2];  
    if(uip_acc32[2] == 0) {
      ++uip_acc32[1];    
      if(uip_acc32[1] == 0) {
	++uip_acc32[0];
      }
    }
  }
}
/*-----------------------------------------------------------------------------------*/
#define YH_DEBUG1	0
#define YH_DEBUG3	0

#if 0
u16_t
uip_chksum(u16_t *sdata, u16_t len)
{
  u16_t acc;
  u8_t* my_pt;
  unsigned int i;


#if 1
  my_pt = uip_appdata;
printf("DEBUG : %s(", my_pt);
for (i=0;i<strlen(my_pt);i++)
{
	printf("%x ", *my_pt++);
}
printf(")\n");
#endif

#if YH_DEBUG3
  my_pt = (u8_t*) sdata;

  if((*sdata & 0x00ff) == 1 ) {
	  printf("DEBUG: ERROR!\n");
	  sdata += 8;
  } else {
	  printf("DEBUG : *sdata = 0x%x, 0x%x\n", *sdata, *sdata & 0x00ff);
  }
#endif
  
  for(acc = 0; len > 1; len -= 2) {
#if YH_DEBUG1
    printf("0x%x + 0x%x = ", acc, *sdata);
#endif
    acc += *sdata;
#if YH_DEBUG1
    printf("0x%x\n", acc);
#endif
    if(acc < *sdata) {
      /* Overflow, so we add the carry to acc (i.e., increase by
         one). */
#if YH_DEBUG1 
    printf("0x%x++ = ", acc);
#endif
      ++acc;
#if YH_DEBUG1
    printf("0x%x\n", acc);
#endif
    }
    ++sdata;
  }

  /* add up any odd byte */
  if(len == 1) {
#if YH_DEBUG1
    printf("0x%x + 0x%x = ", acc, htons(((u16_t)(*(u8_t *)sdata)) << 8));
#endif
    acc += htons(((u16_t)(*(u8_t *)sdata)) << 8);
#if YH_DEBUG1
    printf("0x%x\n", acc);
#endif
    if(acc < htons(((u16_t)(*(u8_t *)sdata)) << 8)) {
#if YH_DEBUG1
    printf("0x%x++ = ", acc);
#endif
      ++acc;
#if YH_DEBUG1
    printf("0x%x\n", acc);
#endif
    }
  }

  return acc;
}
#endif

#if 1
u16_t
uip_chksum(u16_t *sdata, u16_t len)
{
  u16_t acc, tmp, val;
  u8_t* my_pt;

  my_pt = (u8_t*) sdata;
//printf("DEBUG : my_pt = 0x%x , sdata = 0x%x\n", *my_pt, *sdata);

  for(acc = 0; len > 1; len -= 2) {
    tmp = acc;
    val = (*my_pt++);
    val |= (*my_pt++)<<8;
    acc += val;
#if YH_DEBUG1
    printf("0x%x + 0x%x = 0x%x\n", tmp, val, acc);
#endif
    if(acc < tmp) {
      /* Overflow, so we add the carry to acc (i.e., increase by
         one). */
      ++acc;
    }
//    ++my_pt;
  }

  /* add up any odd byte */
  if(len == 1) {
    tmp = acc;
//    val = (*my_pt)<<8;
    val = (*my_pt);
    acc += val;
#if YH_DEBUG1
    printf("0x%x + 0x%x = 0x%x\n", tmp, val, acc);
#endif
    if(acc < tmp) {
      ++acc;
    }
  }

  return acc;
}
#endif

/*-----------------------------------------------------------------------------------*/
u16_t
uip_ipchksum(void)
{
  return uip_chksum((u16_t *)&uip_buf[UIP_LLH_LEN], 20);
}
/*-----------------------------------------------------------------------------------*/
#define YH_DEBUG	0
#define YH_DEBUG2	0


u16_t
uip_tcpchksum(void)
{
  u16_t hsum, sum;
  u8_t* my_pt;
  unsigned int i;

#if 0
  my_pt = uip_appdata;
printf("DEBUG(%s,%d) uip_appdata = 0x%x, 0x%x %x\n", __FUNCTION__, __LINE__, *(u16_t *)uip_appdata, *my_pt, *(++my_pt));
#endif
#if YH_DEBUG2
  my_pt = uip_appdata;
printf("DEBUG : %s(", my_pt);
for (i=0;i<strlen(my_pt);i++)
{
	printf("%x ", *my_pt++);
}
printf(")\n");
  
  /* Compute the checksum of the TCP header. */
printf("== checksum of the TCP header ==\n");
#endif
  hsum = uip_chksum((u16_t *)&uip_buf[20 + UIP_LLH_LEN], 20);

  /* Compute the checksum of the data in the TCP packet and add it to
     the TCP header checksum. */
#if YH_DEBUG2
printf("== checksum of the data in the TCP packet ==\n");
#endif
  sum = uip_chksum((u16_t *)uip_appdata,
		   (u16_t)(((((u16_t)(BUF->len[0]) << 8) + BUF->len[1]) - 40)));

#if YH_DEBUG
printf("== sum += hsum ==\n");
printf("0x%x + 0x%x = ", sum, hsum);
#endif
  if((sum += hsum) < hsum) {
#if YH_DEBUG
printf("++sum = ");
#endif
    ++sum;
  }
#if YH_DEBUG
printf("0x%x\n", sum);
  
printf("0x%x + 0x%x = ", sum, BUF->srcipaddr[0]);
#endif
  if((sum += BUF->srcipaddr[0]) < BUF->srcipaddr[0]) {
#if YH_DEBUG
printf("++sum = ");
#endif
    ++sum;
  }
#if YH_DEBUG
printf("0x%x\n", sum);
printf("0x%x + 0x%x = ", sum, BUF->srcipaddr[1]);
#endif
  if((sum += BUF->srcipaddr[1]) < BUF->srcipaddr[1]) {
#if YH_DEBUG
printf("++sum = ");
#endif
    ++sum;
  }
#if YH_DEBUG
printf("0x%x\n", sum);
printf("0x%x + 0x%x = ", sum, BUF->destipaddr[0]);
#endif
  if((sum += BUF->destipaddr[0]) < BUF->destipaddr[0]) {
#if YH_DEBUG
printf("++sum = ");
#endif
    ++sum;
  }
#if YH_DEBUG
printf("0x%x\n", sum);
printf("0x%x + 0x%x = ", sum, BUF->destipaddr[1]);
#endif
  if((sum += BUF->destipaddr[1]) < BUF->destipaddr[1]) {
#if YH_DEBUG
printf("++sum = ");
#endif
    ++sum;
  }
#if YH_DEBUG
printf("0x%x\n", sum);
printf("0x%x + 0x%x = ", sum, (u16_t)htons((u16_t)IP_PROTO_TCP));
#endif
  if((sum += (u16_t)htons((u16_t)IP_PROTO_TCP)) < (u16_t)htons((u16_t)IP_PROTO_TCP)) {
#if YH_DEBUG
printf("++sum = ");
#endif
    ++sum;
  }
#if YH_DEBUG
printf("0x%x\n", sum);
#endif

  hsum = (u16_t)htons(( ((u16_t)(BUF->len[0]) << 8) + (u16_t)BUF->len[1]) - 20 );
#if YH_DEBUG
printf("DEBUG : BUF->len[0]=0x%x, 0x%x\n", BUF->len[0], BUF->len[1]);
#endif
  
#if YH_DEBUG
printf("0x%x + 0x%x = ", sum, hsum);
#endif
  if((sum += hsum) < hsum) {
#if YH_DEBUG
printf("++sum = ");
#endif
    ++sum;
  }
#if YH_DEBUG
printf("0x%x\n", sum);
#endif
  
  return sum;
}
/*-----------------------------------------------------------------------------------*/
